@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2021 TileDB, Inc.
+ * @copyright Copyright (c) 2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -98,12 +98,15 @@ void write_array() {
   uint64_t data_size = sizeof(data);
 
   // Write in subarray [1,2], [1,4]
-  int subarray[] = {1, 2, 1, 4};
+  tiledb_subarray_t* subarray;
+  tiledb_subarray_alloc(ctx, array, &subarray);
+  int subarray_v[] = {1, 2, 1, 4};
+  tiledb_subarray_set_subarray(ctx, subarray, subarray_v);
 
   // Create the query
   tiledb_query_t* query;
   tiledb_query_alloc(ctx, array, TILEDB_WRITE, &query);
-  tiledb_query_set_subarray(ctx, query, subarray);
+  tiledb_query_set_subarray_t(ctx, query, subarray);
   tiledb_query_set_layout(ctx, query, TILEDB_ROW_MAJOR);
   tiledb_query_set_data_buffer(ctx, query, "a", data, &data_size);
 
@@ -114,6 +117,7 @@ void write_array() {
   tiledb_array_close(ctx, array);
 
   // Clean up
+  tiledb_subarray_free(&subarray);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
   tiledb_ctx_free(&ctx);
@@ -224,4 +228,4 @@ int main() {
   get_fragment_info();
 
   return 0;
-}
\ No newline at end of file
+}
