@@ -76,6 +76,21 @@ extern std::mutex catch2_macro_mutex;
 #define REQUIRE_TILEDB_OK(a) \
   { REQUIRE(a == TILEDB_OK); }
 
+// A variant of the CHECK macro for checking a Status object is okay.
+#define CHECK_TILEDB_STATUS_OK(status) \
+  {                                    \
+    if (!status.ok())                  \
+      INFO(status.to_string());        \
+    CHECK(status.ok());                \
+  }
+
+// A variant of the REQUIRE macro for checking a Status objects is okay.
+#define REQUIRE_TILEDB_STATUS_OK(status) \
+  {                                      \
+    if (!status.ok())                    \
+      INFO(status.to_string());          \
+    REQUIRE(status.ok());                \
+  }
 namespace tiledb {
 
 namespace sm {
@@ -284,6 +299,42 @@ void create_array(
     tiledb_layout_t cell_order,
     uint64_t capacity);
 
+/**
+ * Helper method to create an array schema.
+ *
+ * @param ctx TileDB context.
+ * @param array_name The array name.
+ * @param array_type The array type (dense or sparse).
+ * @param dim_names The names of dimensions.
+ * @param dim_types The types of dimensions.
+ * @param dim_domains The domains of dimensions.
+ * @param tile_extents The tile extents of dimensions.
+ * @param attr_names The names of attributes.
+ * @param attr_types The types of attributes.
+ * @param cell_val_num The number of values per cell of attributes.
+ * @param compressors The compressors of attributes.
+ * @param tile_order The tile order.
+ * @param cell_order The cell order.
+ * @param capacity The tile capacity.
+ * @param allows_dups Whether the array allows coordinate duplicates.
+ * not
+ */
+tiledb_array_schema_t* create_array_schema(
+    tiledb_ctx_t* ctx,
+    tiledb_array_type_t array_type,
+    const std::vector<std::string>& dim_names,
+    const std::vector<tiledb_datatype_t>& dim_types,
+    const std::vector<void*>& dim_domains,
+    const std::vector<void*>& tile_extents,
+    const std::vector<std::string>& attr_names,
+    const std::vector<tiledb_datatype_t>& attr_types,
+    const std::vector<uint32_t>& cell_val_num,
+    const std::vector<std::pair<tiledb_filter_type_t, int>>& compressors,
+    tiledb_layout_t tile_order,
+    tiledb_layout_t cell_order,
+    uint64_t capacity,
+    bool allows_dups = false);
+
 /**
  * Helper method that creates a directory.
  *
@@ -708,6 +759,67 @@ std::string get_commit_dir(std::string array_dir);
 template <class T>
 void check_counts(span<T> vals, std::vector<uint64_t> expected);
 
+/**
+ * Helper function that serializes a query from the "client" or "server"
+ * perspective. The flow being mimicked here is (for read queries):
+ *
+ * - Client sets up read query object including buffers.
+ * - Client submits query to a remote array.
+ * - Internal code (not C API) serializes that query and send it via curl.
+ * - Server receives and deserializes the query using the C API.
+ * - Server submits query.
+ * - Server serializes (using C API) the query and sends it back.
+ * - Client receives response and deserializes the query (not C API). This
+ *   copies the query results into the original user buffers.
+ * - Client's blocking call to tiledb_query_submit() now returns.
+ */
+void serialize_query(
+    const Context& ctx,
+    Query& query,
+    std::vector<uint8_t>* serialized,
+    bool clientside);
+
+/**
+ * Helper function that deserializes a query from the "client" or "server"
+ * perspective.
+ */
+void deserialize_query(
+    const Context& ctx,
+    std::vector<uint8_t>& serialized,
+    Query* query,
+    bool clientside);
+
+int serialize_query(
+    tiledb_ctx_t* ctx,
+    tiledb_query_t* query,
+    std::vector<uint8_t>* serialized,
+    bool clientside);
+
+int deserialize_query(
+    tiledb_ctx_t* ctx,
+    std::vector<uint8_t>& serialized,
+    tiledb_query_t* query,
+    bool clientside);
+
+void submit_serialized_query(tiledb_ctx_t* ctx, tiledb_query_t* query);
+
+void submit_serialized_query(const Context& ctx, Query& query);
+
+void finalize_serialized_query(tiledb_ctx_t* ctx, tiledb_query_t* query);
+
+void finalize_serialized_query(const Context& ctx, Query& query);
+
+void submit_and_finalize_serialized_query(
+    tiledb_ctx_t* ctx, tiledb_query_t* query);
+
+void submit_and_finalize_serialized_query(const Context& ctx, Query& query);
+/**
+ * Helper function that allocates buffers on a query object that has been
+ * deserialized on the "server" side.
+ */
+std::vector<void*> allocate_query_buffers(
+    const Context& ctx, const Array& array, Query* query);
+
 }  // End of namespace test
 
 }  // End of namespace tiledb
