@@ -44,7 +44,7 @@
 #include "tiledb/sm/filesystem/posix.h"
 #endif
 
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 #include <iostream>
 
 using namespace tiledb::sm;
@@ -170,7 +170,7 @@ void set_result_tile_dim(
     std::string dim,
     uint64_t dim_idx,
     std::vector<uint64_t> v) {
-  result_tile.init_coord_tile(dim, dim_idx);
+  result_tile.init_coord_tile(dim, false, dim_idx);
 
   uint64_t* data =
       static_cast<uint64_t*>(tdb_malloc(v.size() * sizeof(uint64_t)));
@@ -182,7 +182,7 @@ void set_result_tile_dim(
       Datatype::UINT64, sizeof(uint64_t), 0, data, v.size() * sizeof(uint64_t));
   auto tile_tuple = result_tile.tile_tuple(dim);
   REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = std::move(tile);
+  tile_tuple->fixed_tile() = std::move(tile);
 }
 
 /* ********************************* */
