@@ -31,8 +31,9 @@
  * Tests of backwards compatibility for opening/reading arrays.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "test/src/helpers.h"
+#include "test/src/serialization_wrappers.h"
 #include "tiledb/common/common.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/misc/constants.h"
@@ -132,16 +133,10 @@ TEST_CASE(
     "[backwards-compat]") {
   Context ctx;
   std::string array_uri(arrays_dir + "/dense_array_v1_3_0");
-  try {
-    Array array(ctx, array_uri, TILEDB_READ);
-    REQUIRE(false);
-  } catch (const TileDBError& e) {
-    // Check correct exception type and error message.
-    std::string msg(e.what());
-    REQUIRE(msg.find("Error: Read buffer overflow") != std::string::npos);
-  } catch (const std::exception& e) {
-    REQUIRE(false);
-  }
+  REQUIRE_THROWS_WITH(
+      Array(ctx, array_uri, TILEDB_READ),
+      "[TileDB::Array] Error: [TileDB::StorageManager] Error: Reading data "
+      "past end of serialized data size.");
 }
 
 template <typename T>
@@ -1273,12 +1268,48 @@ TEST_CASE(
   query_write.set_data_buffer("d1", d1_write);
   query_write.set_data_buffer("d2", d2_write);
 
-  query_write.submit();
-  query_write.finalize();
+  bool serialized_writes = false;
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+  if (!serialized_writes) {
+    query_write.submit();
+    query_write.finalize();
+  } else {
+    submit_and_finalize_serialized_query(ctx, query_write);
+  }
+
   array_write.close();
 
   FragmentInfo fragment_info(ctx, array_name);
   fragment_info.load();
+
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  if (serialized_load) {
+    FragmentInfo deserialized_fragment_info(ctx, array_name);
+    tiledb_fragment_info_serialize(
+        ctx.ptr().get(),
+        array_name.c_str(),
+        fragment_info.ptr().get(),
+        deserialized_fragment_info.ptr().get(),
+        tiledb_serialization_type_t(0));
+    fragment_info = deserialized_fragment_info;
+  }
+
   fragment_uri = fragment_info.fragment_uri(1);
 
   // old version fragment
