@@ -30,7 +30,7 @@
  * Tests of C API for sparse arrays with real domains.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "test/src/helpers.h"
 #include "test/src/vfs_helpers.h"
 #ifdef _WIN32
