@@ -30,7 +30,7 @@
  * Tests of C API for sparse arrays with string dimensions.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "test/src/helpers.h"
 #include "test/src/vfs_helpers.h"
 #ifdef _WIN32
@@ -85,7 +85,6 @@ struct StringDimsFx {
       const std::string& path, tiledb_array_schema_t* array_schema);
   int array_schema_load_wrapper(
       const std::string& path, tiledb_array_schema_t** array_schema);
-  void write_array_ascii(const std::string& array_name);
   void write_array_1d(
       tiledb_ctx_t* ctx,
       const std::string& array_name,
@@ -365,15 +364,7 @@ int StringDimsFx::tiledb_query_submit_wrapper(
   tiledb_buffer_list_t* buff_list1;
   int rc = tiledb_serialize_query(ctx, query, TILEDB_CAPNP, 1, &buff_list1);
 
-  // Global order writes are not (yet) supported for serialization. Just
-  // check that serialization is an error, and then execute the regular query.
-  if (layout == TILEDB_GLOBAL_ORDER && query_type == TILEDB_WRITE) {
-    REQUIRE(rc == TILEDB_ERR);
-    tiledb_buffer_list_free(&buff_list1);
-    return tiledb_query_submit(ctx, query);
-  } else {
-    REQUIRE(rc == TILEDB_OK);
-  }
+  REQUIRE(rc == TILEDB_OK);
 
   // Copy the data to a temporary memory region ("send over the network").
   tiledb_buffer_t* buff1;
@@ -613,49 +604,6 @@ int StringDimsFx::tiledb_query_submit_wrapper(
   return rc;
 }
 
-void StringDimsFx::write_array_ascii(const std::string& array_name) {
-  // Open array
-  tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, array_name.c_str(), &array);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
-  CHECK(rc == TILEDB_OK);
-
-  // Create and submit query
-  tiledb_query_t* query;
-  rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
-  REQUIRE(rc == TILEDB_OK);
-
-  char d_data[] = "aabbbcdddd";
-  uint64_t d_data_size = strlen(d_data);
-  uint64_t d_off[] = {0, 2, 5, 6};
-  uint64_t d_off_size = sizeof(d_off);
-  int32_t a_data[] = {1, 2, 3, 4};
-  uint64_t a_size = sizeof(a_data);
-  rc = tiledb_query_set_data_buffer(ctx_, query, "d", d_data, &d_data_size);
-  REQUIRE(rc == TILEDB_ERR);
-  rc = tiledb_query_set_data_buffer(ctx_, query, "d", d_data, &d_data_size);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_set_offsets_buffer(ctx_, query, "d", d_off, &d_off_size);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_set_data_buffer(ctx_, query, "a", a_data, &a_size);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_submit_wrapper(ctx_, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_finalize(ctx_, query);
-  REQUIRE(rc == TILEDB_OK);
-
-  // Close array
-  rc = tiledb_array_close(ctx_, array);
-  CHECK(rc == TILEDB_OK);
-
-  // Clean up
-  tiledb_array_free(&array);
-  tiledb_query_free(&query);
-}
-
 void StringDimsFx::write_array_1d(
     tiledb_ctx_t* ctx,
     const std::string& array_name,
@@ -688,10 +636,14 @@ void StringDimsFx::write_array_1d(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx, query, layout);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_submit_wrapper(ctx, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_finalize(ctx, query);
-  REQUIRE(rc == TILEDB_OK);
+  if (layout != TILEDB_GLOBAL_ORDER || !serialize_) {
+    rc = tiledb_query_submit(ctx, query);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_query_finalize(ctx, query);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    submit_and_finalize_serialized_query(ctx, query);
+  }
 
   // Close array
   rc = tiledb_array_close(ctx, array);
@@ -739,10 +691,14 @@ void StringDimsFx::write_array_2d(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx, query, layout);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_submit_wrapper(ctx, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_finalize(ctx, query);
-  REQUIRE(rc == TILEDB_OK);
+  if (layout != TILEDB_GLOBAL_ORDER || !serialize_) {
+    rc = tiledb_query_submit(ctx, query);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_query_finalize(ctx, query);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    submit_and_finalize_serialized_query(ctx, query);
+  }
 
   // Close array
   rc = tiledb_array_close(ctx, array);
@@ -1196,7 +1152,6 @@ void StringDimsFx::read_array_2d_default_string_range(
   rc = tiledb_query_set_layout(ctx, query, layout);
   REQUIRE(rc == TILEDB_OK);
 
-  // Submit query
   const char* array_uri;
   rc = tiledb_array_get_uri(ctx, array, &array_uri);
   CHECK(rc == TILEDB_OK);
@@ -1225,7 +1180,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -1333,7 +1290,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -1406,7 +1365,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -1480,7 +1441,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -1553,7 +1516,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -1672,7 +1637,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -1911,7 +1878,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -2098,7 +2067,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -2209,7 +2180,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -2337,7 +2310,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -2599,7 +2574,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -2620,7 +2597,7 @@ TEST_CASE_METHOD(
       {tiledb::test::Compressor(TILEDB_FILTER_NONE, -1)},
       TILEDB_ROW_MAJOR,
       TILEDB_ROW_MAJOR,
-      3,
+      1,
       false,
       false);
 
@@ -2650,17 +2627,24 @@ TEST_CASE_METHOD(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_submit_wrapper(ctx_, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
-
+  if (serialize_) {
+    submit_serialized_query(ctx_, query);
+  } else {
+    rc = tiledb_query_submit(ctx_, query);
+    REQUIRE(rc == TILEDB_OK);
+  }
   // Write "b, 2"
   d_data[0] = 'b';
   a_data[0] = 2;
-  rc = tiledb_query_submit_wrapper(ctx_, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
 
-  rc = tiledb_query_finalize(ctx_, query);
-  REQUIRE(rc == TILEDB_OK);
+  if (serialize_) {
+    submit_and_finalize_serialized_query(ctx_, query);
+  } else {
+    rc = tiledb_query_submit(ctx_, query);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_query_finalize(ctx_, query);
+    CHECK(rc == TILEDB_OK);
+  }
 
   // Close array
   rc = tiledb_array_close(ctx_, array);
@@ -2717,7 +2701,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -2738,7 +2724,7 @@ TEST_CASE_METHOD(
       {tiledb::test::Compressor(TILEDB_FILTER_NONE, -1)},
       TILEDB_ROW_MAJOR,
       TILEDB_ROW_MAJOR,
-      3,
+      1,
       false,
       false);
 
@@ -2768,14 +2754,23 @@ TEST_CASE_METHOD(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_submit_wrapper(ctx_, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
 
+  if (serialize_) {
+    submit_serialized_query(ctx_, query);
+  } else {
+    rc = tiledb_query_submit(ctx_, query);
+    REQUIRE(rc == TILEDB_OK);
+  }
   // Write "b, 2"
   d_data[0] = 'b';
   a_data[0] = 2;
-  rc = tiledb_query_submit_wrapper(ctx_, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
+
+  if (serialize_) {
+    submit_serialized_query(ctx_, query);
+  } else {
+    rc = tiledb_query_submit(ctx_, query);
+    REQUIRE(rc == TILEDB_OK);
+  }
 
   // Write c, 3; d, 4 and e, 5.
   d_data[0] = 'c';
@@ -2787,11 +2782,15 @@ TEST_CASE_METHOD(
   a_data[1] = 4;
   a_data[2] = 5;
   a_size = 3 * sizeof(int32_t);
-  rc = tiledb_query_submit_wrapper(ctx_, query, array_name);
-  REQUIRE(rc == TILEDB_OK);
 
-  rc = tiledb_query_finalize(ctx_, query);
-  REQUIRE(rc == TILEDB_OK);
+  if (serialize_) {
+    submit_and_finalize_serialized_query(ctx_, query);
+  } else {
+    rc = tiledb_query_submit(ctx_, query);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_query_finalize(ctx_, query);
+    CHECK(rc == TILEDB_OK);
+  }
 
   // Close array
   rc = tiledb_array_close(ctx_, array);
@@ -2848,7 +2847,9 @@ TEST_CASE_METHOD(
     serialize_ = false;
   }
   SECTION("- Serialization") {
+#ifdef TILEDB_SERIALIZATION
     serialize_ = true;
+#endif
   }
   SupportedFsLocal local_fs;
   std::string array_name =
@@ -3016,4 +3017,4 @@ TEST_CASE_METHOD(
   tiledb_config_free(&config);
 
   remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
-}
\ No newline at end of file
+}
