@@ -30,8 +30,10 @@
  * Tests the C++ API for array related functions.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "helpers.h"
+#include "test/src/serialization_wrappers.h"
+#include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/misc/constants.h"
@@ -345,8 +347,22 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic]") {
     query.set_data_buffer("a5", a5);
 
     query.set_layout(TILEDB_GLOBAL_ORDER);
-    CHECK(query.submit() == tiledb::Query::Status::COMPLETE);
-    REQUIRE_NOTHROW(query.finalize());
+
+    bool serialized_writes = false;
+    SECTION("no serialization") {
+      serialized_writes = false;
+    }
+    SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+      serialized_writes = true;
+#endif
+    }
+    if (!serialized_writes) {
+      CHECK(query.submit() == tiledb::Query::Status::COMPLETE);
+      REQUIRE_NOTHROW(query.finalize());
+    } else {
+      test::submit_and_finalize_serialized_query(ctx, query);
+    }
 
     // Check non-empty domain while array open in write mode
     CHECK_THROWS(array.non_empty_domain<int>(1));
@@ -413,6 +429,7 @@ TEST_CASE("C++ API: Zero length buffer", "[cppapi][zero-length]") {
   tiledb_array_type_t array_type = TILEDB_DENSE;
   bool null_pointer = true;
 
+  bool serialized_writes = false;
   SECTION("SPARSE") {
     array_type = TILEDB_SPARSE;
     SECTION("GLOBAL_ORDER") {
@@ -425,6 +442,14 @@ TEST_CASE("C++ API: Zero length buffer", "[cppapi][zero-length]") {
       SECTION("NON_NULL_PTR") {
         null_pointer = false;
       }
+      SECTION("no serialization") {
+        serialized_writes = false;
+      }
+      SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+        serialized_writes = true;
+#endif
+      }
     }
 
     SECTION("UNORDERED") {
@@ -477,8 +502,12 @@ TEST_CASE("C++ API: Zero length buffer", "[cppapi][zero-length]") {
     q.set_data_buffer("a", a);
     q.set_offsets_buffer("a", a_offset);
     q.set_data_buffer("b", b);
-    q.submit();
-    q.finalize();
+    if (!serialized_writes || write_layout != TILEDB_GLOBAL_ORDER) {
+      q.submit();
+      q.finalize();
+    } else {
+      test::submit_and_finalize_serialized_query(ctx, q);
+    }
 
     array.close();
   }
@@ -1096,8 +1125,21 @@ TEST_CASE(
   query_w.set_coordinates(coords_w)
       .set_layout(TILEDB_GLOBAL_ORDER)
       .set_data_buffer("a", data_w);
-  query_w.submit();
-  query_w.finalize();
+  bool serialized_writes = false;
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+  if (!serialized_writes) {
+    query_w.submit();
+    query_w.finalize();
+  } else {
+    test::submit_and_finalize_serialized_query(ctx, query_w);
+  }
   array_w.close();
 
   // Read
@@ -1468,8 +1510,21 @@ TEST_CASE(
   query_w.set_coordinates(coords_w)
       .set_layout(TILEDB_GLOBAL_ORDER)
       .set_data_buffer("a", data_w);
-  query_w.submit();
-  query_w.finalize();
+  bool serialized_writes = false;
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+  if (!serialized_writes) {
+    query_w.submit();
+    query_w.finalize();
+  } else {
+    test::submit_and_finalize_serialized_query(ctx, query_w);
+  }
   array_w.close();
 
   // Read
@@ -1517,8 +1572,21 @@ TEST_CASE(
   query_w.set_coordinates(coords_w)
       .set_layout(TILEDB_GLOBAL_ORDER)
       .set_data_buffer("a", data_w);
-  query_w.submit();
-  query_w.finalize();
+  bool serialized_writes = false;
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+  if (!serialized_writes) {
+    query_w.submit();
+    query_w.finalize();
+  } else {
+    test::submit_and_finalize_serialized_query(ctx, query_w);
+  }
   array_w.close();
 
   // Read
@@ -1704,3 +1772,121 @@ TEST_CASE(
       stats.find("\"Context.StorageManager.VFS.file_size_num\": 1") !=
       std::string::npos);
 }
+
+TEST_CASE(
+    "C++ API: Write and read to an array with experimental build enabled",
+    "[cppapi][array][experimental]") {
+  if constexpr (!is_experimental_build) {
+    return;
+  }
+
+  static const std::string arrays_dir =
+      std::string(TILEDB_TEST_INPUTS_DIR) + "/arrays";
+  std::string old_array_name(arrays_dir + "/non_split_coords_v1_4_0");
+  std::string new_array_name(arrays_dir + "/experimental_array_vUINT32_MAX");
+  Context ctx;
+
+  // Try writing to an older-versioned array
+  REQUIRE_THROWS_WITH(
+      Array(ctx, old_array_name, TILEDB_WRITE),
+      Catch::Contains("Array format version") &&
+          Catch::Contains("is not the library format version"));
+
+  // Read from an older-versioned array
+  Array array(ctx, old_array_name, TILEDB_READ);
+
+  // Upgrade version
+  Array::upgrade_version(ctx, old_array_name);
+
+  // Write to upgraded (current) version
+  Array array_write(ctx, old_array_name, TILEDB_WRITE);
+  std::vector<int> subarray_write = {1, 2, 10, 10};
+  std::vector<int> a_write = {11, 12};
+  std::vector<int> d1_write = {1, 2};
+  std::vector<int> d2_write = {10, 10};
+
+  Query query_write(ctx, array_write, TILEDB_WRITE);
+
+  query_write.set_layout(TILEDB_GLOBAL_ORDER);
+  query_write.set_data_buffer("a", a_write);
+  query_write.set_data_buffer("d1", d1_write);
+  query_write.set_data_buffer("d2", d2_write);
+
+  query_write.submit();
+  query_write.finalize();
+  array_write.close();
+
+  FragmentInfo fragment_info(ctx, old_array_name);
+  fragment_info.load();
+
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
+  if (serialized_load) {
+    FragmentInfo deserialized_fragment_info(ctx, old_array_name);
+    tiledb_fragment_info_serialize(
+        ctx.ptr().get(),
+        old_array_name.c_str(),
+        fragment_info.ptr().get(),
+        deserialized_fragment_info.ptr().get(),
+        tiledb_serialization_type_t(0));
+    fragment_info = deserialized_fragment_info;
+  }
+
+  std::string fragment_uri = fragment_info.fragment_uri(1);
+
+  // old version fragment
+  CHECK(fragment_info.version(0) == 1);
+  // new version fragment
+  CHECK(fragment_info.version(1) == tiledb::sm::constants::format_version);
+
+  // Read from upgraded version
+  Array array_read(ctx, old_array_name, TILEDB_READ);
+  std::vector<int> subarray_read = {1, 4, 10, 10};
+  std::vector<int> a_read;
+  a_read.resize(4);
+  std::vector<int> d1_read;
+  d1_read.resize(4);
+  std::vector<int> d2_read;
+  d2_read.resize(4);
+
+  Query query_read(ctx, array_read);
+  query_read.set_subarray(subarray_read)
+      .set_layout(TILEDB_ROW_MAJOR)
+      .set_data_buffer("a", a_read)
+      .set_data_buffer("d1", d1_read)
+      .set_data_buffer("d2", d2_read);
+
+  query_read.submit();
+  array_read.close();
+
+  for (int i = 0; i < 2; i++) {
+    REQUIRE(a_read[i] == i + 11);
+  }
+  for (int i = 2; i < 3; i++) {
+    REQUIRE(a_read[i] == i + 1);
+  }
+
+  // Try writing to a newer-versioned (UINT32_MAX) array
+  REQUIRE_THROWS_WITH(
+      Array(ctx, new_array_name, TILEDB_WRITE),
+      Catch::Contains("Incompatible format version."));
+
+  // Try reading from a newer-versioned (UINT32_MAX) array
+  REQUIRE_THROWS_WITH(
+      Array(ctx, new_array_name, TILEDB_READ),
+      Catch::Contains("Incompatible format version."));
+
+  // Clean up
+  VFS vfs(ctx);
+  vfs.remove_dir(get_fragment_dir(array_read.uri()));
+  vfs.remove_dir(get_commit_dir(array_read.uri()));
+  vfs.remove_dir(array_read.uri() + "/__schema");
+}
