@@ -27,11 +27,12 @@
  *
  * @section DESCRIPTION
  *
- * Tests the CPP API consolidation with timestamps.
+ * Tests the CPP API for consolidation with timestamps.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "test/src/helpers.h"
+#include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/sm/array/array_directory.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
@@ -91,6 +92,7 @@ struct ConsolidationWithTimestampsFx {
   void remove_sparse_array();
   void remove_array(const std::string& array_name);
   bool is_array(const std::string& array_name);
+  bool serialized_writes = false;
 };
 
 ConsolidationWithTimestampsFx::ConsolidationWithTimestampsFx()
@@ -98,7 +100,7 @@ ConsolidationWithTimestampsFx::ConsolidationWithTimestampsFx()
   Config config;
   config.set("sm.consolidation.buffer_size", "1000");
   ctx_ = Context(config);
-  sm_ = ctx_.ptr().get()->ctx_->storage_manager();
+  sm_ = ctx_.ptr().get()->storage_manager();
   vfs_ = VFS(ctx_);
 }
 
@@ -112,7 +114,7 @@ void ConsolidationWithTimestampsFx::set_legacy() {
   config.set("sm.query.sparse_unordered_with_dups.reader", "legacy");
 
   ctx_ = Context(config);
-  sm_ = ctx_.ptr().get()->ctx_->storage_manager();
+  sm_ = ctx_.ptr().get()->storage_manager();
   vfs_ = VFS(ctx_);
 }
 
@@ -175,9 +177,12 @@ void ConsolidationWithTimestampsFx::write_sparse(
   query.set_data_buffer("d1", dim1);
   query.set_data_buffer("d2", dim2);
 
-  // Submit/finalize the query.
-  query.submit();
-  query.finalize();
+  if (!serialized_writes) {
+    query.submit();
+    query.finalize();
+  } else {
+    test::submit_and_finalize_serialized_query(ctx_, query);
+  }
 
   // Close array.
   array.close();
@@ -205,10 +210,12 @@ void ConsolidationWithTimestampsFx::write_sparse_v11(uint64_t timestamp) {
   query.set_data_buffer("a3", buffer_a3);
   query.set_data_buffer("d1", buffer_coords_dim1);
   query.set_data_buffer("d2", buffer_coords_dim2);
-
-  // Submit/finalize the query.
-  query.submit();
-  query.finalize();
+  if (!serialized_writes) {
+    query.submit();
+    query.finalize();
+  } else {
+    test::submit_and_finalize_serialized_query(ctx_, query);
+  }
 
   // Close array.
   array.close();
@@ -373,6 +380,14 @@ TEST_CASE_METHOD(
   remove_sparse_array();
   create_sparse_array();
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse(
       {0, 1, 2, 3, 4, 5, 6, 7},
@@ -400,6 +415,14 @@ TEST_CASE_METHOD(
   remove_sparse_array();
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
 
@@ -475,6 +498,14 @@ TEST_CASE_METHOD(
   remove_sparse_array();
   create_sparse_array();
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse(
       {0, 1, 2, 3, 4, 5, 6, 7},
@@ -512,6 +543,7 @@ TEST_CASE_METHOD(
   remove_sparse_array();
 }
 
+// TODO: remove once tiledb_vfs_copy_dir is implemented for windows.
 #ifndef _WIN32
 TEST_CASE_METHOD(
     ConsolidationWithTimestampsFx,
@@ -522,6 +554,15 @@ TEST_CASE_METHOD(
     return;
   }
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+
   remove_sparse_array();
   create_sparse_array_v11();
   // Write first fragment.
@@ -602,6 +643,15 @@ TEST_CASE_METHOD(
   remove_sparse_array();
   create_sparse_array();
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+
   // Write fragments.
   for (uint64_t i = 0; i < 50; i++) {
     std::vector<int> a(1);
@@ -639,7 +689,17 @@ TEST_CASE_METHOD(
     "across tiles",
     "[cppapi][consolidation-with-timestamps][global-read][across-tiles]") {
   remove_sparse_array();
-  create_sparse_array();
+  bool allow_dups = GENERATE(true, false);
+  create_sparse_array(allow_dups);
+
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
 
   // Write fragments.
   // We write 8 cells per fragments for 6 fragments. Then it gets consolidated
@@ -659,7 +719,7 @@ TEST_CASE_METHOD(
 
   // Test read for both refactored and legacy.
   bool legacy = GENERATE(true, false);
-  uint64_t buffer_size = 8;
+  uint64_t buffer_size = allow_dups ? 48 : 8;
   if (legacy) {
     set_legacy();
     buffer_size = 100;
@@ -674,9 +734,15 @@ TEST_CASE_METHOD(
   std::vector<int> c_a1 = {1, 2, 3, 4, 5, 6, 7, 8};
   std::vector<uint64_t> c_dim1 = {1, 1, 2, 2, 1, 1, 2, 2};
   std::vector<uint64_t> c_dim2 = {1, 2, 1, 2, 3, 4, 3, 4};
-  CHECK(!memcmp(c_a1.data(), a1.data(), c_a1.size() * sizeof(int)));
-  CHECK(!memcmp(c_dim1.data(), dim1.data(), c_dim1.size() * sizeof(uint64_t)));
-  CHECK(!memcmp(c_dim2.data(), dim2.data(), c_dim2.size() * sizeof(uint64_t)));
+  for (uint64_t i = 0; i < 8; i++) {
+    uint64_t max_j = allow_dups ? 6 : 1;
+    for (uint64_t j = 0; j < max_j; j++) {
+      uint64_t idx = i * max_j + j;
+      CHECK(a1[idx] == c_a1[i]);
+      CHECK(dim1[idx] == c_dim1[i]);
+      CHECK(dim2[idx] == c_dim2[i]);
+    }
+  }
 
   remove_sparse_array();
 }
@@ -690,6 +756,14 @@ TEST_CASE_METHOD(
   remove_sparse_array();
   create_sparse_array();
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write fragments.
   for (uint64_t i = 0; i < 50; i++) {
     std::vector<int> a(1);
@@ -733,6 +807,14 @@ TEST_CASE_METHOD(
   remove_sparse_array();
   create_sparse_array();
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write fragments.
   // We write 8 cells per fragment for 6 fragments. Then it gets consolidated
   // into one. So we'll get in order 6xcell1, 6xcell2... total 48 cells. Tile
@@ -752,7 +834,7 @@ TEST_CASE_METHOD(
   // Will only allow to load two tiles out of 3.
   Config cfg;
   cfg.set("sm.mem.total_budget", "10000");
-  cfg.set("sm.mem.reader.sparse_global_order.ratio_coords", "0.38");
+  cfg.set("sm.mem.reader.sparse_global_order.ratio_coords", "0.4");
   ctx_ = Context(cfg);
 
   std::string stats;
@@ -784,6 +866,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
 
@@ -871,6 +961,14 @@ TEST_CASE_METHOD(
   // No duplicates.
   create_sparse_array();
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
 
@@ -941,6 +1039,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1087,6 +1193,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1195,6 +1309,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1288,6 +1410,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1389,6 +1519,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1492,6 +1630,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1573,6 +1719,14 @@ TEST_CASE_METHOD(
     remove_sparse_array();
     create_sparse_array(false);
 
+    SECTION("no serialization") {
+      serialized_writes = false;
+    }
+    SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+      serialized_writes = true;
+#endif
+    }
     // Write first fragment.
     write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
     // Write second fragment.
@@ -1592,6 +1746,14 @@ TEST_CASE_METHOD(
     remove_sparse_array();
     create_sparse_array(false);
 
+    SECTION("no serialization") {
+      serialized_writes = false;
+    }
+    SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+      serialized_writes = true;
+#endif
+    }
     // Write first fragment.
     write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
     // Write second fragment.
@@ -1630,6 +1792,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(dups);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1668,7 +1838,7 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> c_dim2 = {
         1, 2, 1, 2, 3, 4, 3, 4, 1, 2, 1, 2, 3, 3, 4, 4};
     std::vector<uint64_t> c_ts = {
-        1, 1, 5, 3, 5, 1, 1, 3, 5, 3, 5, 7, 3, 7, 7, 7};
+        1, 1, 5, 3, 5, 1, 1, 3, 5, 3, 5, 7, 7, 3, 7, 7};
     CHECK(
         (!memcmp(c_a_1.data(), a.data(), c_a_1.size() * sizeof(int)) ||
          !memcmp(c_a_2.data(), a.data(), c_a_2.size() * sizeof(int))));
@@ -1706,6 +1876,14 @@ TEST_CASE_METHOD(
   // Enable duplicates.
   create_sparse_array(true);
 
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1767,4 +1945,4 @@ TEST_CASE_METHOD(
   array.close();
 
   remove_sparse_array();
-}
\ No newline at end of file
+}
