@@ -31,9 +31,10 @@
  */
 
 #include "test/src/helpers.h"
+#include "test/src/serialization_wrappers.h"
 #include "tiledb/sm/cpp_api/tiledb"
 
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 #include <iostream>
 
 using namespace tiledb;
@@ -93,6 +94,27 @@ TEST_CASE(
     // Load fragment info, array is encrypted
     fragment_info.load();
 
+    bool serialized_load = false;
+    SECTION("no serialization") {
+      serialized_load = false;
+    }
+#ifdef TILEDB_SERIALIZATION
+    SECTION("serialization enabled fragment info load") {
+      serialized_load = true;
+    }
+#endif
+
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get fragment URI
     std::string uri;
     CHECK_THROWS(uri = fragment_info.fragment_uri(1));
@@ -145,6 +167,16 @@ TEST_CASE(
       TILEDB_ROW_MAJOR,
       2);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
   {
     // Create fragment info object
     FragmentInfo fragment_info(ctx, array_name);
@@ -152,6 +184,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // No fragments yet
     auto fragment_num = fragment_info.fragment_num();
     CHECK(fragment_num == 0);
@@ -173,6 +216,17 @@ TEST_CASE(
     // Load fragment info again
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get fragment num again
     auto fragment_num = fragment_info.fragment_num();
     CHECK(fragment_num == 1);
@@ -207,12 +261,32 @@ TEST_CASE(
       ctx.ptr().get(), array_name, 3, subarray, TILEDB_ROW_MAJOR, buffers);
 
   {
+    tiledb::Config cfg;
+    if (serialized_load) {
+      // Set the config that forces preloading of MBRs in remote
+      // case instead of loading them lazily.
+      cfg["sm.fragment_info.preload_mbrs"] = "true";
+    }
+
+    Context ctx2(cfg);
     // Create fragment info object
-    FragmentInfo fragment_info(ctx, array_name);
+    FragmentInfo fragment_info(ctx2, array_name);
 
     // Load fragment info again
     fragment_info.load();
 
+    if (serialized_load) {
+      // serialize and deserialize back fragment info
+      FragmentInfo deserialized_fragment_info(ctx2, array_name);
+      tiledb_fragment_info_serialize(
+          ctx2.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get fragment num again
     auto fragment_num = fragment_info.fragment_num();
     CHECK(fragment_num == 3);
@@ -227,7 +301,7 @@ TEST_CASE(
 
     // Get fragment size
     auto size = fragment_info.fragment_size(1);
-    CHECK(size == 3185);
+    CHECK(size == 3202);
 
     // Get dense / sparse
     auto dense = fragment_info.dense(0);
@@ -377,10 +451,38 @@ TEST_CASE("C++ API: Test MBR fragment info", "[cppapi][fragment_info][mbr]") {
       &written_frag_uri);
 
   {
+    tiledb::Config cfg;
+    bool serialized_load = false;
+
+    SECTION("no serialization") {
+      serialized_load = false;
+    }
+#ifdef TILEDB_SERIALIZATION
+    SECTION("serialization enabled fragment info load") {
+      // Set the config that forces preloading of MBRs in remote
+      // case instead of loading them lazily.
+      cfg["sm.fragment_info.preload_mbrs"] = "true";
+      serialized_load = true;
+    }
+#endif
+
+    Context ctx2(cfg);
     // Create fragment info object
-    FragmentInfo fragment_info(ctx, array_name);
+    FragmentInfo fragment_info(ctx2, array_name);
     // Load fragment info
     fragment_info.load();
+
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx2, array_name);
+      tiledb_fragment_info_serialize(
+          ctx2.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     auto fragment_num = fragment_info.fragment_num();
     CHECK(fragment_num == 3);
 
@@ -455,12 +557,39 @@ TEST_CASE(
       &written_frag_uri);
 
   {
+    tiledb::Config cfg;
+    bool serialized_load = false;
+
+    SECTION("no serialization") {
+      serialized_load = false;
+    }
+#ifdef TILEDB_SERIALIZATION
+    SECTION("serialization enabled fragment info load") {
+      // Set the config that forces preloading of MBRs in remote
+      // case instead of loading them lazily.
+      cfg["sm.fragment_info.preload_mbrs"] = "true";
+      serialized_load = true;
+    }
+#endif
+
+    Context ctx2(cfg);
     // Create fragment info object
-    FragmentInfo fragment_info(ctx, array_name);
+    FragmentInfo fragment_info(ctx2, array_name);
 
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx2, array_name);
+      tiledb_fragment_info_serialize(
+          ctx2.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get non-empty domain
     auto non_empty_domain_str = fragment_info.non_empty_domain_var(0, 0);
     CHECK(std::string("a") == non_empty_domain_str.first);
@@ -537,6 +666,16 @@ TEST_CASE(
   write_array(
       ctx.ptr().get(), array_name, 2, subarray, TILEDB_ROW_MAJOR, buffers);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
   {
     // Create fragment info object
     FragmentInfo fragment_info(ctx, array_name);
@@ -544,6 +683,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Check for consolidated metadata
     auto has = fragment_info.has_consolidated_metadata(0);
     CHECK(has == false);
@@ -568,6 +718,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Check for consolidated metadata
     auto has = fragment_info.has_consolidated_metadata(0);
     CHECK(has == true);
@@ -595,6 +756,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Check for consolidated metadata
     auto has = fragment_info.has_consolidated_metadata(0);
     CHECK(has == true);
@@ -663,6 +835,16 @@ TEST_CASE(
   write_array(
       ctx.ptr().get(), array_name, 2, subarray, TILEDB_ROW_MAJOR, buffers);
 
+  bool serialized_load = false;
+  SECTION("no serialization") {
+    serialized_load = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled fragment info load") {
+    serialized_load = true;
+  }
+#endif
+
   {
     // Create fragment info object
     FragmentInfo fragment_info(ctx, array_name);
@@ -670,6 +852,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get number of fragments to vacuum
     auto to_vacuum_num = fragment_info.to_vacuum_num();
     CHECK(to_vacuum_num == 0);
@@ -691,6 +884,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get consolidated fragment URI
     auto uri = fragment_info.fragment_uri(0);
     CHECK(uri.find_last_of("__1_2") != std::string::npos);
@@ -720,6 +924,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get number of fragments to vacuum
     auto to_vacuum_num = fragment_info.to_vacuum_num();
     CHECK(to_vacuum_num == 2);
@@ -735,6 +950,17 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get number of fragments to vacuum
     auto to_vacuum_num = fragment_info.to_vacuum_num();
     CHECK(to_vacuum_num == 0);
@@ -824,6 +1050,27 @@ TEST_CASE(
     // Load fragment info
     fragment_info.load();
 
+    bool serialized_load = false;
+    SECTION("no serialization") {
+      serialized_load = false;
+    }
+#ifdef TILEDB_SERIALIZATION
+    SECTION("serialization enabled fragment info load") {
+      serialized_load = true;
+    }
+#endif
+
+    if (serialized_load) {
+      FragmentInfo deserialized_fragment_info(ctx, array_name);
+      tiledb_fragment_info_serialize(
+          ctx.ptr().get(),
+          array_name.c_str(),
+          fragment_info.ptr().get(),
+          deserialized_fragment_info.ptr().get(),
+          tiledb_serialization_type_t(0));
+      fragment_info = deserialized_fragment_info;
+    }
+
     // Get Array Schemas for fragments
     ArraySchema frag1_schema = fragment_info.array_schema(0);
     ArraySchema frag2_schema = fragment_info.array_schema(1);
@@ -851,16 +1098,16 @@ TEST_CASE(
         "- Unconsolidated metadata num: 3\n" + "- To vacuum num: 0\n" +
         "- Fragment #1:\n" + "  > URI: " + written_frag_uri_1 + "\n" +
         "  > Type: dense\n" + "  > Non-empty domain: [1, 6]\n" +
-        "  > Size: 3185\n" + "  > Cell num: 10\n" +
+        "  > Size: 3202\n" + "  > Cell num: 10\n" +
         "  > Timestamp range: [1, 1]\n" + "  > Format version: " + ver + "\n" +
         "  > Has consolidated metadata: no\n" + "- Fragment #2:\n" +
         "  > URI: " + written_frag_uri_2 + "\n" + "  > Type: dense\n" +
-        "  > Non-empty domain: [1, 4]\n" + "  > Size: 3134\n" +
+        "  > Non-empty domain: [1, 4]\n" + "  > Size: 3151\n" +
         "  > Cell num: 5\n" + "  > Timestamp range: [2, 2]\n" +
         "  > Format version: " + ver + "\n" +
         "  > Has consolidated metadata: no\n" + "- Fragment #3:\n" +
         "  > URI: " + written_frag_uri_3 + "\n" + "  > Type: dense\n" +
-        "  > Non-empty domain: [5, 6]\n" + "  > Size: 3182\n" +
+        "  > Non-empty domain: [5, 6]\n" + "  > Size: 3202\n" +
         "  > Cell num: 10\n" + "  > Timestamp range: [3, 3]\n" +
         "  > Format version: " + ver + "\n" +
         "  > Has consolidated metadata: no\n";
