@@ -42,7 +42,7 @@
 #include "tiledb/sm/filesystem/posix.h"
 #endif
 
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 
 using namespace tiledb;
 using namespace tiledb::test;
@@ -420,10 +420,10 @@ TEST_CASE_METHOD(
     write_1d_fragment(coords, &coords_size, data, &data_size);
   }
 
-  // Two result tile (2 * (~504 + 8) will be bigger than the per fragment budget
-  // (1000).
+  // Two result tile (2 * (~1200 + 8) will be bigger than the per fragment
+  // budget (1000).
   total_budget_ = "10000";
-  ratio_coords_ = "0.2";
+  ratio_coords_ = "0.30";
   update_config();
 
   tiledb_array_t* array = nullptr;
@@ -948,10 +948,21 @@ TEST_CASE(
   query.set_data_buffer("d1", d1);
   query.set_data_buffer("a", a1_data);
   query.set_offsets_buffer("a", a1_offsets);
-  CHECK_NOTHROW(query.submit());
-
-  // Finalize is necessary in global writes, otherwise a no-op
-  query.finalize();
+  bool serialized_writes = false;
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+  SECTION("serialization enabled global order write") {
+#ifdef TILEDB_SERIALIZATION
+    serialized_writes = true;
+#endif
+  }
+  if (!serialized_writes) {
+    CHECK_NOTHROW(query.submit());
+    query.finalize();
+  } else {
+    submit_and_finalize_serialized_query(ctx, query);
+  }
 
   // Read using a buffer that can't fit a single result
   Array array2(ctx, array_name, TILEDB_READ);
@@ -1154,4 +1165,4 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
-}
\ No newline at end of file
+}
