@@ -275,10 +275,7 @@ inline Status Status_ArraySchemaError(const std::string& msg) {
 inline Status Status_ArraySchemaEvolutionError(const std::string& msg) {
   return {"[TileDB::ArraySchemaEvolution] Error", msg};
 }
-/** Return a Metadata error class Status with a given message **/
-inline Status Status_MetadataError(const std::string& msg) {
-  return {"[TileDB::Metadata] Error", msg};
-}
+
 /** Return a IO error class Status with a given message **/
 inline Status Status_IOError(const std::string& msg) {
   return {"[TileDB::IO] Error", msg};
@@ -335,10 +332,6 @@ inline Status Status_ConsolidatorError(const std::string& msg) {
 inline Status Status_LRUCacheError(const std::string& msg) {
   return {"[TileDB::LRUCache] Error", msg};
 }
-/** Return a Config error class Status with a given message **/
-inline Status Status_ConfigError(const std::string& msg) {
-  return {"[TileDB::Config] Error", msg};
-}
 /** Return a Utils error class Status with a given message **/
 inline Status Status_UtilsError(const std::string& msg) {
   return {"[TileDB::Utils] Error", msg};
@@ -464,6 +457,11 @@ inline Status Status_DenseTilerError(const std::string& msg) {
 inline Status Status_QueryConditionError(const std::string& msg) {
   return {"[TileDB::QueryCondition] Error", msg};
 }
+/** Return a Status_UpdateValueError error class Status with a given
+ * message **/
+inline Status Status_UpdateValueError(const std::string& msg) {
+  return {"[TileDB::UpdateValue] Error", msg};
+}
 /** Return a Status_ArrayDirectoryError error class Status with a given
  * message **/
 inline Status Status_ArrayDirectoryError(const std::string& msg) {
