@@ -37,6 +37,7 @@
 #include "tiledb/common/thread_pool.h"
 #include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/filesystem/vfs.h"
+#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <unordered_map>
 
@@ -150,6 +151,7 @@ class ArrayDirectory {
     std::vector<TimestampedURI> fragment_uris_;
   };
 
+ public:
   /**
    * Class to return a location of a delete tile, which is file URI/offset.
    */
@@ -162,25 +164,48 @@ class ArrayDirectory {
     /**
      * Constructor.
      */
-    DeleteTileLocation(const URI& uri, const storage_size_t offset)
+    DeleteTileLocation(
+        const URI& uri,
+        const std::string condition_marker,
+        const storage_size_t offset)
         : uri_(uri)
+        , condition_marker_(condition_marker)
         , offset_(offset) {
+      std::pair<uint64_t, uint64_t> timestamps;
+      if (!utils::parse::get_timestamp_range(URI(condition_marker), &timestamps)
+               .ok()) {
+        throw std::logic_error("Error parsing uri.");
+      }
+
+      timestamp_ = timestamps.first;
     }
 
     /** Destructor. */
     ~DeleteTileLocation() = default;
 
+    bool operator<(const DeleteTileLocation& rhs) const {
+      return (timestamp_ < rhs.timestamp_);
+    }
+
     /* ********************************* */
     /*                API                */
     /* ********************************* */
-    inline URI& Uri() {
+    inline const URI& uri() const {
       return uri_;
     }
 
-    inline uint64_t Offset() {
+    inline const std::string& condition_marker() const {
+      return condition_marker_;
+    }
+
+    inline uint64_t offset() const {
       return offset_;
     }
 
+    inline uint64_t timestamp() const {
+      return timestamp_;
+    }
+
    private:
     /* ********************************* */
     /*         PRIVATE ATTRIBUTES        */
@@ -189,11 +214,16 @@ class ArrayDirectory {
     /** The URIs of the file. */
     URI uri_;
 
+    /** The condition marker. */
+    std::string condition_marker_;
+
     /** The offset within the file. */
     uint64_t offset_;
+
+    /** Stores the timestamp of the delete for sorting. */
+    uint64_t timestamp_;
   };
 
- public:
   /* ********************************* */
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
@@ -241,6 +271,9 @@ class ArrayDirectory {
   /** Returns the latest array schema URI. */
   const URI& latest_array_schema_uri() const;
 
+  /** Returns the unfiltered fragment uris. */
+  const std::vector<URI>& unfiltered_fragment_uris() const;
+
   /** Returns the URIs of the array metadata files to vacuum. */
   const std::vector<URI>& array_meta_uris_to_vacuum() const;
 
@@ -253,6 +286,9 @@ class ArrayDirectory {
   /** Returns the URIs of the commit files to vacuum. */
   const std::vector<URI>& commit_uris_to_vacuum() const;
 
+  /** Returns the consolidated commit URI set. */
+  const std::unordered_set<std::string>& consolidated_commit_uris_set() const;
+
   /** Returns the URIs of the consolidated commit files to vacuum. */
   const std::vector<URI>& consolidated_commits_uris_to_vacuum() const;
 
@@ -265,6 +301,9 @@ class ArrayDirectory {
   /** Returns the location of delete tiles. */
   const std::vector<DeleteTileLocation>& delete_tiles_location() const;
 
+  /** Returns the fragment absolute path given an array URI and a version */
+  static URI generate_fragment_dir_uri(uint32_t write_version, URI array_uri);
+
   /** Returns the URI to store fragments. */
   URI get_fragments_dir(uint32_t write_version) const;
 
@@ -275,10 +314,10 @@ class ArrayDirectory {
   URI get_commits_dir(uint32_t write_version) const;
 
   /** Returns the URI for either an ok file or wrt file. */
-  tuple<Status, optional<URI>> get_commit_uri(const URI& fragment_uri) const;
+  URI get_commit_uri(const URI& fragment_uri) const;
 
   /** Returns the URI for a vacuum file. */
-  tuple<Status, optional<URI>> get_vaccum_uri(const URI& fragment_uri) const;
+  URI get_vacuum_uri(const URI& fragment_uri) const;
 
   /**
    * The new fragment name is computed
@@ -290,6 +329,7 @@ class ArrayDirectory {
   /** Returns `true` if `load` has been run. */
   bool loaded() const;
 
+  /** Returns the filtered fragment URIs struct. */
   const FilteredFragmentUris filtered_fragment_uris(
       const bool full_overlap_only) const;
 
