@@ -33,7 +33,7 @@
 
 #include "tiledb/sm/buffer/buffer.h"
 
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 #include <iostream>
 
 using namespace tiledb::common;
