@@ -61,8 +61,7 @@ int32_t tiledb_group_create(tiledb_ctx_t* ctx, const char* group_uri) {
   }
 
   // Create the group
-  if (SAVE_ERROR_CATCH(
-          ctx, ctx->ctx_->storage_manager()->group_create(group_uri)))
+  if (SAVE_ERROR_CATCH(ctx, ctx->storage_manager()->group_create(group_uri)))
     return TILEDB_ERR;
 
   // Success
@@ -99,7 +98,7 @@ int32_t tiledb_group_alloc(
 
   // Allocate a group object
   (*group)->group_ = tdb_unique_ptr<tiledb::sm::Group>(
-      tdb_new(tiledb::sm::GroupV1, uri, ctx->ctx_->storage_manager()));
+      tdb_new(tiledb::sm::GroupV1, uri, ctx->storage_manager()));
   if ((*group)->group_ == nullptr) {
     delete *group;
     *group = nullptr;
@@ -479,7 +478,7 @@ int32_t tiledb_group_get_member_by_name(
     return TILEDB_ERR;
 
   try {
-    auto&& [st, uri_str, object_type, name_str] =
+    auto&& [st, uri_str, object_type, name_str, ignored_relative] =
         group->group_->member_by_name(name);
     if (!st.ok()) {
       save_error(ctx, st);
@@ -506,6 +505,27 @@ int32_t tiledb_group_get_member_by_name(
   return TILEDB_ERR;
 }
 
+int32_t tiledb_group_get_is_relative_uri_by_name(
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    const char* name,
+    uint8_t* is_relative) {
+  // Sanity check
+  if (sanity_check(ctx) == TILEDB_ERR || sanity_check(ctx, group) == TILEDB_ERR)
+    return TILEDB_ERR;
+  if (is_relative == nullptr) {
+    return TILEDB_ERR;
+  }
+
+  auto&& [st, uri_str, object_type, name_str, relative] =
+      group->group_->member_by_name(name);
+  throw_if_not_ok(st);
+
+  *is_relative = *relative ? 1 : 0;
+
+  return TILEDB_OK;
+}
+
 int32_t tiledb_group_get_uri(
     tiledb_ctx_t* ctx, tiledb_group_t* group, const char** group_uri) {
   // Sanity checks
@@ -634,9 +654,9 @@ int32_t tiledb_group_consolidate_metadata(
 
   if (SAVE_ERROR_CATCH(
           ctx,
-          ctx->ctx_->storage_manager()->group_metadata_consolidate(
+          ctx->storage_manager()->group_metadata_consolidate(
               group_uri,
-              (config == nullptr) ? &ctx->ctx_->storage_manager()->config() :
+              (config == nullptr) ? &ctx->storage_manager()->config() :
                                     config->config_)))
     return TILEDB_ERR;
 
@@ -651,9 +671,9 @@ int32_t tiledb_group_vacuum_metadata(
 
   if (SAVE_ERROR_CATCH(
           ctx,
-          ctx->ctx_->storage_manager()->group_metadata_vacuum(
+          ctx->storage_manager()->group_metadata_vacuum(
               group_uri,
-              (config == nullptr) ? &ctx->ctx_->storage_manager()->config() :
+              (config == nullptr) ? &ctx->storage_manager()->config() :
                                     config->config_)))
     return TILEDB_ERR;
 
@@ -800,6 +820,15 @@ TILEDB_EXPORT int32_t tiledb_group_get_member_by_name(
       ctx, group, name, uri, type);
 }
 
+TILEDB_EXPORT int32_t tiledb_group_get_is_relative_uri_by_name(
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    const char* name,
+    uint8_t* relative) TILEDB_NOEXCEPT {
+  return api_entry_context<detail::tiledb_group_get_is_relative_uri_by_name>(
+      ctx, group, name, relative);
+}
+
 TILEDB_EXPORT int32_t tiledb_group_is_open(
     tiledb_ctx_t* ctx,
     tiledb_group_t* group,
