@@ -32,6 +32,7 @@
 
 #include "tiledb/sm/consolidator/commits_consolidator.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/query_type.h"
 #include "tiledb/sm/misc/parallel_functions.h"
@@ -64,6 +65,8 @@ Status CommitsConsolidator::consolidate(
     uint32_t key_length) {
   auto timer_se = stats_->start_timer("consolidate_commits");
 
+  check_array_uri(array_name);
+
   // Open array for writing
   auto array_uri = URI(array_name);
   Array array_for_writes(array_uri, storage_manager_);
@@ -79,18 +82,17 @@ Status CommitsConsolidator::consolidate(
   }
 
   // Get the array uri to consolidate from the array directory.
-  ArrayDirectory array_dir;
-  try {
-    array_dir = ArrayDirectory(
-        storage_manager_->vfs(),
-        storage_manager_->compute_tp(),
-        URI(array_name),
-        0,
-        utils::time::timestamp_now_ms(),
-        false,
-        ArrayDirectoryMode::COMMITS);
-  } catch (const std::logic_error& le) {
-    return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
+  auto array_dir = ArrayDirectory(
+      storage_manager_->vfs(),
+      storage_manager_->compute_tp(),
+      URI(array_name),
+      0,
+      utils::time::timestamp_now_ms(),
+      ArrayDirectoryMode::COMMITS);
+
+  // Don't try to consolidate empty array
+  if (array_dir.commit_uris_to_consolidate().size() == 0) {
+    return Status::Ok();
   }
 
   // Get the file name.
@@ -99,19 +101,52 @@ Status CommitsConsolidator::consolidate(
       to_consolidate.front(), to_consolidate.back(), write_version);
   RETURN_NOT_OK(st1);
 
-  // Write consolidated file, URIs are relative to the array URI.
-  std::stringstream ss;
-  auto base_uri_size = array_dir.uri().to_string().size();
-  for (const auto& uri : to_consolidate) {
-    ss << uri.to_string().substr(base_uri_size) << "\n";
+  // Compute size of consolidated file. Save the sizes of the files to re-use
+  // below.
+  storage_size_t total_size = 0;
+  const auto base_uri_size = array_dir.uri().to_string().size();
+  std::vector<storage_size_t> file_sizes(to_consolidate.size());
+  for (uint64_t i = 0; i < to_consolidate.size(); i++) {
+    const auto& uri = to_consolidate[i];
+    total_size += uri.to_string().size() - base_uri_size + 1;
+
+    // If the file is a delete, add the file size to the count and the size of
+    // the size variable.
+    if (stdx::string::ends_with(
+            uri.to_string(), constants::delete_file_suffix)) {
+      RETURN_NOT_OK(storage_manager_->vfs()->file_size(uri, &file_sizes[i]));
+      total_size += file_sizes[i];
+      total_size += sizeof(storage_size_t);
+    }
   }
 
-  auto data = ss.str();
+  // Write consolidated file, URIs are relative to the array URI.
+  std::vector<uint8_t> data(total_size);
+  storage_size_t file_index = 0;
+  for (uint64_t i = 0; i < to_consolidate.size(); i++) {
+    // Add the uri.
+    const auto& uri = to_consolidate[i];
+    std::string relative_uri = uri.to_string().substr(base_uri_size) + "\n";
+    memcpy(&data[file_index], relative_uri.data(), relative_uri.size());
+    file_index += relative_uri.size();
+
+    // For deletes, read the delete condition to the output file.
+    if (stdx::string::ends_with(
+            uri.to_string(), constants::delete_file_suffix)) {
+      memcpy(&data[file_index], &file_sizes[i], sizeof(storage_size_t));
+      file_index += sizeof(storage_size_t);
+      RETURN_NOT_OK(storage_manager_->vfs()->read(
+          uri, 0, &data[file_index], file_sizes[i]));
+      file_index += file_sizes[i];
+    }
+  }
+
+  // Write the file to storage.
   URI consolidated_commits_uri =
       array_dir.get_commits_dir(write_version)
           .join_path(name.value() + constants::con_commits_file_suffix);
   RETURN_NOT_OK(storage_manager_->vfs()->write(
-      consolidated_commits_uri, data.c_str(), data.size()));
+      consolidated_commits_uri, data.data(), data.size()));
   RETURN_NOT_OK(storage_manager_->vfs()->close_file(consolidated_commits_uri));
 
   return Status::Ok();
@@ -126,18 +161,13 @@ Status CommitsConsolidator::vacuum(const char* array_name) {
   auto vfs = storage_manager_->vfs();
   auto compute_tp = storage_manager_->compute_tp();
   ArrayDirectory array_dir;
-  try {
-    array_dir = ArrayDirectory(
-        vfs,
-        compute_tp,
-        URI(array_name),
-        0,
-        utils::time::timestamp_now_ms(),
-        false,
-        ArrayDirectoryMode::COMMITS);
-  } catch (const std::logic_error& le) {
-    return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
-  }
+  array_dir = ArrayDirectory(
+      vfs,
+      compute_tp,
+      URI(array_name),
+      0,
+      utils::time::timestamp_now_ms(),
+      ArrayDirectoryMode::COMMITS);
 
   const auto& commits_uris_to_vacuum = array_dir.commit_uris_to_vacuum();
   const auto& consolidated_commits_uris_to_vacuum =
