@@ -40,6 +40,7 @@
 #include "tiledb/sm/query/query.h"
 #include "tiledb/sm/stats/global_stats.h"
 #include "tiledb/sm/storage_manager/storage_manager.h"
+#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <iostream>
 #include <sstream>
@@ -73,6 +74,8 @@ Status FragmentConsolidator::consolidate(
     uint32_t key_length) {
   auto timer_se = stats_->start_timer("consolidate_frags");
 
+  check_array_uri(array_name);
+
   // Open array for reading
   auto array_for_reads{
       make_shared<Array>(HERE(), URI(array_name), storage_manager_)};
@@ -99,12 +102,12 @@ Status FragmentConsolidator::consolidate(
   // which dense fragments are consolidatable.
   FragmentInfo fragment_info(URI(array_name), storage_manager_);
   auto st = fragment_info.load(
+      array_for_reads->array_directory(),
       config_.timestamp_start_,
       config_.timestamp_end_,
       encryption_type,
       encryption_key,
-      key_length,
-      array_for_reads->array_schema_latest().dense());
+      key_length);
   if (!st.ok()) {
     array_for_reads->close();
     array_for_writes->close();
@@ -132,8 +135,9 @@ Status FragmentConsolidator::consolidate(
     }
 
     // Check if there is anything to consolidate
-    if (to_consolidate.size() <= 1)
+    if (to_consolidate.size() <= 1) {
       break;
+    }
 
     // Consolidate the selected fragments
     URI new_fragment_uri;
@@ -199,18 +203,19 @@ Status FragmentConsolidator::consolidate_fragments(
   }
 
   // Check if there is anything to consolidate
-  if (fragment_uris.size() <= 1)
+  if (fragment_uris.size() <= 1) {
     return Status::Ok();
+  }
 
   // Get all fragment info
   FragmentInfo fragment_info(URI(array_name), storage_manager_);
   auto st = fragment_info.load(
+      array_for_reads->array_directory(),
       0,
       utils::time::timestamp_now_ms(),
       encryption_type,
       encryption_key,
-      key_length,
-      false);
+      key_length);
   if (!st.ok()) {
     array_for_reads->close();
     array_for_writes->close();
@@ -282,21 +287,16 @@ Status FragmentConsolidator::vacuum(const char* array_name) {
     return logger_->status(Status_StorageManagerError(
         "Cannot vacuum fragments; Array name cannot be null"));
 
-  // Get the fragment URIs and vacuum file URIs to be vacuum
+  // Get the fragment URIs and vacuum file URIs to be vacuumed
   auto vfs = storage_manager_->vfs();
   auto compute_tp = storage_manager_->compute_tp();
-  ArrayDirectory array_dir;
-  try {
-    array_dir = ArrayDirectory(
-        vfs,
-        compute_tp,
-        URI(array_name),
-        0,
-        std::numeric_limits<uint64_t>::max(),
-        ArrayDirectoryMode::VACUUM_FRAGMENTS);
-  } catch (const std::logic_error& le) {
-    return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
-  }
+  auto array_dir = ArrayDirectory(
+      vfs,
+      compute_tp,
+      URI(array_name),
+      0,
+      std::numeric_limits<uint64_t>::max(),
+      ArrayDirectoryMode::VACUUM_FRAGMENTS);
 
   auto filtered_fragment_uris = array_dir.filtered_fragment_uris(true);
   const auto& fragment_uris_to_vacuum =
@@ -309,26 +309,8 @@ Status FragmentConsolidator::vacuum(const char* array_name) {
       filtered_fragment_uris.fragment_vac_uris_to_vacuum();
 
   if (commit_uris_to_ignore.size() > 0) {
-    // Write an ignore file to ensure consolidated WRT files still work
-    auto&& [st1, name] = array_dir.compute_new_fragment_name(
-        commit_uris_to_ignore.front(),
-        commit_uris_to_ignore.back(),
-        constants::format_version);
-    RETURN_NOT_OK(st1);
-
-    // Write URIs, relative to the array URI.
-    std::stringstream ss;
-    auto base_uri_size = array_dir.uri().to_string().size();
-    for (const auto& uri : commit_uris_to_ignore) {
-      ss << uri.to_string().substr(base_uri_size) << "\n";
-    }
-
-    auto data = ss.str();
-    URI ignore_file_uri =
-        array_dir.get_commits_dir(constants::format_version)
-            .join_path(name.value() + constants::ignore_file_suffix);
-    RETURN_NOT_OK(vfs->write(ignore_file_uri, data.c_str(), data.size()));
-    RETURN_NOT_OK(vfs->close_file(ignore_file_uri));
+    RETURN_NOT_OK(storage_manager_->write_commit_ignore_file(
+        array_dir, commit_uris_to_ignore));
   }
 
   // Delete the commit files
@@ -399,7 +381,7 @@ Status FragmentConsolidator::consolidate_internal(
     const std::vector<TimestampedURI>& to_consolidate,
     const NDRange& union_non_empty_domains,
     URI* new_fragment_uri) {
-  auto timer_se = stats_->start_timer("consolidate_main");
+  auto timer_se = stats_->start_timer("consolidate_internal");
 
   RETURN_NOT_OK(array_for_reads->load_fragments(to_consolidate));
 
@@ -410,6 +392,34 @@ Status FragmentConsolidator::consolidate_internal(
   // Get schema
   const auto& array_schema = array_for_reads->array_schema_latest();
 
+  // If there are any delete conditions coming after the first fragment or if
+  // there are any fragments with delete meta, the new fragment will include
+  // delete meta.
+  if (!config_.purge_deleted_cells_ &&
+      array_schema.write_version() >= constants::deletes_min_version) {
+    // Get the first fragment first timestamp.
+    std::pair<uint64_t, uint64_t> timestamps;
+    RETURN_NOT_OK(
+        utils::parse::get_timestamp_range(to_consolidate[0].uri_, &timestamps));
+
+    for (auto& delete_tile_location :
+         array_for_reads->array_directory().delete_tiles_location()) {
+      if (delete_tile_location.timestamp() >= timestamps.first) {
+        config_.with_delete_meta_ = true;
+        break;
+      }
+    }
+
+    if (!config_.with_delete_meta_) {
+      for (auto& frag_md : array_for_reads->fragment_metadata()) {
+        if (frag_md->has_delete_meta()) {
+          config_.with_delete_meta_ = true;
+          break;
+        }
+      }
+    }
+  }
+
   // Prepare buffers
   std::vector<ByteVec> buffers;
   std::vector<uint64_t> buffer_sizes;
@@ -432,12 +442,15 @@ Status FragmentConsolidator::consolidate_internal(
   }
 
   // Get the vacuum URI
-  auto&& [st_vac_uri, vac_uri] =
-      array_for_reads->array_directory().get_vaccum_uri(*new_fragment_uri);
-  if (!st_vac_uri.ok()) {
+  URI vac_uri;
+  try {
+    vac_uri =
+        array_for_reads->array_directory().get_vacuum_uri(*new_fragment_uri);
+  } catch (std::exception& e) {
     tdb_delete(query_r);
     tdb_delete(query_w);
-    return st_vac_uri;
+    std::throw_with_nested(
+        std::logic_error("[FragmentConsolidator::consolidate_internal] "));
   }
 
   // Read from one array and write to the other
@@ -462,7 +475,7 @@ Status FragmentConsolidator::consolidate_internal(
   }
 
   // Write vacuum file
-  st = write_vacuum_file(vac_uri.value(), to_consolidate);
+  st = write_vacuum_file(vac_uri, to_consolidate);
   if (!st.ok()) {
     tdb_delete(query_r);
     tdb_delete(query_w);
@@ -534,6 +547,12 @@ Status FragmentConsolidator::create_buffers(
     buffer_num++;
   }
 
+  // Adding buffers for delete meta, one for timestamp and one for condition
+  // index.
+  if (config_.with_delete_meta_) {
+    buffer_num += 2;
+  }
+
   // Create buffers
   buffers->resize(buffer_num);
   buffer_sizes->resize(buffer_num);
@@ -582,21 +601,33 @@ Status FragmentConsolidator::create_queries(
   auto last = (*query_r)->last_fragment_uri();
 
   auto write_version = array_for_reads->array_schema_latest().write_version();
-  auto&& [st, name] =
+  auto&& [st, fragment_name] =
       array_for_reads->array_directory().compute_new_fragment_name(
           first, last, write_version);
   RETURN_NOT_OK(st);
-  auto frag_uri =
-      array_for_reads->array_directory().get_fragments_dir(write_version);
-  *new_fragment_uri = frag_uri.join_path(name.value());
 
   // Create write query
-  *query_w =
-      tdb_new(Query, storage_manager_, array_for_writes, *new_fragment_uri);
+  *query_w = tdb_new(Query, storage_manager_, array_for_writes, fragment_name);
   RETURN_NOT_OK((*query_w)->set_layout(Layout::GLOBAL_ORDER));
   RETURN_NOT_OK((*query_w)->disable_checks_consolidation());
-  if (array_for_reads->array_schema_latest().dense())
+  if (array_for_reads->array_schema_latest().dense()) {
     RETURN_NOT_OK((*query_w)->set_subarray_unsafe(subarray));
+  }
+
+  // Set the processed conditions on new fragment.
+  const auto& delete_tiles_location =
+      (*query_r)->array()->array_directory().delete_tiles_location();
+  std::vector<std::string> processed_conditions;
+  processed_conditions.reserve(delete_tiles_location.size());
+  for (auto& location : delete_tiles_location) {
+    processed_conditions.emplace_back(location.condition_marker());
+  }
+  (*query_w)->set_processed_conditions(processed_conditions);
+
+  // Set the URI for the new fragment.
+  auto frag_uri =
+      array_for_reads->array_directory().get_fragments_dir(write_version);
+  *new_fragment_uri = frag_uri.join_path(fragment_name.value());
 
   return Status::Ok();
 }
@@ -794,6 +825,19 @@ Status FragmentConsolidator::set_query_buffers(
     ++bid;
   }
 
+  if (config_.with_delete_meta_ && !dense) {
+    RETURN_NOT_OK(query->set_data_buffer(
+        constants::delete_timestamps,
+        (void*)&(*buffers)[bid][0],
+        &(*buffer_sizes)[bid]));
+    ++bid;
+    RETURN_NOT_OK(query->set_data_buffer(
+        constants::delete_condition_index,
+        (void*)&(*buffers)[bid][0],
+        &(*buffer_sizes)[bid]));
+    ++bid;
+  }
+
   return Status::Ok();
 }
 
@@ -845,6 +889,7 @@ Status FragmentConsolidator::set_config(const Config* config) {
   assert(found);
   config_.use_refactored_reader_ = reader.compare("refactored") == 0;
   config_.with_timestamps_ = true;
+  config_.with_delete_meta_ = false;
 
   // Sanity checks
   if (config_.min_frags_ > config_.max_frags_)
