@@ -37,6 +37,7 @@
 #include <cassert>
 #include "tiledb/common/status.h"
 #include "tiledb/sm/misc/constants.h"
+#include "tiledb/stdx/utility/to_underlying.h"
 
 using namespace tiledb::common;
 
@@ -50,7 +51,7 @@ namespace sm {
  */
 enum class FilterType : uint8_t {
 #define TILEDB_FILTER_TYPE_ENUM(id) id
-#include "tiledb/sm/c_api/tiledb_enum.h"
+#include "tiledb/api/c_api/filter/filter_api_enum.h"
 #undef TILEDB_FILTER_TYPE_ENUM
   /** Internally used encryption with AES-256-GCM. */
   INTERNAL_FILTER_AES_256_GCM = 11,
@@ -89,6 +90,8 @@ inline const std::string& filter_type_str(FilterType filter_type) {
       return constants::filter_dictionary_str;
     case FilterType::FILTER_SCALE_FLOAT:
       return constants::filter_scale_float_str;
+    case FilterType::FILTER_XOR:
+      return constants::filter_xor_str;
     default:
       return constants::empty_str;
   }
@@ -127,12 +130,27 @@ inline Status filter_type_enum(
     *filter_type = FilterType::FILTER_DICTIONARY;
   else if (filter_type_str == constants::filter_scale_float_str)
     *filter_type = FilterType::FILTER_SCALE_FLOAT;
+  else if (filter_type_str == constants::filter_xor_str)
+    *filter_type = FilterType::FILTER_XOR;
   else {
     return Status_Error("Invalid FilterType " + filter_type_str);
   }
   return Status::Ok();
 }
 
+/** Throws error if the input Filtertype enum is not between 0 and 16. */
+inline void ensure_filtertype_is_valid(uint8_t type) {
+  if (type > 16) {
+    throw std::runtime_error(
+        "Invalid FilterType (" + std::to_string(type) + ")");
+  }
+}
+
+/** Throws error if the input Filtertype's enum is not between 0 and 16. */
+inline void ensure_filtertype_is_valid(FilterType type) {
+  ensure_filtertype_is_valid(::stdx::to_underlying(type));
+}
+
 }  // namespace sm
 }  // namespace tiledb
 
