@@ -31,7 +31,7 @@
  *
  */
 
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 #include "../filter_create.h"
 
 #include "../bit_width_reduction_filter.h"
@@ -70,18 +70,16 @@ TEST_CASE(
   buffer_offset<uint32_t, 1>(p) = sizeof(uint32_t);  // metadata_length
   buffer_offset<uint32_t, 5>(p) = max_window_size0;
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 
   uint32_t max_window_size1 = 0;
   REQUIRE(
-      filter1.value()
-          ->get_option(FilterOption::BIT_WIDTH_MAX_WINDOW, &max_window_size1)
+      filter1->get_option(FilterOption::BIT_WIDTH_MAX_WINDOW, &max_window_size1)
           .ok());
   CHECK(max_window_size0 == max_window_size1);
 }
@@ -95,13 +93,12 @@ TEST_CASE(
   buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
   buffer_offset<uint32_t, 1>(p) = 0;  // metadata_length
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 }
 
 TEST_CASE(
@@ -113,13 +110,12 @@ TEST_CASE(
   buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
   buffer_offset<uint32_t, 1>(p) = 0;  // metadata_length
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 }
 
 TEST_CASE(
@@ -131,13 +127,12 @@ TEST_CASE(
   buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
   buffer_offset<uint32_t, 1>(p) = 0;  // metadata_length
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 }
 
 TEST_CASE(
@@ -149,13 +144,12 @@ TEST_CASE(
   buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
   buffer_offset<uint32_t, 1>(p) = 0;  // metadata_length
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 }
 
 TEST_CASE(
@@ -167,13 +161,12 @@ TEST_CASE(
   buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
   buffer_offset<uint32_t, 1>(p) = 0;  // metadata_length
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 }
 
 TEST_CASE(
@@ -202,13 +195,12 @@ TEST_CASE(
         sizeof(uint8_t) + sizeof(int32_t);  // metadata_length
     buffer_offset<uint8_t, 5>(p) = static_cast<uint8_t>(compressor0);
 
-    ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-    auto&& [st_filter, filter1]{
-        FilterCreate::deserialize(&constbuffer, constants::format_version)};
-    REQUIRE(st_filter.ok());
+    Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+    auto filter1{
+        FilterCreate::deserialize(deserializer, constants::format_version)};
 
     // Check type
-    CHECK(filter1.value()->type() == filtertype0);
+    CHECK(filter1->type() == filtertype0);
   }
 
   SECTION("gzip") {
@@ -224,18 +216,16 @@ TEST_CASE(
     buffer_offset<uint8_t, 5>(p) = static_cast<uint8_t>(compressor0);
     buffer_offset<int32_t, 6>(p) = level0;
 
-    ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-    auto&& [st_filter, filter1]{
-        FilterCreate::deserialize(&constbuffer, constants::format_version)};
-    REQUIRE(st_filter.ok());
+    Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+    auto filter1{
+        FilterCreate::deserialize(deserializer, constants::format_version)};
 
     // Check type
-    CHECK(filter1.value()->type() == filtertype0);
+    CHECK(filter1->type() == filtertype0);
 
     int compressionlevel1 = 0;
     REQUIRE(
-        filter1.value()
-            ->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
+        filter1->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
             .ok());
     CHECK(level0 == compressionlevel1);
   }
@@ -254,18 +244,16 @@ TEST_CASE(
     buffer_offset<uint8_t, 5>(p) = static_cast<uint8_t>(compressor0);
     buffer_offset<int32_t, 6>(p) = level0;
 
-    ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-    auto&& [st_filter, filter1]{
-        FilterCreate::deserialize(&constbuffer, constants::format_version)};
-    REQUIRE(st_filter.ok());
+    Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+    auto filter1{
+        FilterCreate::deserialize(deserializer, constants::format_version)};
 
     // Check type
-    CHECK(filter1.value()->type() == filtertype0);
+    CHECK(filter1->type() == filtertype0);
 
     int compressionlevel1 = 0;
     REQUIRE(
-        filter1.value()
-            ->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
+        filter1->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
             .ok());
     CHECK(level0 == compressionlevel1);
   }
@@ -284,18 +272,16 @@ TEST_CASE(
     buffer_offset<uint8_t, 5>(p) = static_cast<uint8_t>(compressor0);
     buffer_offset<int32_t, 6>(p) = level0;
 
-    ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-    auto&& [st_filter, filter1]{
-        FilterCreate::deserialize(&constbuffer, constants::format_version)};
-    REQUIRE(st_filter.ok());
+    Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+    auto filter1{
+        FilterCreate::deserialize(deserializer, constants::format_version)};
 
     // Check type
-    CHECK(filter1.value()->type() == filtertype0);
+    CHECK(filter1->type() == filtertype0);
 
     int compressionlevel1 = 0;
     REQUIRE(
-        filter1.value()
-            ->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
+        filter1->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
             .ok());
     CHECK(level0 == compressionlevel1);
   }
@@ -314,18 +300,16 @@ TEST_CASE(
     buffer_offset<uint8_t, 5>(p) = static_cast<uint8_t>(compressor0);
     buffer_offset<int32_t, 6>(p) = level0;
 
-    ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-    auto&& [st_filter, filter1]{
-        FilterCreate::deserialize(&constbuffer, constants::format_version)};
-    REQUIRE(st_filter.ok());
+    Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+    auto filter1{
+        FilterCreate::deserialize(deserializer, constants::format_version)};
 
     // Check type
-    CHECK(filter1.value()->type() == filtertype0);
+    CHECK(filter1->type() == filtertype0);
 
     int compressionlevel1 = 0;
     REQUIRE(
-        filter1.value()
-            ->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
+        filter1->get_option(FilterOption::COMPRESSION_LEVEL, &compressionlevel1)
             .ok());
     CHECK(level0 == compressionlevel1);
   }
@@ -339,13 +323,12 @@ TEST_CASE("Filter: Test noop filter deserialization", "[filter][noop]") {
   buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
   buffer_offset<uint32_t, 1>(p) = 0;  // metadata_length
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 }
 
 TEST_CASE(
@@ -358,16 +341,15 @@ TEST_CASE(
   buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
   buffer_offset<uint32_t, 1>(p) = sizeof(uint32_t);  // metadata_length
   buffer_offset<uint32_t, 5>(p) = max_window_size0;
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
 
   // Check type
-  CHECK(filter1.value()->type() == filtertype0);
+  CHECK(filter1->type() == filtertype0);
 
   uint32_t max_window_size1 = 0;
-  REQUIRE(filter1.value()
+  REQUIRE(filter1
               ->get_option(
                   FilterOption::POSITIVE_DELTA_MAX_WINDOW, &max_window_size1)
               .ok());
@@ -393,26 +375,36 @@ TEST_CASE(
   buffer_offset<double, 13>(p) = offset0;
   buffer_offset<uint64_t, 21>(p) = byte_width0;
 
-  ConstBuffer constbuffer(&serialized_buffer, sizeof(serialized_buffer));
-  auto&& [st_filter, filter1]{
-      FilterCreate::deserialize(&constbuffer, constants::format_version)};
-  REQUIRE(st_filter.ok());
-  CHECK(filter1.value()->type() == filtertype0);
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
+  CHECK(filter1->type() == filtertype0);
   double scale1 = 0.0;
-  REQUIRE(filter1.value()
-              ->get_option(FilterOption::SCALE_FLOAT_FACTOR, &scale1)
-              .ok());
+  REQUIRE(filter1->get_option(FilterOption::SCALE_FLOAT_FACTOR, &scale1).ok());
   CHECK(scale0 == scale1);
 
   double offset1 = 0.0;
-  REQUIRE(filter1.value()
-              ->get_option(FilterOption::SCALE_FLOAT_OFFSET, &offset1)
-              .ok());
+  REQUIRE(filter1->get_option(FilterOption::SCALE_FLOAT_OFFSET, &offset1).ok());
   CHECK(offset0 == offset1);
 
   uint64_t byte_width1 = 0;
-  REQUIRE(filter1.value()
-              ->get_option(FilterOption::SCALE_FLOAT_BYTEWIDTH, &byte_width1)
+  REQUIRE(filter1->get_option(FilterOption::SCALE_FLOAT_BYTEWIDTH, &byte_width1)
               .ok());
   CHECK(byte_width0 == byte_width1);
+}
+
+TEST_CASE("Filter: Test XOR filter deserialization", "[filter][xor]") {
+  Buffer buffer;
+  FilterType filtertype0 = FilterType::FILTER_XOR;
+  char serialized_buffer[5];
+  char* p = &serialized_buffer[0];
+  buffer_offset<uint8_t, 0>(p) = static_cast<uint8_t>(filtertype0);
+  buffer_offset<uint32_t, 1>(p) = 0;  // metadata_length
+
+  Deserializer deserializer(&serialized_buffer, sizeof(serialized_buffer));
+  auto filter1{
+      FilterCreate::deserialize(deserializer, constants::format_version)};
+
+  // Check type
+  CHECK(filter1->type() == filtertype0);
 }
\ No newline at end of file
