@@ -57,6 +57,13 @@ using namespace tiledb::sm::stats;
 namespace tiledb {
 namespace sm {
 
+class ReaderStatusException : public StatusException {
+ public:
+  explicit ReaderStatusException(const std::string& message)
+      : StatusException("Reader", message) {
+  }
+};
+
 namespace {
 /**
  * If the given iterator points to an "invalid" element, advance it until the
@@ -103,7 +110,8 @@ Reader::Reader(
     std::unordered_map<std::string, QueryBuffer>& buffers,
     Subarray& subarray,
     Layout layout,
-    QueryCondition& condition)
+    QueryCondition& condition,
+    bool skip_checks_serialization)
     : ReaderBase(
           stats,
           logger->clone("Reader", ++logger_id_),
@@ -114,6 +122,32 @@ Reader::Reader(
           subarray,
           layout,
           condition) {
+  // Sanity checks
+  if (storage_manager_ == nullptr) {
+    throw ReaderStatusException(
+        "Cannot initialize reader; Storage manager not set");
+  }
+
+  if (!skip_checks_serialization && buffers_.empty()) {
+    throw ReaderStatusException("Cannot initialize reader; Buffers not set");
+  }
+
+  if (!skip_checks_serialization && array_schema_.dense() &&
+      !subarray_.is_set()) {
+    throw ReaderStatusException(
+        "Cannot initialize reader; Dense reads must have a subarray set");
+  }
+
+  // Check subarray
+  check_subarray();
+
+  // Initialize the read state
+  init_read_state();
+
+  // Check the validity buffer sizes. This must be performed
+  // after `init_read_state` to ensure we have set the
+  // member state correctly from the config.
+  check_validity_buffer_sizes();
 }
 
 /* ****************************** */
@@ -133,32 +167,6 @@ QueryStatusDetailsReason Reader::status_incomplete_reason() const {
                         QueryStatusDetailsReason::REASON_NONE;
 }
 
-Status Reader::init() {
-  // Sanity checks
-  if (storage_manager_ == nullptr)
-    return logger_->status(Status_ReaderError(
-        "Cannot initialize reader; Storage manager not set"));
-  if (buffers_.empty())
-    return logger_->status(
-        Status_ReaderError("Cannot initialize reader; Buffers not set"));
-  if (array_schema_.dense() && !subarray_.is_set())
-    return logger_->status(Status_ReaderError(
-        "Cannot initialize reader; Dense reads must have a subarray set"));
-
-  // Check subarray
-  RETURN_NOT_OK(check_subarray());
-
-  // Initialize the read state
-  RETURN_NOT_OK(init_read_state());
-
-  // Check the validity buffer sizes. This must be performed
-  // after `init_read_state` to ensure we have set the
-  // member state correctly from the config.
-  RETURN_NOT_OK(check_validity_buffer_sizes());
-
-  return Status::Ok();
-}
-
 Status Reader::initialize_memory_budget() {
   return Status::Ok();
 }
@@ -265,19 +273,73 @@ void Reader::reset() {
 /*         PRIVATE METHODS        */
 /* ****************************** */
 
+Status Reader::load_initial_data() {
+  if (initial_data_loaded_) {
+    return Status::Ok();
+  }
+
+  // Load delete conditions.
+  auto&& [st, delete_conditions] =
+      storage_manager_->load_delete_conditions(*array_);
+  RETURN_CANCEL_OR_ERROR(st);
+  delete_conditions_ = std::move(*delete_conditions);
+
+  // Set timestamps variables
+  user_requested_timestamps_ = buffers_.count(constants::timestamps) != 0 ||
+                               delete_conditions_.size() > 0;
+  const bool partial_consol_fragment_overlap =
+      partial_consolidated_fragment_overlap();
+  use_timestamps_ = partial_consol_fragment_overlap ||
+                    !array_schema_.allows_dups() || user_requested_timestamps_;
+
+  // Add partial overlap condition for timestamps, if required.
+  if (partial_consol_fragment_overlap) {
+    RETURN_NOT_OK(add_partial_overlap_condition());
+  }
+
+  // Legacy reader always uses timestamped conditions. As we process all cell
+  // slabs at once and they could be from fragments consolidated with
+  // timestamps, there is not way to know if we need the regular condition
+  // or the timestamped condition. This reader will have worst performance
+  // for deletes.
+  RETURN_CANCEL_OR_ERROR(generate_timestamped_conditions());
+
+  // Make a list of dim/attr that will be loaded for query condition.
+  if (!condition_.empty()) {
+    qc_loaded_attr_names_set_.merge(condition_.field_names());
+  }
+  for (auto delete_condition : delete_conditions_) {
+    qc_loaded_attr_names_set_.merge(delete_condition.field_names());
+  }
+
+  // Add delete timestamps condition.
+  RETURN_NOT_OK(add_delete_timestamps_condition());
+
+  // Load processed conditions from fragment metadata.
+  if (delete_conditions_.size() > 0) {
+    load_processed_conditions();
+  }
+
+  initial_data_loaded_ = true;
+
+  return Status::Ok();
+}
+
 Status Reader::apply_query_condition(
     std::vector<ResultCellSlab>& result_cell_slabs,
     std::vector<ResultTile*>& result_tiles,
     Subarray& subarray,
     uint64_t stride) {
-  if (condition_.empty() || result_cell_slabs.empty())
+  if ((condition_.empty() && delete_conditions_.empty()) ||
+      result_cell_slabs.empty()) {
     return Status::Ok();
+  }
 
   // To evaluate the query condition, we need to read tiles for the
   // attributes used in the query condition. Build a map of attribute
   // names to read.
   std::unordered_map<std::string, ProcessTileFlags> names;
-  for (const auto& condition_name : condition_.field_names()) {
+  for (const auto& condition_name : qc_loaded_attr_names_set_) {
     names[condition_name] = ProcessTileFlag::READ;
   }
 
@@ -292,7 +354,26 @@ Status Reader::apply_query_condition(
   if (stride == UINT64_MAX)
     stride = 1;
 
-  RETURN_NOT_OK(condition_.apply(array_schema_, result_cell_slabs, stride));
+  if (!condition_.empty()) {
+    RETURN_NOT_OK(condition_.apply(
+        array_schema_, fragment_metadata_, result_cell_slabs, stride));
+  }
+
+  // Apply delete conditions.
+  if (!delete_conditions_.empty()) {
+    for (uint64_t i = 0; i < delete_conditions_.size(); i++) {
+      // For legacy, always run the timestamped condition.
+      RETURN_NOT_OK(timestamped_delete_conditions_[i].apply(
+          array_schema_, fragment_metadata_, result_cell_slabs, stride));
+    }
+  }
+
+  // Process the delete timestamps condition, if required.
+  if (!delete_timestamps_condition_.empty()) {
+    // Remove cells with partial overlap from the bitmap.
+    RETURN_NOT_OK(delete_timestamps_condition_.apply(
+        array_schema_, fragment_metadata_, result_cell_slabs, stride));
+  }
 
   return Status::Ok();
 }
@@ -393,11 +474,9 @@ Status Reader::compute_range_result_coords(
 
     // Apply partial overlap condition, if required.
     const auto frag_meta = fragment_metadata_[tile->frag_idx()];
-    const bool partial_overlap = frag_meta->partial_time_overlap(
-        array_->timestamp_start(), array_->timestamp_end_opened_at());
-    if (frag_meta->has_timestamps() && partial_overlap) {
+    if (process_partial_timestamps(*frag_meta)) {
       RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<uint8_t>(
-          *(frag_meta->array_schema().get()), *tile, result_bitmap, nullptr));
+          *(frag_meta->array_schema().get()), *tile, result_bitmap));
     }
 
     // Gather results
@@ -765,7 +844,7 @@ Status Reader::copy_attribute_values(
     // flag it to copy because we have already preloaded the offsets
     // and read the tiles in `apply_query_condition`.
     ProcessTileFlags flags = ProcessTileFlag::COPY;
-    if (condition_.field_names().count(name) == 0) {
+    if (qc_loaded_attr_names_set_.count(name) == 0) {
       flags |= ProcessTileFlag::READ;
     }
 
@@ -872,12 +951,6 @@ void Reader::compute_fixed_cs_partitions(
   }
 }
 
-uint64_t ReaderBase::offsets_bytesize() const {
-  assert(offsets_bitsize_ == 32 || offsets_bitsize_ == 64);
-  return offsets_bitsize_ == 32 ? sizeof(uint32_t) :
-                                  constants::cell_var_offset_size;
-}
-
 Status Reader::copy_partitioned_fixed_cells(
     const size_t partition_idx,
     const std::string* const name,
@@ -893,9 +966,11 @@ Status Reader::copy_partitioned_fixed_cells(
   auto buffer = (unsigned char*)it->second.buffer_;
   auto buffer_validity = (unsigned char*)it->second.validity_vector_.buffer();
   auto cell_size = array_schema_.cell_size(*name);
+  const auto is_attr = array_schema_.is_attr(*name);
+  const auto is_dim = array_schema_.is_dim(*name);
   ByteVecValue fill_value;
   uint8_t fill_value_validity = 0;
-  if (array_schema_.is_attr(*name)) {
+  if (is_attr) {
     fill_value = array_schema_.attribute(*name)->fill_value();
     fill_value_validity = array_schema_.attribute(*name)->fill_value_validity();
   }
@@ -917,13 +992,15 @@ Status Reader::copy_partitioned_fixed_cells(
 
     // First we check if this is an older (pre TileDB 2.0) array with zipped
     // coordinates and the user has requested split buffer if so we should
-    // proceed to copying the tile If not, and there is no tile or the tile is
+    // proceed to copying the tile. If not, and there is no tile or the tile is
     // empty for the field then this is a read of an older fragment in schema
     // evolution. In that case we want to set the field to fill values for this
     // for this tile.
     const bool split_buffer_for_zipped_coords =
-        array_schema_.is_dim(*name) && cs.tile_->stores_zipped_coords();
-    if ((cs.tile_ == nullptr || cs.tile_->tile_tuple(*name) == nullptr) &&
+        is_dim && cs.tile_->stores_zipped_coords();
+    const bool field_not_present =
+        (is_dim || is_attr) && cs.tile_->tile_tuple(*name) == nullptr;
+    if ((cs.tile_ == nullptr || field_not_present) &&
         !split_buffer_for_zipped_coords) {  // Empty range or attributed added
                                             // in schema evolution
       auto bytes_to_copy = cs_length * cell_size;
@@ -1127,8 +1204,8 @@ Status Reader::compute_var_cell_destinations(
     uint64_t tile_var_size = 0;
     if (cs.tile_ != nullptr && cs.tile_->tile_tuple(name) != nullptr) {
       const auto tile_tuple = cs.tile_->tile_tuple(name);
-      const auto& tile = std::get<0>(*tile_tuple);
-      const auto& tile_var = std::get<1>(*tile_tuple);
+      const auto& tile = tile_tuple->fixed_tile();
+      const auto& tile_var = tile_tuple->var_tile();
 
       // Get the internal buffer to the offset values.
       tile_offsets = (uint64_t*)tile.data();
@@ -1235,9 +1312,9 @@ Status Reader::copy_partitioned_var_cells(
     uint64_t tile_cell_num = 0;
     if (cs.tile_ != nullptr && cs.tile_->tile_tuple(*name) != nullptr) {
       const auto tile_tuple = cs.tile_->tile_tuple(*name);
-      Tile* const tile = &std::get<0>(*tile_tuple);
-      tile_var = &std::get<1>(*tile_tuple);
-      tile_validity = &std::get<2>(*tile_tuple);
+      Tile* const tile = &tile_tuple->fixed_tile();
+      tile_var = &tile_tuple->var_tile();
+      tile_validity = nullable ? &tile_tuple->validity_tile() : nullptr;
 
       // Get the internal buffer to the offset values.
       tile_offsets = (uint64_t*)tile->data();
@@ -1589,6 +1666,18 @@ Status Reader::compute_result_coords(
         unfilter_tiles(constants::timestamps, tmp_result_tiles));
   }
 
+  // Read and unfilter delete timestamps.
+  {
+    std::vector<std::string> delete_timestamps = {constants::delete_timestamps};
+    RETURN_CANCEL_OR_ERROR(load_tile_offsets(
+        read_state_.partitioner_.subarray(), delete_timestamps));
+
+    RETURN_CANCEL_OR_ERROR(
+        read_attribute_tiles(delete_timestamps, tmp_result_tiles));
+    RETURN_CANCEL_OR_ERROR(
+        unfilter_tiles(constants::delete_timestamps, tmp_result_tiles));
+  }
+
   // Compute the read coordinates for all fragments for each subarray range.
   std::vector<std::vector<ResultCoords>> range_result_coords;
   RETURN_CANCEL_OR_ERROR(compute_range_result_coords(
@@ -1748,7 +1837,7 @@ Status Reader::get_all_result_coords(
       partial_overlap) {
     std::vector<uint8_t> result_bitmap(coords_num, 1);
     RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<uint8_t>(
-        *(frag_meta->array_schema().get()), *tile, result_bitmap, nullptr));
+        *(frag_meta->array_schema().get()), *tile, result_bitmap));
 
     for (uint64_t i = 0; i < coords_num; ++i) {
       if (result_bitmap[i]) {
@@ -1773,43 +1862,59 @@ bool Reader::has_separate_coords() const {
   return false;
 }
 
-Status Reader::init_read_state() {
+void Reader::init_read_state() {
   auto timer_se = stats_->start_timer("init_state");
 
   // Check subarray
-  if (subarray_.layout() == Layout::GLOBAL_ORDER && subarray_.range_num() != 1)
-    return logger_->status(
-        Status_ReaderError("Cannot initialize read "
-                           "state; Multi-range "
-                           "subarrays do not "
-                           "support global order"));
+  if (subarray_.layout() == Layout::GLOBAL_ORDER &&
+      subarray_.range_num() != 1) {
+    throw ReaderStatusException(
+        "Cannot initialize read state; Multi-range subarrays do not support "
+        "global order");
+  }
 
   // Get config
   bool found = false;
   uint64_t memory_budget = 0;
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("sm.memory_budget", &memory_budget, &found));
+  if (!config_.get<uint64_t>("sm.memory_budget", &memory_budget, &found).ok()) {
+    throw ReaderStatusException("Cannot get setting");
+  }
   assert(found);
+
   uint64_t memory_budget_var = 0;
-  RETURN_NOT_OK(config_.get<uint64_t>(
-      "sm.memory_budget_var", &memory_budget_var, &found));
+  if (!config_.get<uint64_t>("sm.memory_budget_var", &memory_budget_var, &found)
+           .ok()) {
+    throw ReaderStatusException("Cannot get setting");
+  }
   assert(found);
+
   offsets_format_mode_ = config_.get("sm.var_offsets.mode", &found);
   assert(found);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    return logger_->status(
-        Status_ReaderError("Cannot initialize reader; Unsupported offsets "
-                           "format in configuration"));
+    throw ReaderStatusException(
+        "Cannot initialize reader; Unsupported offsets"
+        " format in configuration");
+  }
+
+  if (!config_
+           .get<bool>(
+               "sm.var_offsets.extra_element", &offsets_extra_element_, &found)
+           .ok()) {
+    throw ReaderStatusException("Cannot get setting");
   }
-  RETURN_NOT_OK(config_.get<bool>(
-      "sm.var_offsets.extra_element", &offsets_extra_element_, &found));
   assert(found);
-  RETURN_NOT_OK(config_.get<uint32_t>(
-      "sm.var_offsets.bitsize", &offsets_bitsize_, &found));
+
+  if (!config_
+           .get<uint32_t>("sm.var_offsets.bitsize", &offsets_bitsize_, &found)
+           .ok()) {
+    throw ReaderStatusException("Cannot get setting");
+  }
+  assert(found);
+
   if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    return logger_->status(
-        Status_ReaderError("Cannot initialize reader; Unsupported offsets "
-                           "bitsize in configuration"));
+    throw ReaderStatusException(
+        "Cannot initialize reader; Unsupported offsets"
+        " bitsize in configuration");
   }
   assert(found);
 
@@ -1839,22 +1944,37 @@ Status Reader::init_read_state() {
     auto buffer_validity_size = a.second.validity_vector_.buffer_size();
     if (!array_schema_.var_size(attr_name)) {
       if (!array_schema_.is_nullable(attr_name)) {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget(
-            attr_name.c_str(), *buffer_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget(attr_name.c_str(), *buffer_size)
+                 .ok()) {
+          throw ReaderStatusException("Cannot set result budget");
+        }
       } else {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget_nullable(
-            attr_name.c_str(), *buffer_size, *buffer_validity_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget_nullable(
+                     attr_name.c_str(), *buffer_size, *buffer_validity_size)
+                 .ok()) {
+          throw ReaderStatusException("Cannot set result budget");
+        }
       }
     } else {
       if (!array_schema_.is_nullable(attr_name)) {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget(
-            attr_name.c_str(), *buffer_size, *buffer_var_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget(
+                     attr_name.c_str(), *buffer_size, *buffer_var_size)
+                 .ok()) {
+          throw ReaderStatusException("Cannot set result budget");
+        }
       } else {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget_nullable(
-            attr_name.c_str(),
-            *buffer_size,
-            *buffer_var_size,
-            *buffer_validity_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget_nullable(
+                     attr_name.c_str(),
+                     *buffer_size,
+                     *buffer_var_size,
+                     *buffer_validity_size)
+                 .ok()) {
+          throw ReaderStatusException("Cannot set result budget");
+        }
       }
     }
   }
@@ -1862,8 +1982,6 @@ Status Reader::init_read_state() {
   read_state_.unsplittable_ = false;
   read_state_.overflowed_ = false;
   read_state_.initialized_ = true;
-
-  return Status::Ok();
 }
 
 Status Reader::sort_result_coords(
@@ -1905,28 +2023,20 @@ Status Reader::sort_result_coords(
 }
 
 Status Reader::sparse_read() {
+  // Load initial data.
+  RETURN_NOT_OK(load_initial_data());
+
   // Compute result coordinates from the sparse fragments
   // `sparse_result_tiles` will hold all the relevant result tiles of
   // sparse fragments
   std::vector<ResultCoords> result_coords;
   std::vector<ResultTile> sparse_result_tiles;
 
-  // Set timestamps variables
-  user_requested_timestamps_ = buffers_.count(constants::timestamps) != 0;
-  const bool partial_consol_fragment_overlap =
-      partial_consolidated_fragment_overlap();
-  use_timestamps_ = partial_consol_fragment_overlap ||
-                    !array_schema_.allows_dups() || user_requested_timestamps_;
-
-  // Add partial overlap condition for timestamps, if required.
-  if (partial_consol_fragment_overlap) {
-    RETURN_NOT_OK(add_partial_overlap_condition());
-  }
-
   RETURN_NOT_OK(compute_result_coords(sparse_result_tiles, result_coords));
   std::vector<ResultTile*> result_tiles;
-  for (auto& srt : sparse_result_tiles)
+  for (auto& srt : sparse_result_tiles) {
     result_tiles.push_back(&srt);
+  }
 
   // Compute result cell slabs
   std::vector<ResultCellSlab> result_cell_slabs;
@@ -2108,5 +2218,225 @@ bool Reader::belong_to_single_fragment(
   return true;
 }
 
+template <class T>
+tuple<Status, optional<bool>> Reader::fill_dense_coords(
+    const Subarray& subarray) {
+  auto timer_se = stats_->start_timer("fill_dense_coords");
+
+  // Reading coordinates with a query condition is currently unsupported.
+  // Query conditions mutate the result cell slabs to filter attributes.
+  // This path does not use result cell slabs, which will fill coordinates
+  // for cells that should be filtered out.
+  if (!condition_.empty()) {
+    return {logger_->status(Status_ReaderError(
+                "Cannot read dense coordinates; dense coordinate "
+                "reads are unsupported with a query condition")),
+            nullopt};
+  }
+
+  // Prepare buffers
+  std::vector<unsigned> dim_idx;
+  std::vector<QueryBuffer*> buffers;
+  auto coords_it = buffers_.find(constants::coords);
+  auto dim_num = array_schema_.dim_num();
+  if (coords_it != buffers_.end()) {
+    buffers.emplace_back(&(coords_it->second));
+    dim_idx.emplace_back(dim_num);
+  } else {
+    for (unsigned d = 0; d < dim_num; ++d) {
+      const auto dim{array_schema_.dimension_ptr(d)};
+      auto it = buffers_.find(dim->name());
+      if (it != buffers_.end()) {
+        buffers.emplace_back(&(it->second));
+        dim_idx.emplace_back(d);
+      }
+    }
+  }
+  std::vector<uint64_t> offsets(buffers.size(), 0);
+
+  bool overflowed = false;
+  if (layout_ == Layout::GLOBAL_ORDER) {
+    auto&& [st, of] =
+        fill_dense_coords_global<T>(subarray, dim_idx, buffers, offsets);
+    RETURN_NOT_OK_TUPLE(st, std::nullopt);
+    overflowed = *of;
+  } else {
+    assert(layout_ == Layout::ROW_MAJOR || layout_ == Layout::COL_MAJOR);
+    auto&& [st, of] =
+        fill_dense_coords_row_col<T>(subarray, dim_idx, buffers, offsets);
+    RETURN_NOT_OK_TUPLE(st, std::nullopt);
+    overflowed = *of;
+  }
+
+  // Update buffer sizes
+  for (size_t i = 0; i < buffers.size(); ++i)
+    *(buffers[i]->buffer_size_) = offsets[i];
+
+  return {Status::Ok(), overflowed};
+}
+
+template <class T>
+tuple<Status, optional<bool>> Reader::fill_dense_coords_global(
+    const Subarray& subarray,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) {
+  auto tile_coords = subarray.tile_coords();
+  auto cell_order = array_schema_.cell_order();
+
+  bool overflowed = false;
+  for (const auto& tc : tile_coords) {
+    auto tile_subarray = subarray.crop_to_tile((const T*)&tc[0], cell_order);
+    auto&& [st, of] =
+        fill_dense_coords_row_col<T>(tile_subarray, dim_idx, buffers, offsets);
+    RETURN_NOT_OK_TUPLE(st, std::nullopt);
+    overflowed |= *of;
+  }
+
+  return {Status::Ok(), overflowed};
+}
+
+template <class T>
+tuple<Status, optional<bool>> Reader::fill_dense_coords_row_col(
+    const Subarray& subarray,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) {
+  auto cell_order = array_schema_.cell_order();
+  auto dim_num = array_schema_.dim_num();
+
+  // Iterate over all coordinates, retrieved in cell slabs
+  CellSlabIter<T> iter(&subarray);
+  RETURN_CANCEL_OR_ERROR_TUPLE(iter.begin());
+  while (!iter.end()) {
+    auto cell_slab = iter.cell_slab();
+    auto coords_num = cell_slab.length_;
+
+    // Check for overflow
+    for (size_t i = 0; i < buffers.size(); ++i) {
+      auto idx = (dim_idx[i] == dim_num) ? 0 : dim_idx[i];
+      auto coord_size{array_schema_.domain().dimension_ptr(idx)->coord_size()};
+      coord_size = (dim_idx[i] == dim_num) ? coord_size * dim_num : coord_size;
+      auto buff_size = *(buffers[i]->buffer_size_);
+      auto offset = offsets[i];
+      if (coords_num * coord_size + offset > buff_size) {
+        return {Status::Ok(), true};
+      }
+    }
+
+    // Copy slab
+    if (layout_ == Layout::ROW_MAJOR ||
+        (layout_ == Layout::GLOBAL_ORDER && cell_order == Layout::ROW_MAJOR))
+      fill_dense_coords_row_slab(
+          &cell_slab.coords_[0], coords_num, dim_idx, buffers, offsets);
+    else
+      fill_dense_coords_col_slab(
+          &cell_slab.coords_[0], coords_num, dim_idx, buffers, offsets);
+
+    ++iter;
+  }
+
+  return {Status::Ok(), false};
+}
+
+template <class T>
+void Reader::fill_dense_coords_row_slab(
+    const T* start,
+    uint64_t num,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) const {
+  // For easy reference
+  auto dim_num = array_schema_.dim_num();
+
+  // Special zipped coordinates
+  if (dim_idx.size() == 1 && dim_idx[0] == dim_num) {
+    auto c_buff = (char*)buffers[0]->buffer_;
+    auto offset = &offsets[0];
+
+    // Fill coordinates
+    for (uint64_t i = 0; i < num; ++i) {
+      // First dim-1 dimensions are copied as they are
+      if (dim_num > 1) {
+        auto bytes_to_copy = (dim_num - 1) * sizeof(T);
+        std::memcpy(c_buff + *offset, start, bytes_to_copy);
+        *offset += bytes_to_copy;
+      }
+
+      // Last dimension is incremented by `i`
+      auto new_coord = start[dim_num - 1] + i;
+      std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+      *offset += sizeof(T);
+    }
+  } else {  // Set of separate coordinate buffers
+    for (uint64_t i = 0; i < num; ++i) {
+      for (size_t b = 0; b < buffers.size(); ++b) {
+        auto c_buff = (char*)buffers[b]->buffer_;
+        auto offset = &offsets[b];
+
+        // First dim-1 dimensions are copied as they are
+        if (dim_num > 1 && dim_idx[b] < dim_num - 1) {
+          std::memcpy(c_buff + *offset, &start[dim_idx[b]], sizeof(T));
+          *offset += sizeof(T);
+        } else {
+          // Last dimension is incremented by `i`
+          auto new_coord = start[dim_num - 1] + i;
+          std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+          *offset += sizeof(T);
+        }
+      }
+    }
+  }
+}
+
+template <class T>
+void Reader::fill_dense_coords_col_slab(
+    const T* start,
+    uint64_t num,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) const {
+  // For easy reference
+  auto dim_num = array_schema_.dim_num();
+
+  // Special zipped coordinates
+  if (dim_idx.size() == 1 && dim_idx[0] == dim_num) {
+    auto c_buff = (char*)buffers[0]->buffer_;
+    auto offset = &offsets[0];
+
+    // Fill coordinates
+    for (uint64_t i = 0; i < num; ++i) {
+      // First dimension is incremented by `i`
+      auto new_coord = start[0] + i;
+      std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+      *offset += sizeof(T);
+
+      // Last dim-1 dimensions are copied as they are
+      if (dim_num > 1) {
+        auto bytes_to_copy = (dim_num - 1) * sizeof(T);
+        std::memcpy(c_buff + *offset, &start[1], bytes_to_copy);
+        *offset += bytes_to_copy;
+      }
+    }
+  } else {  // Separate coordinate buffers
+    for (uint64_t i = 0; i < num; ++i) {
+      for (size_t b = 0; b < buffers.size(); ++b) {
+        auto c_buff = (char*)buffers[b]->buffer_;
+        auto offset = &offsets[b];
+
+        // First dimension is incremented by `i`
+        if (dim_idx[b] == 0) {
+          auto new_coord = start[0] + i;
+          std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+          *offset += sizeof(T);
+        } else {  // Last dim-1 dimensions are copied as they are
+          std::memcpy(c_buff + *offset, &start[dim_idx[b]], sizeof(T));
+          *offset += sizeof(T);
+        }
+      }
+    }
+  }
+}
+
 }  // namespace sm
 }  // namespace tiledb
