@@ -73,7 +73,8 @@ class Reader : public ReaderBase, public IQueryStrategy {
       std::unordered_map<std::string, QueryBuffer>& buffers,
       Subarray& subarray,
       Layout layout,
-      QueryCondition& condition);
+      QueryCondition& condition,
+      bool skip_checks_serialization = false);
 
   /** Destructor. */
   ~Reader() = default;
@@ -98,9 +99,6 @@ class Reader : public ReaderBase, public IQueryStrategy {
   /** Returns the status details reason. */
   QueryStatusDetailsReason status_incomplete_reason() const;
 
-  /** Initializes the reader. */
-  Status init();
-
   /** Initialize the memory budget variables. */
   Status initialize_memory_budget();
 
@@ -238,6 +236,13 @@ class Reader : public ReaderBase, public IQueryStrategy {
   /*           PRIVATE METHODS         */
   /* ********************************* */
 
+  /**
+   * Load data used for sparse reads.
+   *
+   * @return Status.
+   */
+  Status load_initial_data();
+
   /**
    * Applies the query condition, `condition_`, to filter cell indexes
    * within `result_cell_slabs`. This mutates `result_cell_slabs`.
@@ -613,7 +618,7 @@ class Reader : public ReaderBase, public IQueryStrategy {
   bool has_separate_coords() const;
 
   /** Initializes the read state. */
-  Status init_read_state();
+  void init_read_state();
 
   /**
    * Sorts the input result coordinates according to the subarray layout.
@@ -704,6 +709,124 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return timestamp.
    */
   uint64_t get_timestamp(const ResultCoords& rc) const;
+
+  /**
+   * Fills the coordinate buffer with coordinates. Applicable only to dense
+   * arrays when the user explicitly requests the coordinates to be
+   * materialized.
+   *
+   * @tparam T The domain type.
+   * @param subarray The input subarray.
+   * @return Status, overflowed
+   */
+  template <class T>
+  tuple<Status, optional<bool>> fill_dense_coords(const Subarray& subarray);
+
+  /**
+   * Fills the coordinate buffers with coordinates. Applicable only to dense
+   * arrays when the user explicitly requests the coordinates to be
+   * materialized. Also applicable only to global order.
+   *
+   * @tparam T The domain type.
+   * @param subarray The input subarray.
+   * @param dim_idx The dimension indices of the corresponding `buffers`.
+   *     For the special zipped coordinates, `dim_idx`, `buffers` and `offsets`
+   *     contain a single element and `dim_idx` contains `dim_num` as
+   *     the dimension index.
+   * @param buffers The buffers to copy from. It could be the special
+   *     zipped coordinates or separate coordinate buffers.
+   * @param offsets The offsets that will be used eventually to update
+   *     the buffer sizes, determining the useful results written in
+   *     the buffers.
+   * @return Status, overflowed.
+   */
+  template <class T>
+  tuple<Status, optional<bool>> fill_dense_coords_global(
+      const Subarray& subarray,
+      const std::vector<unsigned>& dim_idx,
+      const std::vector<QueryBuffer*>& buffers,
+      std::vector<uint64_t>& offsets);
+
+  /**
+   * Fills the coordinate buffers with coordinates. Applicable only to dense
+   * arrays when the user explicitly requests the coordinates to be
+   * materialized. Also applicable only to row-/col-major order.
+   *
+   * @tparam T The domain type.
+   * @param subarray The input subarray.
+   * @param dim_idx The dimension indices of the corresponding `buffers`.
+   *     For the special zipped coordinates, `dim_idx`, `buffers` and `offsets`
+   *     contain a single element and `dim_idx` contains `dim_num` as
+   *     the dimension index.
+   * @param buffers The buffers to copy from. It could be the special
+   *     zipped coordinates or separate coordinate buffers.
+   * @param offsets The offsets that will be used eventually to update
+   *     the buffer sizes, determining the useful results written in
+   *     the buffers.
+   * @return Status, overflowed.
+   */
+  template <class T>
+  tuple<Status, optional<bool>> fill_dense_coords_row_col(
+      const Subarray& subarray,
+      const std::vector<unsigned>& dim_idx,
+      const std::vector<QueryBuffer*>& buffers,
+      std::vector<uint64_t>& offsets);
+
+  /**
+   * Fills coordinates in the input buffers for a particular cell slab,
+   * following a row-major layout. For instance, if the starting coordinate are
+   * [3, 1] and the number of coords to be written is 3, this function will
+   * write to the input buffer (starting at the input offset) coordinates
+   * [3, 1], [3, 2], and [3, 3].
+   *
+   * @tparam T The domain type.
+   * @param start The starting coordinates in the slab.
+   * @param num The number of coords to be written.
+   * @param dim_idx The dimension indices of the corresponding `buffers`.
+   *     For the special zipped coordinates, `dim_idx`, `buffers` and `offsets`
+   *     contain a single element and `dim_idx` contains `dim_num` as
+   *     the dimension index.
+   * @param buffers The buffers to copy from. It could be the special
+   *     zipped coordinates or separate coordinate buffers.
+   * @param offsets The offsets that will be used eventually to update
+   *     the buffer sizes, determining the useful results written in
+   *     the buffers.
+   */
+  template <class T>
+  void fill_dense_coords_row_slab(
+      const T* start,
+      uint64_t num,
+      const std::vector<unsigned>& dim_idx,
+      const std::vector<QueryBuffer*>& buffers,
+      std::vector<uint64_t>& offsets) const;
+
+  /**
+   * Fills coordinates in the input buffers for a particular cell slab,
+   * following a col-major layout. For instance, if the starting coordinate are
+   * [3, 1] and the number of coords to be written is 3, this function will
+   * write to the input buffer (starting at the input offset) coordinates
+   * [4, 1], [5, 1], and [6, 1].
+   *
+   * @tparam T The domain type.
+   * @param start The starting coordinates in the slab.
+   * @param num The number of coords to be written.
+   * @param dim_idx The dimension indices of the corresponding `buffers`.
+   *     For the special zipped coordinates, `dim_idx`, `buffers` and `offsets`
+   *     contain a single element and `dim_idx` contains `dim_num` as
+   *     the dimension index.
+   * @param buffers The buffers to copy from. It could be the special
+   *     zipped coordinates or separate coordinate buffers.
+   * @param offsets The offsets that will be used eventually to update
+   *     the buffer sizes, determining the useful results written in
+   *     the buffers.
+   */
+  template <class T>
+  void fill_dense_coords_col_slab(
+      const T* start,
+      uint64_t num,
+      const std::vector<unsigned>& dim_idx,
+      const std::vector<QueryBuffer*>& buffers,
+      std::vector<uint64_t>& offsets) const;
 };
 
 }  // namespace sm
