@@ -38,6 +38,7 @@
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/utils.h"
+#include "tiledb/sm/query/legacy/cell_slab_iter.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/query/readers/dense_reader.h"
 #include "tiledb/sm/query/readers/result_tile.h"
@@ -45,6 +46,8 @@
 #include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/subarray/subarray.h"
 
+#include <numeric>
+
 using namespace tiledb;
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
@@ -52,6 +55,13 @@ using namespace tiledb::sm::stats;
 namespace tiledb {
 namespace sm {
 
+class DenseReaderStatusException : public StatusException {
+ public:
+  explicit DenseReaderStatusException(const std::string& message)
+      : StatusException("DenseReader", message) {
+  }
+};
+
 /* ****************************** */
 /*          CONSTRUCTORS          */
 /* ****************************** */
@@ -65,7 +75,8 @@ DenseReader::DenseReader(
     std::unordered_map<std::string, QueryBuffer>& buffers,
     Subarray& subarray,
     Layout layout,
-    QueryCondition& condition)
+    QueryCondition& condition,
+    bool skip_checks_serialization)
     : ReaderBase(
           stats,
           logger->clone("DenseReader", ++logger_id_),
@@ -77,6 +88,40 @@ DenseReader::DenseReader(
           layout,
           condition) {
   elements_mode_ = false;
+
+  // Sanity checks.
+  if (storage_manager_ == nullptr) {
+    throw DenseReaderStatusException(
+        "Cannot initialize dense reader; Storage manager not set");
+  }
+
+  if (!skip_checks_serialization && buffers_.empty()) {
+    throw DenseReaderStatusException(
+        "Cannot initialize dense reader; Buffers not set");
+  }
+
+  if (!skip_checks_serialization && !subarray_.is_set()) {
+    throw DenseReaderStatusException(
+        "Cannot initialize reader; Dense reads must have a subarray set");
+  }
+
+  // Check subarray.
+  check_subarray();
+
+  // Initialize the read state.
+  init_read_state();
+
+  // Check the validity buffer sizes.
+  check_validity_buffer_sizes();
+
+  bool found = false;
+  uint64_t tile_cache_size = 0;
+  if (!config_.get<uint64_t>("sm.tile_cache_size", &tile_cache_size, &found)
+           .ok()) {
+    throw DenseReaderStatusException("Cannot get setting");
+  }
+  assert(found);
+  disable_cache_ = tile_cache_size == 0;
 }
 
 /* ****************************** */
@@ -92,37 +137,6 @@ QueryStatusDetailsReason DenseReader::status_incomplete_reason() const {
                         QueryStatusDetailsReason::REASON_NONE;
 }
 
-Status DenseReader::init() {
-  // Sanity checks.
-  if (storage_manager_ == nullptr)
-    return LOG_STATUS(Status_DenseReaderError(
-        "Cannot initialize dense reader; Storage manager not set"));
-  if (buffers_.empty())
-    return LOG_STATUS(Status_DenseReaderError(
-        "Cannot initialize dense reader; Buffers not set"));
-  if (!subarray_.is_set())
-    return LOG_STATUS(Status_ReaderError(
-        "Cannot initialize reader; Dense reads must have a subarray set"));
-
-  // Check subarray.
-  RETURN_NOT_OK(check_subarray());
-
-  // Initialize the read state.
-  RETURN_NOT_OK(init_read_state());
-
-  // Check the validity buffer sizes.
-  RETURN_NOT_OK(check_validity_buffer_sizes());
-
-  bool found = false;
-  uint64_t tile_cache_size = 0;
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("sm.tile_cache_size", &tile_cache_size, &found));
-  assert(found);
-  disable_cache_ = tile_cache_size == 0;
-
-  return Status::Ok();
-}
-
 Status DenseReader::initialize_memory_budget() {
   return Status::Ok();
 }
@@ -391,6 +405,13 @@ Status DenseReader::dense_read() {
       num_range_threads);
   RETURN_CANCEL_OR_ERROR(st);
 
+  // For `qc_coords_mode` just fill in the coordinates and skip attribute
+  // processing.
+  if (qc_coords_mode_) {
+    fill_dense_coords<DimType>(subarray, qc_result);
+    return Status::Ok();
+  }
+
   // Process attributes.
   std::vector<std::string> to_read(1);
   for (auto& name : names) {
@@ -417,64 +438,85 @@ Status DenseReader::dense_read() {
     clear_tiles(name, result_tiles);
   }
 
-  if (read_state_.overflowed_)
-    Status::Ok();
+  if (read_state_.overflowed_) {
+    return Status::Ok();
+  }
 
   // Fill coordinates if the user requested them.
   if (!read_state_.overflowed_ && has_coords()) {
-    auto&& [st, overflowed] = fill_dense_coords<DimType>(subarray);
-    RETURN_CANCEL_OR_ERROR(st);
-    read_state_.overflowed_ = *overflowed;
+    fill_dense_coords<DimType>(subarray, nullopt);
   }
 
   return Status::Ok();
 }
 
-Status DenseReader::init_read_state() {
+void DenseReader::init_read_state() {
   auto timer_se = stats_->start_timer("init_state");
 
   // Check subarray.
-  if (subarray_.layout() == Layout::GLOBAL_ORDER && subarray_.range_num() != 1)
-    return LOG_STATUS(
-        Status_ReaderError("Cannot initialize read "
-                           "state; Multi-range "
-                           "subarrays do not "
-                           "support global order"));
+  if (subarray_.layout() == Layout::GLOBAL_ORDER &&
+      subarray_.range_num() != 1) {
+    throw DenseReaderStatusException(
+        "Cannot initialize read state; Multi-range subarrays do not support "
+        "global order");
+  }
 
   // Get config values.
   bool found = false;
   uint64_t memory_budget = 0;
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("sm.memory_budget", &memory_budget, &found));
+  if (!config_.get<uint64_t>("sm.memory_budget", &memory_budget, &found).ok()) {
+    throw DenseReaderStatusException("Cannot get setting");
+  }
   assert(found);
 
   uint64_t memory_budget_var = 0;
-  RETURN_NOT_OK(config_.get<uint64_t>(
-      "sm.memory_budget_var", &memory_budget_var, &found));
+  if (!config_.get<uint64_t>("sm.memory_budget_var", &memory_budget_var, &found)
+           .ok()) {
+    throw DenseReaderStatusException("Cannot get setting");
+  }
   assert(found);
 
   offsets_format_mode_ = config_.get("sm.var_offsets.mode", &found);
   assert(found);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    return LOG_STATUS(
-        Status_ReaderError("Cannot initialize reader; Unsupported offsets "
-                           "format in configuration"));
+    throw DenseReaderStatusException(
+        "Cannot initialize reader; Unsupported offsets format in "
+        "configuration");
   }
   elements_mode_ = offsets_format_mode_ == "elements";
 
-  RETURN_NOT_OK(config_.get<bool>(
-      "sm.var_offsets.extra_element", &offsets_extra_element_, &found));
-  assert(found);
-
-  RETURN_NOT_OK(config_.get<uint32_t>(
-      "sm.var_offsets.bitsize", &offsets_bitsize_, &found));
-  if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    return LOG_STATUS(
-        Status_ReaderError("Cannot initialize reader; Unsupported offsets "
-                           "bitsize in configuration"));
+  if (!config_
+           .get<bool>(
+               "sm.var_offsets.extra_element", &offsets_extra_element_, &found)
+           .ok()) {
+    throw DenseReaderStatusException("Cannot get setting");
   }
   assert(found);
 
+  if (!config_
+           .get<uint32_t>("sm.var_offsets.bitsize", &offsets_bitsize_, &found)
+           .ok()) {
+    throw DenseReaderStatusException("Cannot get setting");
+  }
+  if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
+    throw DenseReaderStatusException(
+        "Cannot initialize reader; Unsupported offsets bitsize in "
+        "configuration");
+  }
+  assert(found);
+
+  if (!config_
+           .get<bool>("sm.query.dense.qc_coords_mode", &qc_coords_mode_, &found)
+           .ok()) {
+    throw DenseReaderStatusException("Cannot get setting");
+  }
+  assert(found);
+
+  if (qc_coords_mode_ && condition_.empty()) {
+    throw DenseReaderStatusException(
+        "sm.query.dense.qc_coords_mode requires a query condition");
+  }
+
   // Consider the validity memory budget to be identical to `sm.memory_budget`
   // because the validity vector is currently a bytemap. When converted to a
   // bitmap, this can be budgeted as `sm.memory_budget` / 8.
@@ -493,7 +535,7 @@ Status DenseReader::init_read_state() {
   read_state_.overflowed_ = false;
   read_state_.unsplittable_ = false;
 
-  // Set result size budget.
+  // Set result size budget
   for (const auto& a : buffers_) {
     auto attr_name = a.first;
     auto buffer_size = a.second.buffer_size_;
@@ -501,22 +543,37 @@ Status DenseReader::init_read_state() {
     auto buffer_validity_size = a.second.validity_vector_.buffer_size();
     if (!array_schema_.var_size(attr_name)) {
       if (!array_schema_.is_nullable(attr_name)) {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget(
-            attr_name.c_str(), *buffer_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget(attr_name.c_str(), *buffer_size)
+                 .ok()) {
+          throw DenseReaderStatusException("Cannot set result budget");
+        }
       } else {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget_nullable(
-            attr_name.c_str(), *buffer_size, *buffer_validity_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget_nullable(
+                     attr_name.c_str(), *buffer_size, *buffer_validity_size)
+                 .ok()) {
+          throw DenseReaderStatusException("Cannot set result budget");
+        }
       }
     } else {
       if (!array_schema_.is_nullable(attr_name)) {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget(
-            attr_name.c_str(), *buffer_size, *buffer_var_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget(
+                     attr_name.c_str(), *buffer_size, *buffer_var_size)
+                 .ok()) {
+          throw DenseReaderStatusException("Cannot set result budget");
+        }
       } else {
-        RETURN_NOT_OK(read_state_.partitioner_.set_result_budget_nullable(
-            attr_name.c_str(),
-            *buffer_size,
-            *buffer_var_size,
-            *buffer_validity_size));
+        if (!read_state_.partitioner_
+                 .set_result_budget_nullable(
+                     attr_name.c_str(),
+                     *buffer_size,
+                     *buffer_var_size,
+                     *buffer_validity_size)
+                 .ok()) {
+          throw DenseReaderStatusException("Cannot set result budget");
+        }
       }
     }
   }
@@ -524,8 +581,6 @@ Status DenseReader::init_read_state() {
   read_state_.unsplittable_ = false;
   read_state_.overflowed_ = false;
   read_state_.initialized_ = true;
-
-  return Status::Ok();
 }
 
 /** Apply the query condition. */
@@ -685,8 +740,9 @@ uint64_t DenseReader::fix_offsets_buffer(
   // Set the output offset buffer sizes.
   *buffers_[name].buffer_size_ = cell_num * sizeof(OffType);
 
-  if (nullable)
+  if (nullable) {
     *buffers_[name].validity_vector_.buffer_size() = cell_num;
+  }
 
   // Return the buffer size.
   return offset;
@@ -962,8 +1018,7 @@ Status DenseReader::copy_fixed_tiles(
         auto dest_validity_ptr = dst_val_buf + cell_offset;
 
         // Get the tile buffers.
-        const Tile* const tile = &std::get<0>(*tile_tuples[fd]);
-        const Tile* const tile_nullable = &std::get<2>(*tile_tuples[fd]);
+        const auto& tile = tile_tuples[fd]->fixed_tile();
 
         auto src_offset = iter.pos_in_tile() + start * stride;
 
@@ -972,28 +1027,31 @@ Status DenseReader::copy_fixed_tiles(
         if (stride == 1) {
           std::memcpy(
               dest_ptr + cell_size * start,
-              tile->data_as<char>() + cell_size * src_offset,
+              tile.data_as<char>() + cell_size * src_offset,
               cell_size * (end - start + 1));
 
           if (nullable) {
+            const auto& tile_nullable = tile_tuples[fd]->validity_tile();
             std::memcpy(
                 dest_validity_ptr + start,
-                tile_nullable->data_as<char>() + src_offset,
+                tile_nullable.data_as<char>() + src_offset,
                 (end - start + 1));
           }
         } else {
           // Go cell by cell.
-          auto src = tile->data_as<char>() + cell_size * src_offset;
-          auto src_validity =
-              nullable ? tile_nullable->data_as<char>() + src_offset : nullptr;
+          auto src = tile.data_as<char>() + cell_size * src_offset;
           auto dest = dest_ptr + cell_size * start;
-          auto dest_validity = dest_validity_ptr + start;
           for (uint64_t i = 0; i < end - start + 1; ++i) {
             std::memcpy(dest, src, cell_size);
             src += cell_size * stride;
             dest += cell_size;
+          }
 
-            if (nullable) {
+          if (nullable) {
+            const auto& tile_nullable = tile_tuples[fd]->validity_tile();
+            auto src_validity = tile_nullable.data_as<char>() + src_offset;
+            auto dest_validity = dest_validity_ptr + start;
+            for (uint64_t i = 0; i < end - start + 1; ++i) {
               memcpy(dest_validity, src_validity, 1);
               src_validity += stride;
               dest_validity++;
@@ -1154,17 +1212,12 @@ Status DenseReader::copy_offset_tiles(
         auto dest_validity_ptr = dst_val_buf + cell_offset;
 
         // Get the tile buffers.
-        const Tile* const t_var = &std::get<1>(*tile_tuples[fd]);
+        const auto& t_var = tile_tuples[fd]->var_tile();
 
         // Setup variables for the copy.
         auto src_buff =
-            static_cast<uint64_t*>(std::get<0>(*tile_tuples[fd]).data()) +
+            static_cast<uint64_t*>(tile_tuples[fd]->fixed_tile().data()) +
             start * stride + src_cell;
-        auto src_buff_validity =
-            nullable ?
-                static_cast<uint8_t*>(std::get<2>(*tile_tuples[fd]).data()) +
-                    start + src_cell :
-                nullptr;
         auto div = elements_mode_ ? data_type_size : 1;
         auto dest = (OffType*)dest_ptr + start;
 
@@ -1174,28 +1227,29 @@ Status DenseReader::copy_offset_tiles(
         for (; i < end - start; ++i) {
           auto i_src = i * stride;
           dest[i] = (src_buff[i_src + 1] - src_buff[i_src]) / div;
-          var_data_buff[i + start] = t_var->data_as<char>() + src_buff[i_src];
-        }
-
-        if (nullable) {
-          i = 0;
-          for (; i < end - start; ++i) {
-            dest_validity_ptr[start + i] = src_buff_validity[i * stride];
-          }
+          var_data_buff[i + start] = t_var.data_as<char>() + src_buff[i_src];
         }
 
         // Copy the last value.
         if (start + src_cell + (end - start) * stride >=
             cell_num_per_tile - 1) {
-          dest[i] = (t_var->size() - src_buff[i * stride]) / div;
+          dest[i] = (t_var.size() - src_buff[i * stride]) / div;
         } else {
           auto i_src = i * stride;
           dest[i] = (src_buff[i_src + 1] - src_buff[i_src]) / div;
         }
-        var_data_buff[i + start] =
-            t_var->data_as<char>() + src_buff[i * stride];
+        var_data_buff[i + start] = t_var.data_as<char>() + src_buff[i * stride];
 
         if (nullable) {
+          auto src_buff_validity =
+              static_cast<uint8_t*>(tile_tuples[fd]->validity_tile().data()) +
+              start + src_cell;
+
+          for (i = 0; i < end - start; ++i) {
+            dest_validity_ptr[start + i] = src_buff_validity[i * stride];
+          }
+
+          // Copy last validity value.
           dest_validity_ptr[start + i] = src_buff_validity[i * stride];
         }
 
@@ -1378,5 +1432,243 @@ Status DenseReader::add_extra_offset() {
   return Status::Ok();
 }
 
+template <class T>
+void DenseReader::fill_dense_coords(
+    const Subarray& subarray, const optional<std::vector<uint8_t>> qc_results) {
+  auto timer_se = stats_->start_timer("fill_dense_coords");
+
+  // Count the number of cells.
+  auto cell_num = subarray.cell_num();
+  if (qc_results.has_value()) {
+    auto func = [](uint64_t res, const uint8_t v) { return res + v; };
+    cell_num = std::accumulate(
+        qc_results.value().begin(), qc_results.value().end(), 0, func);
+  }
+
+  // Prepare buffers.
+  std::vector<unsigned> dim_idx;
+  std::vector<QueryBuffer*> buffers;
+  auto coords_it = buffers_.find(constants::coords);
+  auto dim_num = array_schema_.dim_num();
+  if (coords_it != buffers_.end()) {
+    // Check for overflow.
+    if (coords_it->second.original_buffer_size_ <
+        cell_num * array_schema_.cell_size(constants::coords)) {
+      read_state_.overflowed_ = true;
+      return;
+    }
+    buffers.emplace_back(&(coords_it->second));
+    dim_idx.emplace_back(dim_num);
+  } else {
+    for (unsigned d = 0; d < dim_num; ++d) {
+      const auto dim{array_schema_.dimension_ptr(d)};
+      auto it = buffers_.find(dim->name());
+      if (it != buffers_.end()) {
+        // Check for overflow.
+        if (it->second.original_buffer_size_ <
+            cell_num * array_schema_.cell_size(dim->name())) {
+          read_state_.overflowed_ = true;
+          return;
+        }
+        buffers.emplace_back(&(it->second));
+        dim_idx.emplace_back(d);
+      }
+    }
+  }
+
+  uint64_t qc_results_index = 0;
+  std::vector<uint64_t> offsets(buffers.size(), 0);
+  if (layout_ == Layout::GLOBAL_ORDER) {
+    fill_dense_coords_global<T>(
+        subarray, qc_results, qc_results_index, dim_idx, buffers, offsets);
+  } else {
+    assert(layout_ == Layout::ROW_MAJOR || layout_ == Layout::COL_MAJOR);
+    fill_dense_coords_row_col<T>(
+        subarray, qc_results, qc_results_index, dim_idx, buffers, offsets);
+  }
+
+  // Update buffer sizes.
+  for (size_t i = 0; i < buffers.size(); ++i) {
+    *(buffers[i]->buffer_size_) = offsets[i];
+  }
+}
+
+template <class T>
+void DenseReader::fill_dense_coords_global(
+    const Subarray& subarray,
+    const optional<std::vector<uint8_t>> qc_results,
+    uint64_t& qc_results_index,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) {
+  auto tile_coords = subarray.tile_coords();
+  auto cell_order = array_schema_.cell_order();
+
+  for (const auto& tc : tile_coords) {
+    auto tile_subarray = subarray.crop_to_tile((const T*)&tc[0], cell_order);
+    fill_dense_coords_row_col<T>(
+        tile_subarray, qc_results, qc_results_index, dim_idx, buffers, offsets);
+  }
+}
+
+template <class T>
+void DenseReader::fill_dense_coords_row_col(
+    const Subarray& subarray,
+    const optional<std::vector<uint8_t>> qc_results,
+    uint64_t& qc_results_index,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) {
+  auto cell_order = array_schema_.cell_order();
+
+  // Iterate over all coordinates, retrieved in cell slabs.
+  CellSlabIter<T> iter(&subarray);
+  if (!iter.begin().ok()) {
+    throw DenseReaderStatusException("Cannot begin iteration");
+  }
+  while (!iter.end()) {
+    auto cell_slab = iter.cell_slab();
+    auto coords_num = cell_slab.length_;
+
+    // Copy slab.
+    if (layout_ == Layout::ROW_MAJOR ||
+        (layout_ == Layout::GLOBAL_ORDER && cell_order == Layout::ROW_MAJOR))
+      fill_dense_coords_row_slab(
+          &cell_slab.coords_[0],
+          qc_results,
+          qc_results_index,
+          coords_num,
+          dim_idx,
+          buffers,
+          offsets);
+    else
+      fill_dense_coords_col_slab(
+          &cell_slab.coords_[0],
+          qc_results,
+          qc_results_index,
+          coords_num,
+          dim_idx,
+          buffers,
+          offsets);
+
+    ++iter;
+  }
+}
+
+template <class T>
+void DenseReader::fill_dense_coords_row_slab(
+    const T* start,
+    const optional<std::vector<uint8_t>> qc_results,
+    uint64_t& qc_results_index,
+    uint64_t num,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) const {
+  // For easy reference.
+  auto dim_num = array_schema_.dim_num();
+
+  // Special zipped coordinates.
+  if (dim_idx.size() == 1 && dim_idx[0] == dim_num) {
+    auto c_buff = (char*)buffers[0]->buffer_;
+    auto offset = &offsets[0];
+
+    // Fill coordinates.
+    for (uint64_t i = 0; i < num; ++i) {
+      if (!qc_results.has_value() || qc_results.value()[qc_results_index]) {
+        // First dim-1 dimensions are copied as they are.
+        if (dim_num > 1) {
+          auto bytes_to_copy = (dim_num - 1) * sizeof(T);
+          std::memcpy(c_buff + *offset, start, bytes_to_copy);
+          *offset += bytes_to_copy;
+        }
+
+        // Last dimension is incremented by `i`.
+        auto new_coord = start[dim_num - 1] + i;
+        std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+        *offset += sizeof(T);
+      }
+      qc_results_index++;
+    }
+  } else {  // Set of separate coordinate buffers.
+    for (uint64_t i = 0; i < num; ++i) {
+      for (size_t b = 0; b < buffers.size(); ++b) {
+        if (!qc_results.has_value() || qc_results.value()[qc_results_index]) {
+          auto c_buff = (char*)buffers[b]->buffer_;
+          auto offset = &offsets[b];
+
+          // First dim-1 dimensions are copied as they are.
+          if (dim_num > 1 && dim_idx[b] < dim_num - 1) {
+            std::memcpy(c_buff + *offset, &start[dim_idx[b]], sizeof(T));
+            *offset += sizeof(T);
+          } else {
+            // Last dimension is incremented by `i`.
+            auto new_coord = start[dim_num - 1] + i;
+            std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+            *offset += sizeof(T);
+          }
+        }
+      }
+      qc_results_index++;
+    }
+  }
+}
+
+template <class T>
+void DenseReader::fill_dense_coords_col_slab(
+    const T* start,
+    const optional<std::vector<uint8_t>> qc_results,
+    uint64_t& qc_results_index,
+    uint64_t num,
+    const std::vector<unsigned>& dim_idx,
+    const std::vector<QueryBuffer*>& buffers,
+    std::vector<uint64_t>& offsets) const {
+  // For easy reference.
+  auto dim_num = array_schema_.dim_num();
+
+  // Special zipped coordinates.
+  if (dim_idx.size() == 1 && dim_idx[0] == dim_num) {
+    auto c_buff = (char*)buffers[0]->buffer_;
+    auto offset = &offsets[0];
+
+    // Fill coordinates.
+    for (uint64_t i = 0; i < num; ++i) {
+      if (!qc_results.has_value() || qc_results.value()[qc_results_index]) {
+        // First dimension is incremented by `i`.
+        auto new_coord = start[0] + i;
+        std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+        *offset += sizeof(T);
+
+        // Last dim-1 dimensions are copied as they are.
+        if (dim_num > 1) {
+          auto bytes_to_copy = (dim_num - 1) * sizeof(T);
+          std::memcpy(c_buff + *offset, &start[1], bytes_to_copy);
+          *offset += bytes_to_copy;
+        }
+      }
+      qc_results_index++;
+    }
+  } else {  // Separate coordinate buffers.
+    for (uint64_t i = 0; i < num; ++i) {
+      for (size_t b = 0; b < buffers.size(); ++b) {
+        if (!qc_results.has_value() || qc_results.value()[qc_results_index]) {
+          auto c_buff = (char*)buffers[b]->buffer_;
+          auto offset = &offsets[b];
+
+          // First dimension is incremented by `i`.
+          if (dim_idx[b] == 0) {
+            auto new_coord = start[0] + i;
+            std::memcpy(c_buff + *offset, &new_coord, sizeof(T));
+            *offset += sizeof(T);
+          } else {  // Last dim-1 dimensions are copied as they are
+            std::memcpy(c_buff + *offset, &start[dim_idx[b]], sizeof(T));
+            *offset += sizeof(T);
+          }
+        }
+      }
+      qc_results_index++;
+    }
+  }
+}
+
 }  // namespace sm
 }  // namespace tiledb
