@@ -50,6 +50,13 @@
 namespace tiledb {
 namespace sm {
 
+class SparseIndexReaderBaseStatusException : public StatusException {
+ public:
+  explicit SparseIndexReaderBaseStatusException(const std::string& message)
+      : StatusException("SparseIndexReaderBase", message) {
+  }
+};
+
 /* ****************************** */
 /*          CONSTRUCTORS          */
 /* ****************************** */
@@ -74,7 +81,6 @@ SparseIndexReaderBase::SparseIndexReaderBase(
           subarray,
           layout,
           condition)
-    , initial_data_loaded_(false)
     , memory_budget_(0)
     , array_memory_tracker_(array->memory_tracker())
     , memory_used_for_coords_total_(0)
@@ -84,7 +90,9 @@ SparseIndexReaderBase::SparseIndexReaderBase(
     , memory_budget_ratio_query_condition_(0.25)
     , memory_budget_ratio_tile_ranges_(0.1)
     , memory_budget_ratio_array_data_(0.1)
-    , buffers_full_(false) {
+    , buffers_full_(false)
+    , deletes_consolidation_no_purge_(
+          buffers_.count(constants::delete_timestamps) != 0) {
   read_state_.done_adding_result_tiles_ = false;
   disable_cache_ = true;
 }
@@ -102,45 +110,61 @@ typename SparseIndexReaderBase::ReadState* SparseIndexReaderBase::read_state() {
   return &read_state_;
 }
 
-Status SparseIndexReaderBase::init() {
+void SparseIndexReaderBase::init(bool skip_checks_serialization) {
   // Sanity checks
-  if (storage_manager_ == nullptr)
-    return logger_->status(Status_ReaderError(
+  if (storage_manager_ == nullptr) {
+    throw SparseIndexReaderBaseStatusException(
         "Cannot initialize sparse global order reader; Storage manager not "
-        "set"));
-  if (buffers_.empty())
-    return logger_->status(Status_ReaderError(
-        "Cannot initialize sparse global order reader; Buffers not set"));
+        "set");
+  }
+
+  if (!skip_checks_serialization && buffers_.empty()) {
+    throw SparseIndexReaderBaseStatusException(
+        "Cannot initialize sparse global order reader; Buffers not set");
+  }
 
   // Check subarray
-  RETURN_NOT_OK(check_subarray());
+  check_subarray();
 
   // Load offset configuration options.
   bool found = false;
   offsets_format_mode_ = config_.get("sm.var_offsets.mode", &found);
   assert(found);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    return logger_->status(
-        Status_ReaderError("Cannot initialize reader; Unsupported offsets "
-                           "format in configuration"));
+    throw SparseIndexReaderBaseStatusException(
+        "Cannot initialize reader; Unsupported offsets format in "
+        "configuration");
   }
   elements_mode_ = offsets_format_mode_ == "elements";
 
-  RETURN_NOT_OK(config_.get<bool>(
-      "sm.var_offsets.extra_element", &offsets_extra_element_, &found));
+  if (!config_
+           .get<bool>(
+               "sm.var_offsets.extra_element", &offsets_extra_element_, &found)
+           .ok()) {
+    throw SparseIndexReaderBaseStatusException("Cannot get setting");
+  }
+  assert(found);
+
+  if (!config_
+           .get<uint32_t>("sm.var_offsets.bitsize", &offsets_bitsize_, &found)
+           .ok()) {
+    throw SparseIndexReaderBaseStatusException("Cannot get setting");
+  }
   assert(found);
-  RETURN_NOT_OK(config_.get<uint32_t>(
-      "sm.var_offsets.bitsize", &offsets_bitsize_, &found));
   if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    return logger_->status(
-        Status_ReaderError("Cannot initialize reader; "
-                           "Unsupported offsets bitsize in configuration"));
+    throw SparseIndexReaderBaseStatusException(
+        "Cannot initialize reader; Unsupported offsets bitsize in "
+        "configuration");
   }
 
   // Check the validity buffer sizes.
-  RETURN_NOT_OK(check_validity_buffer_sizes());
+  check_validity_buffer_sizes();
+}
 
-  return Status::Ok();
+bool SparseIndexReaderBase::has_post_deduplication_conditions(
+    FragmentMetadata& frag_meta) {
+  return frag_meta.has_delete_meta() || !condition_.empty() ||
+         (!delete_conditions_.empty() && !deletes_consolidation_no_purge_);
 }
 
 uint64_t SparseIndexReaderBase::cells_copied(
@@ -189,8 +213,8 @@ SparseIndexReaderBase::get_coord_tiles_size(
 
   // Compute query condition tile sizes.
   uint64_t tiles_size_qc = 0;
-  if (!qc_loaded_names_.empty()) {
-    for (auto& name : qc_loaded_names_) {
+  if (!qc_loaded_attr_names_.empty()) {
+    for (auto& name : qc_loaded_attr_names_) {
       // Calculate memory consumption for this tile.
       auto&& [st, tile_size] = get_attribute_tile_size(name, f, t);
       RETURN_NOT_OK_TUPLE(st, nullopt);
@@ -202,22 +226,49 @@ SparseIndexReaderBase::get_coord_tiles_size(
 }
 
 Status SparseIndexReaderBase::load_initial_data(bool include_coords) {
-  if (initial_data_loaded_)
+  if (initial_data_loaded_) {
     return Status::Ok();
+  }
 
   auto timer_se = stats_->start_timer("load_initial_data");
   read_state_.done_adding_result_tiles_ = false;
 
+  // Load delete conditions.
+  auto&& [st, delete_conditions] =
+      storage_manager_->load_delete_conditions(*array_);
+  RETURN_CANCEL_OR_ERROR(st);
+  delete_conditions_ = std::move(*delete_conditions);
+  bool make_timestamped_conditions = need_timestamped_conditions();
+
+  if (make_timestamped_conditions) {
+    RETURN_CANCEL_OR_ERROR(generate_timestamped_conditions());
+  }
+
+  // Load processed conditions from fragment metadata.
+  if (delete_conditions_.size() > 0) {
+    load_processed_conditions();
+  }
+
   // Make a list of dim/attr that will be loaded for query condition.
-  if (!initial_data_loaded_) {
-    if (!condition_.empty()) {
-      for (auto& name : condition_.field_names()) {
-        if (!array_schema_.is_dim(name) || !include_coords) {
-          qc_loaded_names_.emplace_back(name);
-        }
+  if (!condition_.empty()) {
+    for (auto& name : condition_.field_names()) {
+      if (!array_schema_.is_dim(name) || !include_coords) {
+        qc_loaded_attr_names_set_.insert(name);
       }
     }
   }
+  for (auto delete_condition : delete_conditions_) {
+    for (auto& name : delete_condition.field_names()) {
+      if (!array_schema_.is_dim(name) || !include_coords) {
+        qc_loaded_attr_names_set_.insert(name);
+      }
+    }
+  }
+
+  qc_loaded_attr_names_.reserve(qc_loaded_attr_names_set_.size());
+  for (auto& name : qc_loaded_attr_names_set_) {
+    qc_loaded_attr_names_.emplace_back(name);
+  }
 
   // For easy reference.
   auto fragment_num = fragment_metadata_.size();
@@ -301,13 +352,17 @@ Status SparseIndexReaderBase::load_initial_data(bool include_coords) {
   const bool partial_consol_fragment_overlap =
       partial_consolidated_fragment_overlap();
   use_timestamps_ = partial_consol_fragment_overlap ||
-                    !array_schema_.allows_dups() || user_requested_timestamps_;
+                    !array_schema_.allows_dups() ||
+                    user_requested_timestamps_ || make_timestamped_conditions;
 
   // Add partial overlap condition, if required.
   if (partial_consol_fragment_overlap) {
     RETURN_CANCEL_OR_ERROR(add_partial_overlap_condition());
   }
 
+  // Add delete timestamps condition.
+  RETURN_CANCEL_OR_ERROR(add_delete_timestamps_condition());
+
   // Add timestamps and filter by timestamps condition if required. If the user
   // has requested timestamps the special attribute will already be in the list,
   // so don't include it again
@@ -315,6 +370,14 @@ Status SparseIndexReaderBase::load_initial_data(bool include_coords) {
     attr_tile_offsets_to_load.emplace_back(constants::timestamps);
   }
 
+  // Load delete timestamps, always.
+  attr_tile_offsets_to_load.emplace_back(constants::delete_timestamps);
+
+  // Load delete condition marker hashes for delete consolidation.
+  if (deletes_consolidation_no_purge_) {
+    attr_tile_offsets_to_load.emplace_back(constants::delete_condition_index);
+  }
+
   // Load tile offsets and var sizes for attributes.
   RETURN_CANCEL_OR_ERROR(load_tile_var_sizes(subarray_, var_size_to_load));
   RETURN_CANCEL_OR_ERROR(
@@ -329,10 +392,6 @@ Status SparseIndexReaderBase::read_and_unfilter_coords(
     bool include_coords, const std::vector<ResultTile*>& result_tiles) {
   auto timer_se = stats_->start_timer("read_and_unfilter_coords");
 
-  // Not including coords or no query condition, exit.
-  if (!include_coords && condition_.empty() && !use_timestamps_)
-    return Status::Ok();
-
   if (subarray_.is_set() || include_coords) {
     // Read and unfilter zipped coordinate tiles. Note that
     // this will ignore fragments with a version >= 5.
@@ -349,41 +408,34 @@ Status SparseIndexReaderBase::read_and_unfilter_coords(
     }
   }
 
+  // Compute attributes to load.
+  std::vector<std::string> attr_to_load;
+  attr_to_load.reserve(
+      1 + deletes_consolidation_no_purge_ + use_timestamps_ +
+      qc_loaded_attr_names_.size());
   if (use_timestamps_) {
-    std::vector<std::string> timestamps_names = {constants::timestamps};
-    RETURN_CANCEL_OR_ERROR(
-        read_attribute_tiles(timestamps_names, result_tiles));
-    RETURN_CANCEL_OR_ERROR(unfilter_tiles(constants::timestamps, result_tiles));
+    attr_to_load.emplace_back(constants::timestamps);
   }
+  attr_to_load.emplace_back(constants::delete_timestamps);
+  if (deletes_consolidation_no_purge_) {
+    attr_to_load.emplace_back(constants::delete_condition_index);
+  }
+  std::copy(
+      qc_loaded_attr_names_.begin(),
+      qc_loaded_attr_names_.end(),
+      std::back_inserter(attr_to_load));
 
-  if (!condition_.empty()) {
-    // Read and unfilter tiles for query condition.
-    RETURN_CANCEL_OR_ERROR(
-        read_attribute_tiles(qc_loaded_names_, result_tiles));
+  // Read and unfilter attribute tiles.
+  RETURN_CANCEL_OR_ERROR(read_attribute_tiles(attr_to_load, result_tiles));
 
-    for (const auto& name : qc_loaded_names_) {
-      RETURN_CANCEL_OR_ERROR(unfilter_tiles(name, result_tiles));
-    }
+  for (const auto& name : attr_to_load) {
+    RETURN_CANCEL_OR_ERROR(unfilter_tiles(name, result_tiles));
   }
 
   logger_->debug("Done reading and unfiltering coords tiles");
   return Status::Ok();
 }
 
-template <class BitmapType>
-Status SparseIndexReaderBase::allocate_tile_bitmap(
-    ResultTileWithBitmap<BitmapType>* rt) {
-  // Bitmap was already computed for this tile.
-  if (rt->bitmap_result_num_ != std::numeric_limits<uint64_t>::max()) {
-    return Status::Ok();
-  }
-
-  auto cell_num = fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
-  rt->bitmap_.resize(cell_num, 1);
-
-  return Status::Ok();
-}
-
 template <class BitmapType>
 Status SparseIndexReaderBase::compute_tile_bitmaps(
     std::vector<ResultTile*>& result_tiles) {
@@ -407,7 +459,7 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
     num_range_threads = 1 + ((num_threads - 1) / result_tiles.size());
   }
 
-  // Perforance runs have shown that running multiple parallel_for's has a
+  // Performance runs have shown that running multiple parallel_for's has a
   // measurable performance impact. So only pre-allocate tile bitmaps if we
   // are going to run multiple range threads.
   if (num_range_threads != 1) {
@@ -417,8 +469,9 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
         0,
         result_tiles.size(),
         [&](uint64_t t) {
-          return allocate_tile_bitmap(
-              (ResultTileWithBitmap<BitmapType>*)result_tiles[t]);
+          static_cast<ResultTileWithBitmap<BitmapType>*>(result_tiles[t])
+              ->alloc_bitmap();
+          return Status::Ok();
         });
     RETURN_NOT_OK_ELSE(status, logger_->status(status));
   }
@@ -435,14 +488,11 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
         auto rt = (ResultTileWithBitmap<BitmapType>*)result_tiles[t];
         auto cell_num =
             fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
-
-        // Bitmap was already computed for this tile.
-        if (rt->bitmap_result_num_ != std::numeric_limits<uint64_t>::max())
-          return Status::Ok();
+        stats_->add_counter("cell_num", cell_num);
 
         // Allocate the bitmap if not preallocated.
         if (num_range_threads == 1) {
-          RETURN_NOT_OK(allocate_tile_bitmap(rt));
+          rt->alloc_bitmap();
         }
 
         // Prevent processing past the end of the cells in case there are more
@@ -505,7 +555,7 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
                 dim_idx,
                 ranges_for_dim,
                 relevant_ranges,
-                rt->bitmap_,
+                rt->bitmap(),
                 cell_order,
                 min,
                 max));
@@ -515,7 +565,7 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
         // Only compute bitmap cells here if we are processing a single cell
         // range. If not, it will be done below.
         if (num_range_threads == 1) {
-          RETURN_NOT_OK(count_tile_bitmap_cells(rt));
+          rt->count_cells();
         }
 
         return Status::Ok();
@@ -531,8 +581,9 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
         0,
         result_tiles.size(),
         [&](uint64_t t) {
-          return count_tile_bitmap_cells(
-              (ResultTileWithBitmap<BitmapType>*)result_tiles[t]);
+          static_cast<ResultTileWithBitmap<BitmapType>*>(result_tiles[t])
+              ->count_cells();
+          return Status::Ok();
         });
     RETURN_NOT_OK_ELSE(status, logger_->status(status));
   }
@@ -541,40 +592,12 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
   return Status::Ok();
 }
 
-/** Count the number of cells in a bitmap. */
-template <class BitmapType>
-Status SparseIndexReaderBase::count_tile_bitmap_cells(
-    ResultTileWithBitmap<BitmapType>* rt) {
-  // Bitmap was already computed for this tile.
-  if (rt->bitmap_result_num_ != std::numeric_limits<uint64_t>::max()) {
-    return Status::Ok();
-  }
-
-  // Compute number of cells in this tile.
-  auto cell_num = fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
-  rt->bitmap_result_num_ = 0;
-  for (uint64_t c = 0; c < cell_num; ++c) {
-    rt->bitmap_result_num_ += rt->bitmap_[c];
-  }
-
-  const bool non_overlapping = std::is_same<BitmapType, uint8_t>::value;
-  if (non_overlapping) {
-    // Clear the bitmap, which will also signal the copy operation to copy the
-    // whole tile.
-    if (rt->bitmap_result_num_ == cell_num) {
-      rt->bitmap_.resize(0);
-    }
-  }
-
-  return Status::Ok();
-}
-
 template <class ResultTileType, class BitmapType>
 Status SparseIndexReaderBase::apply_query_condition(
     std::vector<ResultTile*>& result_tiles) {
   auto timer_se = stats_->start_timer("apply_query_condition");
 
-  if (!condition_.empty() || use_timestamps_) {
+  if (!condition_.empty() || !delete_conditions_.empty() || use_timestamps_) {
     // Process all tiles in parallel.
     auto status = parallel_for(
         storage_manager_->compute_tp(),
@@ -585,36 +608,102 @@ Status SparseIndexReaderBase::apply_query_condition(
           auto rt = static_cast<ResultTileType*>(result_tiles[t]);
           const auto frag_meta = fragment_metadata_[rt->frag_idx()];
 
-          auto cell_num = frag_meta->cell_num(rt->tile_idx());
-
           // If timestamps are present and fragment is partially included,
           // filter out tiles based on time by applying the query condition
-          if (frag_meta->has_timestamps() &&
-              frag_meta->partial_time_overlap(
-                  array_->timestamp_start(),
-                  array_->timestamp_end_opened_at())) {
-            // Full overlap in bitmap calculation, make a bitmap.
+          if (process_partial_timestamps(*frag_meta)) {
+            // Make a bitmap, if required.
             if (!rt->has_bmp()) {
-              rt->bitmap_.resize(cell_num, 1);
-              rt->bitmap_result_num_ = cell_num;
+              rt->alloc_bitmap();
             }
 
             // Remove cells with partial overlap from the bitmap.
             RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<BitmapType>(
-                *(frag_meta->array_schema().get()),
-                *rt,
-                rt->bitmap_,
-                &rt->bitmap_result_num_));
+                *(frag_meta->array_schema().get()), *rt, rt->bitmap()));
+            rt->count_cells();
           }
 
-          // Compute the result of the query condition for this tile.
+          // Make sure we have a condition bitmap if needed.
+          if (has_post_deduplication_conditions(*frag_meta) ||
+              deletes_consolidation_no_purge_) {
+            rt->ensure_bitmap_for_query_condition();
+          }
+
+          // If the fragment has delete meta, process the delete timestamps.
+          if (frag_meta->has_delete_meta() &&
+              !deletes_consolidation_no_purge_) {
+            // Remove cells deleted cells using the open timestamp.
+            RETURN_NOT_OK(delete_timestamps_condition_.apply_sparse<BitmapType>(
+                *(frag_meta->array_schema().get()),
+                *rt,
+                rt->post_dedup_bitmap()));
+            if (array_schema_.allows_dups()) {
+              rt->count_cells();
+            }
+          }
+
+          // Compute the result of the query condition for this tile
           if (!condition_.empty()) {
-            rt->ensure_bitmap_for_query_condition(cell_num);
             RETURN_NOT_OK(condition_.apply_sparse<BitmapType>(
                 *(frag_meta->array_schema().get()),
                 *rt,
-                rt->bitmap_with_qc(),
-                rt->qc_result_num_ptr()));
+                rt->post_dedup_bitmap()));
+            if (array_schema_.allows_dups()) {
+              rt->count_cells();
+            }
+          }
+
+          // Apply delete conditions.
+          if (!delete_conditions_.empty()) {
+            // Allocate delete condition idx vector if required. This vector
+            // is used to store which delete condition deleted a particular
+            // cell.
+            if (deletes_consolidation_no_purge_) {
+              rt->allocate_per_cell_delete_condition_vector();
+            }
+
+            for (uint64_t i = 0; i < delete_conditions_.size(); i++) {
+              if (!frag_meta->has_delete_meta() ||
+                  frag_meta->get_processed_conditions_set().count(
+                      delete_conditions_[i].condition_marker()) == 0) {
+                auto delete_timestamp =
+                    delete_conditions_[i].condition_timestamp();
+
+                // Check the delete condition timestamp is after the fragment
+                // start.
+                if (delete_timestamp >= frag_meta->timestamp_range().first) {
+                  // Apply timestamped condition or regular condition.
+                  if (!frag_meta->has_timestamps() ||
+                      delete_timestamp > frag_meta->timestamp_range().second) {
+                    RETURN_NOT_OK(
+                        delete_conditions_[i].apply_sparse<BitmapType>(
+                            *(frag_meta->array_schema().get()),
+                            *rt,
+                            rt->post_dedup_bitmap()));
+                  } else {
+                    RETURN_NOT_OK(timestamped_delete_conditions_[i]
+                                      .apply_sparse<BitmapType>(
+                                          *(frag_meta->array_schema().get()),
+                                          *rt,
+                                          rt->post_dedup_bitmap()));
+                  }
+
+                  if (deletes_consolidation_no_purge_) {
+                    // This is a post processing step during deletes
+                    // consolidation to set the delete condition pointer to
+                    // the current delete condition if the cells was cleared
+                    // by this condition and not any previous conditions.
+                    rt->compute_per_cell_delete_condition(
+                        &delete_conditions_[i]);
+                  } else {
+                    // Count cells is dups are allowed as the regular bitmap was
+                    // modified.
+                    if (array_schema_.allows_dups()) {
+                      rt->count_cells();
+                    }
+                  }
+                }
+              }
+            }
           }
 
           return Status::Ok();
