@@ -53,6 +53,14 @@ using namespace tiledb::sm::stats;
 namespace tiledb {
 namespace sm {
 
+class SparseUnorderedWithDupsReaderStatusException : public StatusException {
+ public:
+  explicit SparseUnorderedWithDupsReaderStatusException(
+      const std::string& message)
+      : StatusException("SparseUnorderedWithDupsReader", message) {
+  }
+};
+
 /* ****************************** */
 /*          CONSTRUCTORS          */
 /* ****************************** */
@@ -67,7 +75,8 @@ SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
     std::unordered_map<std::string, QueryBuffer>& buffers,
     Subarray& subarray,
     Layout layout,
-    QueryCondition& condition)
+    QueryCondition& condition,
+    bool skip_checks_serialization)
     : SparseIndexReaderBase(
           stats,
           logger->clone("SparseUnorderedWithDupsReader", ++logger_id_),
@@ -78,6 +87,13 @@ SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
           subarray,
           layout,
           condition) {
+  SparseIndexReaderBase::init(skip_checks_serialization);
+
+  // Initialize memory budget variables.
+  if (!initialize_memory_budget().ok()) {
+    throw SparseUnorderedWithDupsReaderStatusException(
+        "Cannot initialize memory budget");
+  }
 }
 
 /* ****************************** */
@@ -92,27 +108,19 @@ bool SparseUnorderedWithDupsReader<BitmapType>::incomplete() const {
 template <class BitmapType>
 QueryStatusDetailsReason
 SparseUnorderedWithDupsReader<BitmapType>::status_incomplete_reason() const {
-  if (array_->is_remote())
+  if (array_->is_remote()) {
     return QueryStatusDetailsReason::REASON_USER_BUFFER_SIZE;
+  }
 
-  if (!incomplete())
+  if (!incomplete()) {
     return QueryStatusDetailsReason::REASON_NONE;
+  }
 
   return result_tiles_.empty() ?
              QueryStatusDetailsReason::REASON_MEMORY_BUDGET :
              QueryStatusDetailsReason::REASON_USER_BUFFER_SIZE;
 }
 
-template <class BitmapType>
-Status SparseUnorderedWithDupsReader<BitmapType>::init() {
-  RETURN_NOT_OK(SparseIndexReaderBase::init());
-
-  // Initialize memory budget variables.
-  RETURN_NOT_OK(initialize_memory_budget());
-
-  return Status::Ok();
-}
-
 template <class BitmapType>
 Status SparseUnorderedWithDupsReader<BitmapType>::initialize_memory_budget() {
   bool found = false;
@@ -204,8 +212,9 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
     std::vector<ResultTile*> result_tiles_created;
     std::vector<ResultTile*> result_tiles_loaded;
     for (auto& result_tile : result_tiles_) {
-      if (!result_tile.coords_loaded_) {
-        result_tile.coords_loaded_ = true;
+      if (!result_tile.coords_loaded()) {
+        result_tile.set_coords_loaded();
+        ;
         result_tiles_created.emplace_back(&result_tile);
       } else {
         result_tiles_loaded.emplace_back(&result_tile);
@@ -229,8 +238,8 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
       // Clear result tiles that are not necessary anymore.
       uint64_t current = 0;
       for (uint64_t i = 0; i < result_tiles_created.size(); i++) {
-        if (((UnorderedWithDupsResultTile<uint64_t>*)result_tiles_created[i])
-                ->bitmap_result_num_ != 0) {
+        if (((ResultTileWithBitmap<uint64_t>*)result_tiles_created[i])
+                ->result_num() != 0) {
           result_tiles_created[current++] = result_tiles_created[i];
         }
       }
@@ -240,7 +249,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
       auto it = result_tiles_.begin();
       while (it != result_tiles_.end()) {
         auto f = it->frag_idx();
-        if (it->bitmap_result_num_ == 0) {
+        if (it->result_num() == 0) {
           RETURN_NOT_OK(remove_result_tile(f, it++));
         } else {
           it++;
@@ -270,12 +279,16 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
   } while (!buffers_full_ && incomplete());
 
   // Fix the output buffer sizes.
-  RETURN_NOT_OK(resize_output_buffers(cells_copied(names)));
+  const auto cells = cells_copied(names);
+  stats_->add_counter("result_num", cells);
+  RETURN_NOT_OK(resize_output_buffers(cells));
 
   if (offsets_extra_element_) {
     RETURN_NOT_OK(add_extra_offset());
   }
 
+  stats_->add_counter("ignored_tiles", ignored_tiles_.size());
+
   return Status::Ok();
 }
 
@@ -292,13 +305,16 @@ SparseUnorderedWithDupsReader<BitmapType>::get_coord_tiles_size(
           subarray_.is_set(), dim_num, f, t);
   RETURN_NOT_OK_TUPLE(st, nullopt);
 
+  auto frag_meta = fragment_metadata_[f];
+
   // Add the result tile structure size.
   tiles_sizes->first += sizeof(UnorderedWithDupsResultTile<BitmapType>);
 
-  // Add the tile bitmap size if there is a subarray or query condition set.
-  if (subarray_.is_set() || !condition_.empty()) {
-    tiles_sizes->first +=
-        fragment_metadata_[f]->cell_num(t) * sizeof(BitmapType);
+  // Add the tile bitmap size if there is a subarray or any condition to
+  // process.
+  if (subarray_.is_set() || has_post_deduplication_conditions(*frag_meta) ||
+      process_partial_timestamps(*frag_meta)) {
+    tiles_sizes->first += frag_meta->cell_num(t) * sizeof(BitmapType);
   }
 
   return {Status::Ok(), *tiles_sizes};
@@ -313,7 +329,7 @@ SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
     const unsigned f,
     const uint64_t t,
     const uint64_t last_t,
-    const ArraySchema& array_schema) {
+    const FragmentMetadata& frag_md) {
   // Calculate memory consumption for this tile.
   auto&& [st, tiles_sizes] = get_coord_tiles_size(dim_num, f, t);
   RETURN_NOT_OK_TUPLE(st, nullopt);
@@ -331,7 +347,7 @@ SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
   memory_used_qc_tiles_total_ += tiles_size_qc;
 
   // Add the result tile.
-  result_tiles_.emplace_back(f, t, array_schema);
+  result_tiles_.emplace_back(f, t, frag_md);
 
   // Are all tiles loaded for this fragment.
   if (t == last_t)
@@ -374,7 +390,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
                 f,
                 t,
                 last_t,
-                *(fragment_metadata_[f]->array_schema()).get());
+                *fragment_metadata_[f]);
             RETURN_NOT_OK(st);
 
             // Make sure we can add at least one tile.
@@ -425,7 +441,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
               f,
               t,
               tile_num - 1,
-              *(fragment_metadata_[f]->array_schema()).get());
+              *fragment_metadata_[f]);
           (void)st;
           // Make sure we can add at least one tile.
           if (*exceeded) {
@@ -511,9 +527,9 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
     uint8_t* val_buffer,
     const void** var_data) {
   // Get source buffers.
-  const auto tile_tuple = rt->tile_tuple(name);
   const auto cell_num =
       fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
+  const auto tile_tuple = rt->tile_tuple(name);
 
   // If the tile_tuple is null, this is a field added in schema
   // evolution. Use the fill value.
@@ -528,11 +544,11 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
         array_schema_.attribute(name)->fill_value().size());
     src_var_buff = array_schema_.attribute(name)->fill_value().data();
   } else {
-    const auto t = &std::get<0>(*tile_tuple);
-    const auto t_var = &std::get<1>(*tile_tuple);
-    t_var_size = t_var->size();
-    src_buff = t->template data_as<uint64_t>();
-    src_var_buff = t_var->template data_as<uint8_t>();
+    const auto& t = tile_tuple->fixed_tile();
+    const auto& t_var = tile_tuple->var_tile();
+    t_var_size = t_var.size();
+    src_buff = t.template data_as<uint64_t>();
+    src_var_buff = t_var.template data_as<uint8_t>();
   }
 
   // Process all cells. Last cell might be taken out for vectorization.
@@ -541,7 +557,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
                      src_max_pos;
   if (!use_fill_value) {
     for (uint64_t c = src_min_pos; c < end; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+      for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
         *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
         buffer++;
         *var_data = src_var_buff + src_buff[c];
@@ -551,7 +567,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
 
     // Do last cell.
     if (src_max_pos == cell_num) {
-      for (uint64_t i = 0; i < rt->bitmap_[src_max_pos - 1]; i++) {
+      for (uint64_t i = 0; i < rt->bitmap()[src_max_pos - 1]; i++) {
         *buffer =
             (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
         buffer++;
@@ -561,7 +577,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
     }
   } else {
     for (uint64_t c = src_min_pos; c < end; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+      for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
         *buffer = fill_value_size / offset_div;
         buffer++;
         *var_data = src_var_buff;
@@ -573,10 +589,10 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
   // Copy nullable values.
   if (nullable) {
     if (!use_fill_value) {
-      const auto t_val = &std::get<2>(*tile_tuple);
-      const auto src_val_buff = t_val->data_as<uint8_t>();
+      const auto& t_val = tile_tuple->validity_tile();
+      const auto src_val_buff = t_val.data_as<uint8_t>();
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
           *val_buffer = src_val_buff[c];
           val_buffer++;
         }
@@ -584,7 +600,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
     } else {
       uint8_t v = array_schema_.attribute(name)->fill_value_validity();
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
           *val_buffer = v;
           val_buffer++;
         }
@@ -626,21 +642,21 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
         array_schema_.attribute(name)->fill_value().size());
     src_var_buff = array_schema_.attribute(name)->fill_value().data();
   } else {
-    const auto t = &std::get<0>(*tile_tuple);
-    const auto t_var = &std::get<1>(*tile_tuple);
-    t_var_size = t_var->size();
-    src_buff = t->template data_as<uint64_t>();
-    src_var_buff = t_var->template data_as<uint8_t>();
+    const auto& t = tile_tuple->fixed_tile();
+    const auto& t_var = tile_tuple->var_tile();
+    t_var_size = t_var.size();
+    src_buff = t.template data_as<uint64_t>();
+    src_var_buff = t_var.template data_as<uint8_t>();
   }
 
-  if (rt->has_bmp() || use_fill_value) {
+  if (!rt->copy_full_tile() || use_fill_value) {
     // Process all cells. Last cell might be taken out for vectorization.
     uint64_t end = (src_max_pos == cell_num && !use_fill_value) ?
                        src_max_pos - 1 :
                        src_max_pos;
     if (!use_fill_value) {
       for (uint64_t c = src_min_pos; c < end; c++) {
-        if (rt->bitmap_[c]) {
+        if (rt->bitmap()[c]) {
           *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
           buffer++;
           *var_data = src_var_buff + src_buff[c];
@@ -649,14 +665,14 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
       }
 
       // Do last cell.
-      if (src_max_pos == cell_num && rt->bitmap_[src_max_pos - 1]) {
+      if (src_max_pos == cell_num && rt->bitmap()[src_max_pos - 1]) {
         *buffer =
             (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
         *var_data = src_var_buff + src_buff[src_max_pos - 1];
       }
     } else {
       for (uint64_t c = src_min_pos; c < end; c++) {
-        if (!rt->has_bmp() || rt->bitmap_[c]) {
+        if (!rt->has_bmp() || rt->bitmap()[c]) {
           *buffer = fill_value_size / offset_div;
           buffer++;
           *var_data = src_var_buff;
@@ -668,10 +684,10 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
     // Copy nullable values.
     if (nullable) {
       if (!use_fill_value) {
-        const auto t_val = &std::get<2>(*tile_tuple);
-        const auto src_val_buff = t_val->data_as<uint8_t>();
+        const auto& t_val = tile_tuple->validity_tile();
+        const auto src_val_buff = t_val.data_as<uint8_t>();
         for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-          if (rt->bitmap_[c]) {
+          if (rt->bitmap()[c]) {
             *val_buffer = src_val_buff[c];
             val_buffer++;
           }
@@ -679,7 +695,7 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
       } else {
         uint8_t v = array_schema_.attribute(name)->fill_value_validity();
         for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-          if (rt->bitmap_[c]) {
+          if (rt->bitmap()[c]) {
             *val_buffer = v;
             val_buffer++;
           }
@@ -704,8 +720,8 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
 
     // Copy nullable values.
     if (nullable) {
-      const auto t_val = &std::get<2>(*tile_tuple);
-      const auto src_val_buff = t_val->template data_as<uint8_t>();
+      const auto& t_val = tile_tuple->validity_tile();
+      const auto src_val_buff = t_val.data_as<uint8_t>();
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
         *val_buffer = src_val_buff[c];
         val_buffer++;
@@ -756,15 +772,8 @@ Status SparseUnorderedWithDupsReader<BitmapType>::copy_offsets_tiles(
         // Adjust max cell if this is the last tile.
         if (i == result_tiles.size() - 1) {
           auto to_copy = cell_offsets[i + 1] - cell_offsets[i];
-
-          // No bitmap, just use the cell offsets. Otherwise, we need to check
-          // the bitmap to determine the max.
-          if (rt->bitmap_result_num_ == std::numeric_limits<uint64_t>::max()) {
-            max_pos_tile = min_pos_tile + to_copy;
-          } else {
-            max_pos_tile =
-                rt->pos_with_given_result_sum(min_pos_tile, to_copy) + 1;
-          }
+          max_pos_tile =
+              rt->pos_with_given_result_sum(min_pos_tile, to_copy) + 1;
         }
 
         auto&& [skip_copy, src_min_pos, src_max_pos, dest_cell_offset] =
@@ -920,7 +929,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_fixed_data_tile(
     use_fill_value = true;
     src_buff = array_schema_.attribute(name)->fill_value().data();
   } else {
-    const auto& t = std::get<0>(*tile_tuple);
+    const auto& t = tile_tuple->fixed_tile();
     src_buff = t.template data_as<uint8_t>();
   }
 
@@ -928,14 +937,14 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_fixed_data_tile(
   if (!stores_zipped_coords) {
     if (!use_fill_value) {
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
           memcpy(buffer, src_buff + c * cell_size, cell_size);
           buffer += cell_size;
         }
       }
     } else {
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
           memcpy(buffer, src_buff, cell_size);
           buffer += cell_size;
         }
@@ -944,7 +953,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_fixed_data_tile(
   } else {  // Copy for zipped coords.
     const auto dim_num = rt->domain()->dim_num();
     for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+      for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
         auto pos = c * dim_num + dim_idx;
         memcpy(buffer, src_buff + pos * cell_size, cell_size);
         buffer += cell_size;
@@ -955,10 +964,10 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_fixed_data_tile(
   // Copy nullable values.
   if (nullable) {
     if (!use_fill_value) {
-      const auto t_val = &std::get<2>(*tile_tuple);
-      const auto src_val_buff = t_val->data_as<uint8_t>();
+      const auto& t_val = tile_tuple->validity_tile();
+      const auto src_val_buff = t_val.data_as<uint8_t>();
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
           *val_buffer = src_val_buff[c];
           val_buffer++;
         }
@@ -966,7 +975,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_fixed_data_tile(
     } else {
       uint8_t v = array_schema_.attribute(name)->fill_value_validity();
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
           *val_buffer = v;
           val_buffer++;
         }
@@ -1004,11 +1013,11 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
     use_fill_value = true;
     src_buff = array_schema_.attribute(name)->fill_value().data();
   } else {
-    const auto& t = std::get<0>(*tile_tuple);
+    const auto& t = tile_tuple->fixed_tile();
     src_buff = t.template data_as<uint8_t>();
   }
 
-  if (rt->has_bmp() || use_fill_value) {
+  if (!rt->copy_full_tile() || use_fill_value) {
     // Copy values.
     if (!stores_zipped_coords) {
       if (!use_fill_value) {
@@ -1017,7 +1026,7 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
         uint64_t length = 0;
         uint64_t start = src_min_pos;
         for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-          if (rt->bitmap_[c]) {
+          if (rt->bitmap()[c]) {
             length++;
           } else {
             if (length != 0) {
@@ -1037,7 +1046,7 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
         }
       } else {
         for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-          if (!rt->has_bmp() || rt->bitmap_[c]) {
+          if (!rt->has_bmp() || rt->bitmap()[c]) {
             memcpy(buffer, src_buff, cell_size);
             buffer += cell_size;
           }
@@ -1046,7 +1055,7 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
     } else {  // Copy for zipped coords.
       const auto dim_num = rt->domain()->dim_num();
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        if (rt->bitmap_[c]) {
+        if (rt->bitmap()[c]) {
           auto pos = c * dim_num + dim_idx;
           memcpy(buffer, src_buff + pos * cell_size, cell_size);
           buffer += cell_size;
@@ -1059,12 +1068,12 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
       if (!use_fill_value) {
         // Go through bitmap, when there is a hole in cell contiguity, do a
         // memcpy.
-        const auto t_val = &std::get<2>(*tile_tuple);
+        const auto& t_val = tile_tuple->validity_tile();
         uint64_t length = 0;
         uint64_t start = src_min_pos;
-        const auto src_val_buff = t_val->data_as<uint8_t>();
+        const auto src_val_buff = t_val.data_as<uint8_t>();
         for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-          if (rt->bitmap_[c]) {
+          if (rt->bitmap()[c]) {
             length++;
           } else {
             if (length != 0) {
@@ -1085,7 +1094,7 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
       } else {
         uint8_t v = array_schema_.attribute(name)->fill_value_validity();
         for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-          for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+          for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
             *val_buffer = v;
             val_buffer++;
           }
@@ -1099,8 +1108,8 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
         (src_max_pos - src_min_pos) * cell_size);
 
     if (nullable) {
-      const auto t_val = &std::get<2>(*tile_tuple);
-      const auto src_val_buff = t_val->template data_as<uint8_t>();
+      const auto& t_val = tile_tuple->validity_tile();
+      const auto src_val_buff = t_val.data_as<uint8_t>();
       memcpy(val_buffer, src_val_buff + src_min_pos, src_max_pos - src_min_pos);
     }
   }
@@ -1116,14 +1125,19 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_timestamp_data_tile(
     uint8_t* buffer) {
   // Get source buffers.
   const auto tile_tuple = rt->tile_tuple(constants::timestamps);
-  const auto t = &std::get<0>(*tile_tuple);
-  const auto src_buff = t->data_as<uint8_t>();
+  Tile* t = nullptr;
+  uint8_t* src_buff = nullptr;
+  if (tile_tuple != nullptr) {
+    t = &tile_tuple->fixed_tile();
+    src_buff = t->data_as<uint8_t>();
+  }
+
   const uint64_t cell_size = constants::timestamp_size;
 
   if (fragment_metadata_[rt->frag_idx()]->has_timestamps()) {
     // Copy values.
     for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+      for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
         memcpy(buffer, src_buff + c * cell_size, cell_size);
         buffer += cell_size;
       }
@@ -1132,7 +1146,7 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_timestamp_data_tile(
     // Copy fragment timestamp.
     auto timestamp = fragment_timestamp(rt);
     for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+      for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
         memcpy(buffer, &timestamp, cell_size);
         buffer += cell_size;
       }
@@ -1150,20 +1164,24 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_timestamp_data_tile(
   auto timer_se = stats_->start_timer("copy_timestamps_tiles");
   // Get source buffers.
   const auto tile_tuple = rt->tile_tuple(constants::timestamps);
-  const auto t = &std::get<0>(*tile_tuple);
-  const auto src_buff = t->data_as<uint8_t>();
+  Tile* t = nullptr;
+  uint8_t* src_buff = nullptr;
+  if (tile_tuple != nullptr) {
+    t = &tile_tuple->fixed_tile();
+    src_buff = t->data_as<uint8_t>();
+  }
+
   const uint64_t cell_size = constants::timestamp_size;
   auto frag_timestamp = fragment_timestamp(rt);
 
-  // Partial tile copy, full tile copy done below.
-  if (rt->has_bmp()) {
+  if (!rt->copy_full_tile()) {
     // Copy values.
     // Go through bitmap, when there is a hole in cell contiguity, do a
     // memcpy.
     uint64_t length = 0;
     uint64_t start = src_min_pos;
     for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-      if (rt->bitmap_[c]) {
+      if (rt->bitmap()[c]) {
         length++;
       } else {
         if (length != 0) {
@@ -1247,15 +1265,8 @@ Status SparseUnorderedWithDupsReader<BitmapType>::copy_fixed_data_tiles(
         // Adjust max cell if this is the last tile.
         if (i == result_tiles.size() - 1) {
           auto to_copy = cell_offsets[i + 1] - cell_offsets[i];
-
-          // No bitmap, just use the cell offsets. Otherwise, we need to check
-          // the bitmap to determine the max.
-          if (rt->bitmap_result_num_ == std::numeric_limits<uint64_t>::max()) {
-            max_pos_tile = min_pos_tile + to_copy;
-          } else {
-            max_pos_tile =
-                rt->pos_with_given_result_sum(min_pos_tile, to_copy) + 1;
-          }
+          max_pos_tile =
+              rt->pos_with_given_result_sum(min_pos_tile, to_copy) + 1;
         }
 
         auto&& [skip_copy, src_min_pos, src_max_pos, dest_cell_offset] =
@@ -1341,24 +1352,15 @@ SparseUnorderedWithDupsReader<BitmapType>::compute_fixed_results_to_copy(
   // in the fragment metadata to do so.
   uint64_t cell_offset = cells_copied(names);
   for (uint64_t i = 0; i < result_tiles.size(); i++) {
-    auto rt = (UnorderedWithDupsResultTile<BitmapType>*)result_tiles[i];
-    auto cell_num =
-        rt->bitmap_result_num_ != std::numeric_limits<uint64_t>::max() ?
-            rt->bitmap_result_num_ :
-            fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
+    auto rt = (ResultTileWithBitmap<BitmapType>*)result_tiles[i];
+    auto cell_num = rt->result_num();
 
     // First tile might have been partially copied. Adjust cell_num to account
     // for it.
     if (i == 0) {
       if (read_state_.frag_idx_[rt->frag_idx()].tile_idx_ == rt->tile_idx()) {
-        if (rt->bitmap_result_num_ != std::numeric_limits<uint64_t>::max()) {
-          if (rt->bitmap_result_num_ != 0) {
-            auto pos = read_state_.frag_idx_[rt->frag_idx()].cell_idx_;
-            cell_num -= rt->result_num_between_pos(0, pos);
-          }
-        } else {
-          cell_num -= read_state_.frag_idx_[rt->frag_idx()].cell_idx_;
-        }
+        auto pos = read_state_.frag_idx_[rt->frag_idx()].cell_idx_;
+        cell_num -= rt->result_num_between_pos(0, pos);
       }
     }
 
@@ -1410,8 +1412,9 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
         // For dimensions, when we have a subarray, tiles are already all
         // loaded in memory.
         if ((subarray_.is_set() && array_schema_.is_dim(name)) ||
-            condition_.field_names().count(name) != 0 || is_timestamps)
+            qc_loaded_attr_names_set_.count(name) != 0 || is_timestamps) {
           return Status::Ok();
+        }
 
         // Get the size for all tiles.
         uint64_t idx = 0;
@@ -1432,12 +1435,7 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
           // Account for the pointers to the var data that is created in
           // copy_tiles for var sized attributes.
           if (var_sized) {
-            auto cell_num =
-                rt->bitmap_result_num_ != std::numeric_limits<uint64_t>::max() ?
-                    rt->bitmap_result_num_ :
-                    fragment_metadata_[rt->frag_idx()]->cell_num(
-                        rt->tile_idx());
-            *tile_size += sizeof(void*) * cell_num;
+            *tile_size += sizeof(void*) * rt->result_num();
           }
 
           // Stop when we reach the budget.
@@ -1521,10 +1519,9 @@ SparseUnorderedWithDupsReader<BitmapType>::compute_var_size_offsets(
           new_result_tiles_size > 0 ? cell_offsets[new_result_tiles_size - 1] :
                                       0;
 
-      const bool has_bmp = last_tile->has_bmp();
       const auto min_pos = new_result_tiles_size == 1 ? first_tile_min_pos : 0;
       for (uint64_t c = min_pos; c < last_tile_num_cells - 1; c++) {
-        auto cell_count = has_bmp ? last_tile->bitmap_[c] : 1;
+        auto cell_count = last_tile->has_bmp() ? last_tile->bitmap()[c] : 1;
         auto new_size =
             ((OffType*)query_buffer
                  .buffer_)[cell_offsets[new_result_tiles_size] + cell_count];
@@ -1655,7 +1652,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
         for (const auto& idx : *index_to_copy) {
           const auto& name_to_clear = names[idx];
           const auto is_dim_to_clear = array_schema_.is_dim(name_to_clear);
-          if (condition_.field_names().count(name_to_clear) == 0 &&
+          if (qc_loaded_attr_names_set_.count(name_to_clear) == 0 &&
               (!subarray_.is_set() || !is_dim_to_clear)) {
             clear_tiles(name_to_clear, result_tiles, new_result_tiles_size);
           }
@@ -1692,7 +1689,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
         *query_buffer.validity_vector_.buffer_size() = total_cells;
 
       // Clear tiles from memory.
-      if (condition_.field_names().count(name) == 0 &&
+      if (qc_loaded_attr_names_set_.count(name) == 0 &&
           (!subarray_.is_set() || !is_dim) && name != constants::timestamps &&
           name != constants::delete_timestamps) {
         clear_tiles(name, result_tiles);
@@ -1708,12 +1705,8 @@ Status SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
   auto last_tile_cells_copied =
       cell_offsets[result_tiles.size()] - cell_offsets[result_tiles.size() - 1];
   if (frag_tile_idx.tile_idx_ == last_tile->tile_idx()) {
-    if (last_tile->bitmap_result_num_ != std::numeric_limits<uint64_t>::max()) {
-      last_tile_cells_copied +=
-          last_tile->result_num_between_pos(0, frag_tile_idx.cell_idx_);
-    } else {
-      last_tile_cells_copied += frag_tile_idx.cell_idx_;
-    }
+    last_tile_cells_copied +=
+        last_tile->result_num_between_pos(0, frag_tile_idx.cell_idx_);
   }
 
   // Adjust tile index.
@@ -1722,18 +1715,10 @@ Status SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
   }
 
   // If the last tile is not fully copied, save the cell index.
-  auto last_tile_num_cells =
-      fragment_metadata_[last_tile->frag_idx()]->cell_num(
-          last_tile->tile_idx());
-  if (last_tile->bitmap_result_num_ != std::numeric_limits<uint64_t>::max()) {
-    if (last_tile->bitmap_result_num_ != last_tile_cells_copied) {
-      frag_tile_idx.tile_idx_ = last_tile->tile_idx();
-      frag_tile_idx.cell_idx_ =
-          last_tile->pos_with_given_result_sum(0, last_tile_cells_copied) + 1;
-    }
-  } else if (last_tile_num_cells != last_tile_cells_copied) {
+  if (last_tile->result_num() != last_tile_cells_copied) {
     frag_tile_idx.tile_idx_ = last_tile->tile_idx();
-    frag_tile_idx.cell_idx_ = last_tile_cells_copied;
+    frag_tile_idx.cell_idx_ =
+        last_tile->pos_with_given_result_sum(0, last_tile_cells_copied) + 1;
   }
 
   logger_->debug("Done copying tiles");
@@ -1801,7 +1786,8 @@ template SparseUnorderedWithDupsReader<uint8_t>::SparseUnorderedWithDupsReader(
     std::unordered_map<std::string, QueryBuffer>&,
     Subarray&,
     Layout,
-    QueryCondition&);
+    QueryCondition&,
+    bool);
 template SparseUnorderedWithDupsReader<uint64_t>::SparseUnorderedWithDupsReader(
     stats::Stats*,
     shared_ptr<Logger>,
@@ -1811,7 +1797,8 @@ template SparseUnorderedWithDupsReader<uint64_t>::SparseUnorderedWithDupsReader(
     std::unordered_map<std::string, QueryBuffer>&,
     Subarray&,
     Layout,
-    QueryCondition&);
+    QueryCondition&,
+    bool);
 
 }  // namespace sm
 }  // namespace tiledb
\ No newline at end of file
