@@ -76,7 +76,9 @@ UnorderedWriter::UnorderedWriter(
     Layout layout,
     std::vector<WrittenFragmentInfo>& written_fragment_info,
     Query::CoordsInfo& coords_info,
-    URI fragment_uri)
+    bool remote_query,
+    optional<std::string> fragment_name,
+    bool skip_checks_serialization)
     : WriterBase(
           stats,
           logger,
@@ -89,7 +91,21 @@ UnorderedWriter::UnorderedWriter(
           written_fragment_info,
           false,
           coords_info,
-          fragment_uri) {
+          remote_query,
+          fragment_name,
+          skip_checks_serialization) {
+  if (layout != Layout::UNORDERED) {
+    throw StatusException(Status_WriterError(
+        "Failed to initialize UnorderedWriter; The unordered writer does not "
+        "support layout " +
+        layout_str(layout)));
+  }
+
+  if (array_schema_.dense()) {
+    throw StatusException(Status_WriterError(
+        "Failed to initialize UnorderedWriter; The unordered "
+        "writer does not support dense arrays."));
+  }
 }
 
 UnorderedWriter::~UnorderedWriter() {
@@ -122,7 +138,6 @@ Status UnorderedWriter::finalize() {
 }
 
 void UnorderedWriter::reset() {
-  initialized_ = false;
 }
 
 /* ****************************** */
@@ -665,17 +680,26 @@ Status UnorderedWriter::unordered_write() {
       frag_meta->compute_fragment_min_max_sum_null_count(), clean_up(uri));
 
   // Write the fragment metadata
-  RETURN_CANCEL_OR_ERROR_ELSE(
-      frag_meta->store(array_->get_encryption_key()), clean_up(uri));
+  try {
+    frag_meta->store(array_->get_encryption_key());
+  } catch (...) {
+    clean_up(uri);
+    throw;
+  }
 
   // Add written fragment info
   RETURN_NOT_OK_ELSE(add_written_fragment_info(uri), clean_up(uri));
 
   // The following will make the fragment visible
-  auto&& [st, commit_uri] = array_->array_directory().get_commit_uri(uri);
-  RETURN_NOT_OK_ELSE(st, storage_manager_->vfs()->remove_dir(uri));
-  RETURN_NOT_OK_ELSE(
-      storage_manager_->vfs()->touch(commit_uri.value()), clean_up(uri));
+  URI commit_uri;
+  try {
+    commit_uri = array_->array_directory().get_commit_uri(uri);
+  } catch (std::exception& e) {
+    RETURN_NOT_OK(storage_manager_->vfs()->remove_dir(uri));
+    std::throw_with_nested(
+        std::logic_error("[UnorderedWriter::unordered_write] "));
+  }
+  RETURN_NOT_OK_ELSE(storage_manager_->vfs()->touch(commit_uri), clean_up(uri));
 
   return Status::Ok();
 }
