@@ -53,7 +53,7 @@
 #include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/sm/tile/tile_metadata_generator.h"
 #include "tiledb/sm/tile/writer_tile.h"
-#include "tiledb/storage_format/uri/parse_uri.h"
+#include "tiledb/storage_format/uri/generate_uri.h"
 
 using namespace tiledb;
 using namespace tiledb::common;
@@ -62,6 +62,13 @@ using namespace tiledb::sm::stats;
 namespace tiledb {
 namespace sm {
 
+class WriterBaseStatusException : public StatusException {
+ public:
+  explicit WriterBaseStatusException(const std::string& message)
+      : StatusException("WriterBase", message) {
+  }
+};
+
 /* ****************************** */
 /*   CONSTRUCTORS & DESTRUCTORS   */
 /* ****************************** */
@@ -78,7 +85,9 @@ WriterBase::WriterBase(
     std::vector<WrittenFragmentInfo>& written_fragment_info,
     bool disable_checks_consolidation,
     Query::CoordsInfo& coords_info,
-    URI fragment_uri)
+    bool remote_query,
+    optional<std::string> fragment_name,
+    bool skip_checks_serialization)
     : StrategyBase(
           stats,
           logger->clone("Writer", ++logger_id_),
@@ -94,9 +103,116 @@ WriterBase::WriterBase(
     , check_coord_oob_(false)
     , check_global_order_(false)
     , dedup_coords_(false)
-    , initialized_(false)
-    , written_fragment_info_(written_fragment_info) {
-  fragment_uri_ = fragment_uri;
+    , written_fragment_info_(written_fragment_info)
+    , remote_query_(remote_query) {
+  // Sanity checks
+  if (storage_manager_ == nullptr) {
+    throw WriterBaseStatusException(
+        "Cannot initialize query; Storage manager not set");
+  }
+
+  if (!skip_checks_serialization && buffers_.empty()) {
+    throw WriterBaseStatusException(
+        "Cannot initialize writer; Buffers not set");
+  }
+
+  if (array_schema_.dense() &&
+      (layout_ == Layout::ROW_MAJOR || layout_ == Layout::COL_MAJOR)) {
+    for (const auto& b : buffers_) {
+      if (array_schema_.is_dim(b.first)) {
+        throw WriterBaseStatusException(
+            "Cannot initialize writer; Sparse coordinates for dense arrays "
+            "cannot be provided if the query layout is ROW_MAJOR or COL_MAJOR");
+      }
+    }
+  }
+
+  // Get configuration parameters
+  const char *check_coord_dups, *check_coord_oob, *check_global_order;
+  const char* dedup_coords;
+  if (!config_.get("sm.check_coord_dups", &check_coord_dups).ok()) {
+    throw WriterBaseStatusException("Cannot get setting");
+  }
+
+  if (!config_.get("sm.check_coord_oob", &check_coord_oob).ok()) {
+    throw WriterBaseStatusException("Cannot get setting");
+  }
+
+  if (!config_.get("sm.check_global_order", &check_global_order).ok()) {
+    throw WriterBaseStatusException("Cannot get setting");
+  }
+
+  if (!config_.get("sm.dedup_coords", &dedup_coords).ok()) {
+    throw WriterBaseStatusException("Cannot get setting");
+  }
+
+  assert(check_coord_dups != nullptr && dedup_coords != nullptr);
+  check_coord_dups_ =
+      disable_checks_consolidation_ ? false : !strcmp(check_coord_dups, "true");
+  check_coord_oob_ = !strcmp(check_coord_oob, "true");
+  check_global_order_ = disable_checks_consolidation_ ?
+                            false :
+                            !strcmp(check_global_order, "true");
+  dedup_coords_ = !strcmp(dedup_coords, "true");
+  bool found = false;
+  offsets_format_mode_ = config_.get("sm.var_offsets.mode", &found);
+  assert(found);
+  if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
+    throw WriterBaseStatusException(
+        "Cannot initialize writer; Unsupported offsets format in "
+        "configuration");
+  }
+  if (!config_
+           .get<bool>(
+               "sm.var_offsets.extra_element", &offsets_extra_element_, &found)
+           .ok()) {
+    throw WriterBaseStatusException("Cannot get setting");
+  }
+  assert(found);
+
+  if (!config_
+           .get<uint32_t>("sm.var_offsets.bitsize", &offsets_bitsize_, &found)
+           .ok()) {
+    throw WriterBaseStatusException("Cannot get setting");
+  }
+  assert(found);
+
+  if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
+    throw WriterBaseStatusException(
+        "Cannot initialize writer; Unsupported offsets bitsize in "
+        "configuration");
+  }
+
+  // Set a default subarray
+  if (!subarray_.is_set()) {
+    subarray_ = Subarray(array_, layout_, stats_, logger_);
+  }
+
+  if (offsets_extra_element_) {
+    check_extra_element();
+  }
+
+  if (!skip_checks_serialization) {
+    check_subarray();
+    check_buffer_sizes();
+  }
+
+  optimize_layout_for_1D();
+  check_var_attr_offsets();
+
+  // Get the timestamp the array was opened and the array write version.
+  uint64_t timestamp = array_->timestamp_end_opened_at();
+  auto write_version = array_->array_schema_latest().write_version();
+
+  // Set the fragment URI using either the provided fragment name or a generated
+  // fragment name.
+  auto new_fragment_str =
+      fragment_name.has_value() ?
+          fragment_name.value() :
+          storage_format::generate_fragment_name(timestamp, write_version);
+  auto frag_dir_uri =
+      array_->array_directory().get_fragments_dir(write_version);
+  fragment_uri_ = frag_dir_uri.join_path(new_fragment_str);
 }
 
 WriterBase::~WriterBase() {
@@ -131,7 +247,7 @@ void WriterBase::set_dedup_coords(bool b) {
   dedup_coords_ = b;
 }
 
-Status WriterBase::check_var_attr_offsets() const {
+void WriterBase::check_var_attr_offsets() const {
   for (const auto& it : buffers_) {
     const auto& attr = it.first;
     if (!array_schema_.var_size(attr)) {
@@ -143,25 +259,27 @@ Status WriterBase::check_var_attr_offsets() const {
         get_offset_buffer_size(*it.second.buffer_size_);
     const uint64_t* buffer_val_size = it.second.buffer_var_size_;
     auto num_offsets = buffer_off_size / constants::cell_var_offset_size;
-    if (num_offsets == 0)
-      return Status::Ok();
+    if (num_offsets == 0) {
+      return;
+    }
 
     uint64_t prev_offset = get_offset_buffer_element(buffer_off, 0);
     // Allow the initial offset to be equal to the size, this indicates
     // the first and only value in the buffer is to be empty
-    if (prev_offset > *buffer_val_size)
-      return logger_->status(Status_WriterError(
+    if (prev_offset > *buffer_val_size) {
+      throw WriterBaseStatusException(
           "Invalid offsets for attribute " + attr + "; offset " +
           std::to_string(prev_offset) + " specified for buffer of size " +
-          std::to_string(*buffer_val_size)));
+          std::to_string(*buffer_val_size));
+    }
 
     for (uint64_t i = 1; i < num_offsets; i++) {
       uint64_t cur_offset = get_offset_buffer_element(buffer_off, i);
       if (cur_offset < prev_offset)
-        return logger_->status(Status_WriterError(
+        throw WriterBaseStatusException(
             "Invalid offsets for attribute " + attr +
             "; offsets must be given in "
-            "strictly ascending order."));
+            "strictly ascending order.");
 
       // Allow the last offset(s) to be equal to the size, this indicates the
       // last value(s) are to be empty
@@ -170,89 +288,15 @@ Status WriterBase::check_var_attr_offsets() const {
            get_offset_buffer_element(
                buffer_off, (i < num_offsets - 1 ? i + 1 : i)) !=
                *buffer_val_size))
-        return logger_->status(Status_WriterError(
+        throw WriterBaseStatusException(
             "Invalid offsets for attribute " + attr + "; offset " +
             std::to_string(cur_offset) + " specified at index " +
             std::to_string(i) + " for buffer of size " +
-            std::to_string(*buffer_val_size)));
+            std::to_string(*buffer_val_size));
 
       prev_offset = cur_offset;
     }
   }
-
-  return Status::Ok();
-}
-
-Status WriterBase::init() {
-  // Sanity checks
-  if (storage_manager_ == nullptr)
-    return logger_->status(
-        Status_WriterError("Cannot initialize query; Storage manager not set"));
-  if (buffers_.empty())
-    return logger_->status(
-        Status_WriterError("Cannot initialize writer; Buffers not set"));
-  if (array_schema_.dense() &&
-      (layout_ == Layout::ROW_MAJOR || layout_ == Layout::COL_MAJOR)) {
-    for (const auto& b : buffers_) {
-      if (array_schema_.is_dim(b.first)) {
-        return logger_->status(Status_WriterError(
-            "Cannot initialize writer; Sparse coordinates "
-            "for dense arrays cannot be provided "
-            "if the query layout is ROW_MAJOR or COL_MAJOR"));
-      }
-    }
-  }
-
-  // Get configuration parameters
-  const char *check_coord_dups, *check_coord_oob, *check_global_order;
-  const char* dedup_coords;
-  RETURN_NOT_OK(config_.get("sm.check_coord_dups", &check_coord_dups));
-  RETURN_NOT_OK(config_.get("sm.check_coord_oob", &check_coord_oob));
-  RETURN_NOT_OK(config_.get("sm.check_global_order", &check_global_order));
-  RETURN_NOT_OK(config_.get("sm.dedup_coords", &dedup_coords));
-  assert(check_coord_dups != nullptr && dedup_coords != nullptr);
-  check_coord_dups_ =
-      disable_checks_consolidation_ ? false : !strcmp(check_coord_dups, "true");
-  check_coord_oob_ = !strcmp(check_coord_oob, "true");
-  check_global_order_ = disable_checks_consolidation_ ?
-                            false :
-                            !strcmp(check_global_order, "true");
-  dedup_coords_ = !strcmp(dedup_coords, "true");
-  bool found = false;
-  offsets_format_mode_ = config_.get("sm.var_offsets.mode", &found);
-  assert(found);
-  if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    return logger_->status(
-        Status_WriterError("Cannot initialize writer; Unsupported offsets "
-                           "format in configuration"));
-  }
-  RETURN_NOT_OK(config_.get<bool>(
-      "sm.var_offsets.extra_element", &offsets_extra_element_, &found));
-  assert(found);
-  RETURN_NOT_OK(config_.get<uint32_t>(
-      "sm.var_offsets.bitsize", &offsets_bitsize_, &found));
-  if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    return logger_->status(
-        Status_WriterError("Cannot initialize writer; Unsupported offsets "
-                           "bitsize in configuration"));
-  }
-  assert(found);
-
-  // Set a default subarray
-  if (!subarray_.is_set())
-    subarray_ = Subarray(array_, layout_, stats_, logger_);
-
-  if (offsets_extra_element_)
-    RETURN_NOT_OK(check_extra_element());
-
-  RETURN_NOT_OK(check_subarray());
-  RETURN_NOT_OK(check_buffer_sizes());
-
-  optimize_layout_for_1D();
-  RETURN_NOT_OK(check_var_attr_offsets());
-  initialized_ = true;
-
-  return Status::Ok();
 }
 
 Status WriterBase::initialize_memory_budget() {
@@ -301,11 +345,12 @@ Status WriterBase::calculate_hilbert_values(
   return Status::Ok();
 }
 
-Status WriterBase::check_buffer_sizes() const {
+void WriterBase::check_buffer_sizes() const {
   // This is applicable only to dense arrays and ordered layout
   if (!array_schema_.dense() ||
-      (layout_ != Layout::ROW_MAJOR && layout_ != Layout::COL_MAJOR))
-    return Status::Ok();
+      (layout_ != Layout::ROW_MAJOR && layout_ != Layout::COL_MAJOR)) {
+    return;
+  }
 
   auto cell_num = array_schema_.domain().cell_num(subarray_.ndrange(0));
   uint64_t expected_cell_num = 0;
@@ -333,7 +378,7 @@ Status WriterBase::check_buffer_sizes() const {
               "given for ";
         ss << "attribute '" << attr << "'";
         ss << " (" << expected_validity_num << " != " << cell_num << ")";
-        return logger_->status(Status_WriterError(ss.str()));
+        throw WriterBaseStatusException(ss.str());
       }
     } else {
       if (expected_cell_num != cell_num) {
@@ -341,12 +386,10 @@ Status WriterBase::check_buffer_sizes() const {
         ss << "Buffer sizes check failed; Invalid number of cells given for ";
         ss << "attribute '" << attr << "'";
         ss << " (" << expected_cell_num << " != " << cell_num << ")";
-        return logger_->status(Status_WriterError(ss.str()));
+        throw WriterBaseStatusException(ss.str());
       }
     }
   }
-
-  return Status::Ok();
 }
 
 Status WriterBase::check_coord_oob() const {
@@ -394,20 +437,19 @@ Status WriterBase::check_coord_oob() const {
   return Status::Ok();
 }
 
-Status WriterBase::check_subarray() const {
+void WriterBase::check_subarray() const {
   if (array_schema_.dense()) {
-    if (subarray_.range_num() != 1)
-      return LOG_STATUS(
-          Status_WriterError("Multi-range dense writes "
-                             "are not supported"));
+    if (subarray_.range_num() != 1) {
+      throw WriterBaseStatusException(
+          "Multi-range dense writes are not supported");
+    }
 
-    if (layout_ == Layout::GLOBAL_ORDER && !subarray_.coincides_with_tiles())
-      return logger_->status(
-          Status_WriterError("Cannot initialize query; In global writes for "
-                             "dense arrays, the subarray "
-                             "must coincide with the tile bounds"));
+    if (layout_ == Layout::GLOBAL_ORDER && !subarray_.coincides_with_tiles()) {
+      throw WriterBaseStatusException(
+          "Cannot initialize query; In global writes for dense arrays, the "
+          "subarray must coincide with the tile bounds");
+    }
   }
-  return Status::Ok();
 }
 
 void WriterBase::clear_coord_buffers() {
@@ -545,10 +587,11 @@ Status WriterBase::compute_tiles_metadata(
       const auto var_size = array_schema_.var_size(attr);
       const auto cell_size = array_schema_.cell_size(attr);
       const auto cell_val_num = array_schema_.cell_val_num(attr);
-      TileMetadataGenerator md_generator(
-          type, is_dim, var_size, cell_size, cell_val_num);
       for (auto& tile : attr_tiles) {
-        md_generator.process_tile(tile);
+        TileMetadataGenerator md_generator(
+            type, is_dim, var_size, cell_size, cell_val_num);
+        md_generator.process_full_tile(tile);
+        md_generator.set_tile_metadata(tile);
       }
 
       return Status::Ok();
@@ -566,7 +609,8 @@ Status WriterBase::compute_tiles_metadata(
       auto st = parallel_for(compute_tp, 0, tile_num, [&](uint64_t t) {
         TileMetadataGenerator md_generator(
             type, is_dim, var_size, cell_size, cell_val_num);
-        md_generator.process_tile(attr_tiles[t]);
+        md_generator.process_full_tile(attr_tiles[t]);
+        md_generator.set_tile_metadata(attr_tiles[t]);
 
         return Status::Ok();
       });
@@ -595,25 +639,23 @@ std::string WriterBase::coords_to_str(uint64_t i) const {
 }
 
 Status WriterBase::create_fragment(
-    bool dense, shared_ptr<FragmentMetadata>& frag_meta) const {
-  URI uri;
+    bool dense, shared_ptr<FragmentMetadata>& frag_meta) {
+  // Get write version, timestamp array was opened,  and a reference to the
+  // array directory.
+  auto write_version = array_->array_schema_latest().write_version();
   uint64_t timestamp = array_->timestamp_end_opened_at();
-  if (!fragment_uri_.to_string().empty()) {
-    uri = fragment_uri_;
-  } else {
-    std::string new_fragment_str;
-    auto write_version = array_->array_schema_latest().write_version();
-    RETURN_NOT_OK(
-        new_fragment_name(timestamp, write_version, &new_fragment_str));
+  auto& array_dir = array_->array_directory();
 
-    auto& array_dir = array_->array_directory();
-    auto frag_uri = array_dir.get_fragments_dir(write_version);
-    RETURN_NOT_OK(storage_manager_->vfs()->create_dir(frag_uri));
-    auto commit_uri = array_dir.get_commits_dir(write_version);
-    RETURN_NOT_OK(storage_manager_->vfs()->create_dir(commit_uri));
+  // Create the directories.
+  // Create the fragment directory, the directory for the new fragment URI, and
+  // the commit directory.
+  throw_if_not_ok(storage_manager_->vfs()->create_dir(
+      array_dir.get_fragments_dir(write_version)));
+  throw_if_not_ok(storage_manager_->create_dir(fragment_uri_));
+  throw_if_not_ok(storage_manager_->vfs()->create_dir(
+      array_dir.get_commits_dir(write_version)));
 
-    uri = frag_uri.join_path(new_fragment_str);
-  }
+  // Create fragment metadata.
   auto timestamp_range = std::pair<uint64_t, uint64_t>(timestamp, timestamp);
   const bool has_timestamps = buffers_.count(constants::timestamps) != 0;
   const bool has_delete_metadata =
@@ -623,14 +665,14 @@ Status WriterBase::create_fragment(
       storage_manager_,
       nullptr,
       array_->array_schema_latest_ptr(),
-      uri,
+      fragment_uri_,
       timestamp_range,
       dense,
       has_timestamps,
       has_delete_metadata);
 
   RETURN_NOT_OK((frag_meta)->init(subarray_.ndrange(0)));
-  return storage_manager_->create_dir(uri);
+  return Status::Ok();
 }
 
 Status WriterBase::filter_tiles(
@@ -790,30 +832,13 @@ Status WriterBase::init_tiles(
   return Status::Ok();
 }
 
-Status WriterBase::new_fragment_name(
-    uint64_t timestamp, uint32_t format_version, std::string* frag_uri) const {
-  timestamp = (timestamp != 0) ? timestamp : utils::time::timestamp_now_ms();
-
-  if (frag_uri == nullptr)
-    return Status_WriterError("Null fragment uri argument.");
-  std::string uuid;
-  frag_uri->clear();
-  RETURN_NOT_OK(uuid::generate_uuid(&uuid, false));
-  std::stringstream ss;
-  ss << "/__" << timestamp << "_" << timestamp << "_" << uuid << "_"
-     << format_version;
-
-  *frag_uri = ss.str();
-  return Status::Ok();
-}
-
 void WriterBase::optimize_layout_for_1D() {
   if (array_schema_.dim_num() == 1 && layout_ != Layout::GLOBAL_ORDER &&
       layout_ != Layout::UNORDERED)
     layout_ = array_schema_.cell_order();
 }
 
-Status WriterBase::check_extra_element() {
+void WriterBase::check_extra_element() {
   for (const auto& it : buffers_) {
     const auto& attr = it.first;
     if (!array_schema_.var_size(attr) || array_schema_.is_dim(attr))
@@ -830,15 +855,14 @@ Status WriterBase::check_extra_element() {
     const uint64_t last_offset =
         get_offset_buffer_element(buffer_off, num_offsets - 1);
 
-    if (last_offset != max_offset)
-      return logger_->status(Status_WriterError(
+    if (last_offset != max_offset) {
+      throw WriterBaseStatusException(
           "Invalid offsets for attribute " + attr +
           "; the last offset: " + std::to_string(last_offset) +
           " is not equal to the size of the data buffer: " +
-          std::to_string(max_offset)));
+          std::to_string(max_offset));
+    }
   }
-
-  return Status::Ok();
 }
 
 Status WriterBase::split_coords_buffer() {
@@ -1012,26 +1036,45 @@ Status WriterBase::write_tiles(
     }
   }
 
-  // Close files, except in the case of global order
-  if (close_files && layout_ != Layout::GLOBAL_ORDER) {
+  // Close files or flush multipart upload buffers in case of global order
+  // writes
+  if (close_files) {
+    std::vector<URI> closing_uris;
     auto&& [st1, uri] = frag_meta->uri(name);
     RETURN_NOT_OK(st1);
+    closing_uris.push_back(*uri);
 
-    RETURN_NOT_OK(storage_manager_->close_file(*uri));
     if (var_size) {
       auto&& [st2, var_uri] = frag_meta->var_uri(name);
       RETURN_NOT_OK(st2);
-      RETURN_NOT_OK(storage_manager_->close_file(*var_uri));
+      closing_uris.push_back(*var_uri);
     }
     if (nullable) {
       auto&& [st2, validity_uri] = frag_meta->validity_uri(name);
       RETURN_NOT_OK(st2);
-      RETURN_NOT_OK(storage_manager_->close_file(*validity_uri));
+      closing_uris.push_back(*validity_uri);
+    }
+    for (auto& u : closing_uris) {
+      if (layout_ == Layout::GLOBAL_ORDER) {
+        // Flushing the multipart buffers after each write stage is a
+        // requirement of remote global order writes, it should only be
+        // done if this code is executed as a result of a remote query
+        if (remote_query()) {
+          RETURN_NOT_OK(
+              storage_manager_->vfs()->flush_multipart_file_buffer(u));
+        }
+      } else {
+        RETURN_NOT_OK(storage_manager_->close_file(u));
+      }
     }
   }
 
   return Status::Ok();
 }
 
+bool WriterBase::remote_query() const {
+  return remote_query_;
+}
+
 }  // namespace sm
 }  // namespace tiledb
