@@ -371,6 +371,9 @@ class Curl {
   /** Verbose logging in curl. */
   bool verbose_;
 
+  /** Max curl buffer size for received data. */
+  uint64_t curl_buffer_size_;
+
   /**
    * Populates the curl slist with authorization (token or username+password),
    * and any extra headers.
