@@ -37,6 +37,7 @@
 #include "tiledb/sm/serialization/array_schema_evolution.h"
 #include "tiledb/sm/serialization/array.h"
 #include "tiledb/sm/serialization/config.h"
+#include "tiledb/sm/serialization/fragment_info.h"
 #include "tiledb/sm/serialization/group.h"
 #include "tiledb/sm/serialization/query.h"
 #include "tiledb/sm/serialization/tiledb-rest.h"
@@ -224,8 +225,11 @@ RestClient::get_array_schema_from_rest(const URI& uri) {
   // Ensure data has a null delimiter for cap'n proto if using JSON
   RETURN_NOT_OK_TUPLE(
       ensure_json_null_delimited_string(&returned_data), nullopt);
-  return serialization::array_schema_deserialize(
-      serialization_type_, returned_data);
+  return {Status::Ok(),
+          make_shared<ArraySchema>(
+              HERE(),
+              serialization::array_schema_deserialize(
+                  serialization_type_, returned_data))};
 }
 
 Status RestClient::post_array_schema_to_rest(
@@ -772,6 +776,67 @@ Status RestClient::finalize_query_to_rest(const URI& uri, Query* query) {
       returned_data, serialization_type_, true, nullptr, query, compute_tp_);
 }
 
+Status RestClient::submit_and_finalize_query_to_rest(
+    const URI& uri, Query* query) {
+  serialization::CopyState copy_state;
+
+  // Get array
+  const Array* array = query->array();
+  if (array == nullptr) {
+    return LOG_STATUS(Status_RestError(
+        "Error while submit_and_finalize query to REST; null array."));
+  }
+
+  auto rest_scratch = query->rest_scratch();
+
+  // Serialize query to send
+  BufferList serialized;
+  RETURN_NOT_OK(serialization::query_serialize(
+      query, serialization_type_, true, &serialized));
+
+  // Init curl and form the URL
+  Curl curlc(logger_);
+  std::string array_ns, array_uri;
+  RETURN_NOT_OK(uri.get_rest_components(&array_ns, &array_uri));
+  const std::string cache_key = array_ns + ":" + array_uri;
+  RETURN_NOT_OK(
+      curlc.init(config_, extra_headers_, &redirect_meta_, &redirect_mtx_));
+  std::string url =
+      redirect_uri(cache_key) + "/v2/arrays/" + array_ns + "/" +
+      curlc.url_escape(array_uri) +
+      "/query/submit_and_finalize?type=" + query_type_str(query->type());
+
+  auto write_cb = std::bind(
+      &RestClient::query_post_call_back,
+      this,
+      std::placeholders::_1,
+      std::placeholders::_2,
+      std::placeholders::_3,
+      std::placeholders::_4,
+      rest_scratch,
+      query,
+      &copy_state);
+
+  const Status st = curlc.post_data(
+      stats_,
+      url,
+      serialization_type_,
+      &serialized,
+      rest_scratch.get(),
+      std::move(write_cb),
+      cache_key);
+
+  if (!st.ok() && copy_state.empty()) {
+    return LOG_STATUS(Status_RestError(
+        "Error while submit_and_finalize query to REST; "
+        "server returned no data. "
+        "Curl error: " +
+        st.message()));
+  }
+
+  return st;
+}
+
 Status RestClient::subarray_to_str(
     const ArraySchema& schema,
     const void* subarray,
@@ -973,6 +1038,48 @@ Status RestClient::post_array_schema_evolution_to_rest(
   return sc;
 }
 
+Status RestClient::post_fragment_info_from_rest(
+    const URI& uri, FragmentInfo* fragment_info) {
+  if (fragment_info == nullptr)
+    return LOG_STATUS(Status_RestError(
+        "Error getting fragment info from REST; fragment info is null."));
+
+  Buffer buff;
+  RETURN_NOT_OK(serialization::fragment_info_request_serialize(
+      *fragment_info, serialization_type_, &buff));
+  // Wrap in a list
+  BufferList serialized;
+  RETURN_NOT_OK(serialized.add_buffer(std::move(buff)));
+
+  // Init curl and form the URL
+  Curl curlc(logger_);
+  std::string array_ns, array_uri;
+  RETURN_NOT_OK(uri.get_rest_components(&array_ns, &array_uri));
+  const std::string cache_key = array_ns + ":" + array_uri;
+  RETURN_NOT_OK(
+      curlc.init(config_, extra_headers_, &redirect_meta_, &redirect_mtx_));
+  const std::string url = redirect_uri(cache_key) + "/v1/arrays/" + array_ns +
+                          "/" + curlc.url_escape(array_uri) + "/fragment_info";
+
+  // Get the data
+  Buffer returned_data;
+  RETURN_NOT_OK(curlc.post_data(
+      stats_,
+      url,
+      serialization_type_,
+      &serialized,
+      &returned_data,
+      cache_key));
+  if (returned_data.data() == nullptr || returned_data.size() == 0)
+    return LOG_STATUS(Status_RestError(
+        "Error getting fragment info from REST; server returned no data."));
+
+  // Ensure data has a null delimiter for cap'n proto if using JSON
+  RETURN_NOT_OK(ensure_json_null_delimited_string(&returned_data));
+  return serialization::fragment_info_deserialize(
+      fragment_info, serialization_type_, uri, returned_data);
+}
+
 Status RestClient::post_group_metadata_from_rest(const URI& uri, Group* group) {
   if (group == nullptr) {
     return LOG_STATUS(Status_RestError(
@@ -1231,6 +1338,11 @@ Status RestClient::finalize_query_to_rest(const URI&, Query*) {
       Status_RestError("Cannot use rest client; serialization not enabled."));
 }
 
+Status RestClient::submit_and_finalize_query_to_rest(const URI&, Query*) {
+  return LOG_STATUS(
+      Status_RestError("Cannot use rest client; serialization not enabled."));
+}
+
 Status RestClient::get_query_est_result_sizes(const URI&, Query*) {
   return LOG_STATUS(
       Status_RestError("Cannot use rest client; serialization not enabled."));
@@ -1261,6 +1373,11 @@ Status RestClient::post_group_metadata_from_rest(const URI&, Group*) {
       Status_RestError("Cannot use rest client; serialization not enabled."));
 }
 
+Status RestClient::post_fragment_info_from_rest(const URI&, FragmentInfo*) {
+  return LOG_STATUS(
+      Status_RestError("Cannot use rest client; serialization not enabled."));
+}
+
 Status RestClient::put_group_metadata_to_rest(const URI&, Group*) {
   return LOG_STATUS(
       Status_RestError("Cannot use rest client; serialization not enabled."));
