@@ -67,6 +67,7 @@ namespace sm {
 
 class Array;
 class ArraySchema;
+class DimensionLabelReference;
 class EncryptionKey;
 class FragIdx;
 class FragmentMetadata;
@@ -249,6 +250,53 @@ class Subarray {
   /** equivalent for older Query::set_subarray(const void *subarray); */
   Status set_subarray(const void* subarray);
 
+  /**
+   * Adds a range along the dimension with the given index for the requested
+   * dimension label.
+   *
+   * @param dim_label_ref Cache data about the dimension label definition.
+   * @param range The range to add.
+   * @param read_range_oob_error If ``true`` return error for input range
+   *     larger than the dimension label domain, otherwise return a warning
+   *     and truncate the range.
+   */
+  void add_label_range(
+      const DimensionLabelReference& dim_label_ref,
+      Range&& range,
+      const bool read_range_oob_error = true);
+
+  /**
+   * Adds a range along the dimension with the given index for the requested
+   * dimension label.
+   *
+   * @param label_name The name of the dimension label to add the range to.
+   * @param start The range start.
+   * @param end The range end.
+   * @param stride The range stride.
+   */
+  void add_label_range(
+      const std::string& label_name,
+      const void* start,
+      const void* end,
+      const void* stride);
+
+  /**
+   * Adds a variable-sized range along the dimension with the given index for
+   * the requested dimension label.
+   *
+   * @param label_name The name of the dimension label to add the range to.
+   * @param start The range start.
+   * @param start_size The size of the start data.
+   * @param end The range end.
+   * @param end_size The size of the end data.
+   */
+  void add_label_range_var(
+      const std::string& label_name,
+      const void* start,
+      uint64_t start_size,
+      const void* end,
+      uint64_t end_size);
+
   /** Adds a range along the dimension with the given index. */
   Status add_range(
       uint32_t dim_idx, Range&& range, const bool read_range_oob_error = true);
@@ -272,6 +320,19 @@ class Subarray {
    */
   Status add_point_ranges(unsigned dim_idx, const void* start, uint64_t count);
 
+  /**
+   * @brief Set ranges from an array of ranges (paired { begin,end } )
+   *
+   * @param dim_idx Dimension index
+   * @param start Pointer to start of the array
+   * @param count Number of pairs to add
+   * @return Status
+   * @note The pairs list is logically { {begin1,end1}, {begin2,end2}, ...} but
+   * because of typing considerations from the C api is simply presented as
+   * a linear list of individual items, though they should be multiple of 2
+   */
+  Status add_ranges_list(unsigned dim_idx, const void* start, uint64_t count);
+
   /**
    * Adds a variable-sized range to the (read/write) query on the input
    * dimension by index, in the form of (start, end).
@@ -312,6 +373,73 @@ class Subarray {
       const void* end,
       uint64_t end_size);
 
+  /**
+   * Retrieves reference to attribute ranges.
+   *
+   * @param attr_name Name of the attribute to get ragnes for.
+   */
+  const std::vector<Range>& get_attribute_ranges(
+      const std::string& attr_name) const;
+
+  /**
+   * Retrieves a range from a dimension label name in the form (start, end,
+   * stride).
+   *
+   * @param label_name The name of the dimension label to retrieve the range
+   *     from.
+   * @param range_idx The id of the range to retrieve.
+   * @param start The range start to retrieve.
+   * @param end The range end to retrieve.
+   * @param stride The range stride to retrieve.
+   */
+  void get_label_range(
+      const std::string& label_name,
+      uint64_t range_idx,
+      const void** start,
+      const void** end,
+      const void** stride) const;
+
+  /**
+   * Retrieves the number of ranges of the subarray for the given dimension
+   * label name.
+   *
+   * @param label_name The name of the dimension label to get the number of
+   *     ranges on.
+   * @param range_num The number of ranges set for the dimension label.
+   */
+  void get_label_range_num(
+      const std::string& label_name, uint64_t* range_num) const;
+
+  /**
+   * Retrieves a range's sizes for a variable-length dimension label name
+   *
+   * @param label_name The name of dimension label to retrieve the range sizes
+   *     from.
+   * @param range_idx The id of the range to retrieve.
+   * @param start_size The range start size in bytes.
+   * @param end_size The range end size in bytes.
+   */
+  void get_label_range_var_size(
+      const std::string& label_name,
+      uint64_t range_idx,
+      uint64_t* start_size,
+      uint64_t* end_size) const;
+
+  /**
+   * Retrieves a range from a variable-length dimension label name in the form
+   * (start, end).
+   *
+   * @param label_name The name of dimension label to retrieve the range from.
+   * @param range_idx The id of the range to retrieve.
+   * @param start The range start.
+   * @param end The range end.
+   */
+  void get_label_range_var(
+      const std::string& label_name,
+      uint64_t range_idx,
+      void* start,
+      void* end) const;
+
   /**
    * Retrieves the number of ranges of the subarray for the given dimension
    * name.
@@ -707,7 +835,7 @@ class Subarray {
       ThreadPool* compute_tp);
 
   /** Retrieves the query type of the subarray's array. */
-  Status get_query_type(QueryType* type) const;
+  QueryType get_query_type() const;
 
   /**
    * Returns the range coordinates (for all dimensions) given a flattened
@@ -790,6 +918,21 @@ class Subarray {
     return range_subset_[dim_idx].ranges();
   }
 
+  /**
+   * Adds ranges for an attribute.
+   *
+   * This method is designed to copy label ranges from a parent subarray to
+   * attribute ranges in a dimension label array. The ranges will only be
+   * accessed by the dimension label readers, and it is assumed all checks on
+   * validity of the ranges has already been ran when adding the label ranges to
+   * the parent subarray.
+   *
+   * @param attr_name Name of the attribute to add the ranges for.
+   * @param ranges Ranges to add.
+   */
+  void set_attribute_ranges(
+      const std::string& attr_name, const std::vector<Range>& ranges);
+
   /**
    * Directly sets the `Range` vector for the given dimension index, making
    * a deep copy.
@@ -1026,6 +1169,34 @@ class Subarray {
     uint64_t range_len_;
   };
 
+  /**
+   * Wrapper for optional<tuple<std::string, RangeSetAndSuperset>> for
+   * cleaner data access.
+   */
+  struct LabelRangeSubset {
+   public:
+    /**
+     * Default constructor is not C.41.
+     **/
+    LabelRangeSubset() = delete;
+
+    /**
+     * Constructor
+     *
+     * @param ref Dimension label reference to the label this will contain
+     * ranges for.
+     * @param coalesce_ranges Set if ranges should be combined when adjacent.
+     */
+    LabelRangeSubset(
+        const DimensionLabelReference& ref, bool coalesce_ranges = true);
+
+    /** Name of the dimension label. */
+    std::string name;
+
+    /** The ranges set on the dimension label. */
+    RangeSetAndSuperset ranges;
+  };
+
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
@@ -1067,6 +1238,16 @@ class Subarray {
    */
   std::vector<RangeSetAndSuperset> range_subset_;
 
+  /**
+   * Stores LabelRangeSubset objects for handling ranges on dimension labels.
+   */
+  std::vector<optional<LabelRangeSubset>> label_range_subset_;
+
+  /**
+   * Stores ranges for attributes.
+   */
+  std::unordered_map<std::string, std::vector<Range>> attr_range_subset_;
+
   /**
    * Flag storing if each dimension is a default value or not.
    *
